@@ -27,7 +27,7 @@
 
 /**
  * SECTION:clutter-behaviour-ellipse
- * @short_description: elliptic path behaviour.
+ * @short_description: A behaviour interpolating position along an ellipse
  *
  * #ClutterBehaviourEllipse interpolates actors along a path defined by
  *  an ellipse.
@@ -70,7 +70,7 @@ enum
   PROP_CENTER,
   PROP_WIDTH,
   PROP_HEIGHT,
-  PROP_ANGLE_BEGIN,
+  PROP_ANGLE_START,
   PROP_ANGLE_END,
   PROP_ANGLE_TILT_X,
   PROP_ANGLE_TILT_Y,
@@ -85,7 +85,7 @@ struct _ClutterBehaviourEllipsePrivate
   gint         a;
   gint         b;
 
-  ClutterAngle angle_begin;
+  ClutterAngle angle_start;
   ClutterAngle angle_end;
   ClutterAngle angle_tilt_x;
   ClutterAngle angle_tilt_y;
@@ -109,8 +109,8 @@ clutter_behaviour_ellipse_advance (ClutterBehaviourEllipse *e,
   ClutterBehaviourEllipsePrivate *priv = e->priv;
   gint x, y, z;
 
-  x = CLUTTER_FIXED_INT (priv->a * clutter_cosi (angle));
-  y = CLUTTER_FIXED_INT (priv->b * clutter_sini (angle));
+  x = CLUTTER_FIXED_TO_INT (priv->a * clutter_cosi (angle));
+  y = CLUTTER_FIXED_TO_INT (priv->b * clutter_sini (angle));
   z = 0;
 
   if (priv->angle_tilt_z)
@@ -131,8 +131,8 @@ clutter_behaviour_ellipse_advance (ClutterBehaviourEllipse *e,
       y2 = y * clutter_cosi (priv->angle_tilt_z)
            + x * clutter_sini (priv->angle_tilt_z);
 
-      x = CLUTTER_FIXED_INT (x2);
-      y = CLUTTER_FIXED_INT (y2);
+      x = CLUTTER_FIXED_TO_INT (x2);
+      y = CLUTTER_FIXED_TO_INT (y2);
     }
 
   if (priv->angle_tilt_x)
@@ -143,8 +143,8 @@ clutter_behaviour_ellipse_advance (ClutterBehaviourEllipse *e,
 
       y2 = y * clutter_cosi (priv->angle_tilt_x);
 
-      z = CLUTTER_FIXED_INT (z2);
-      y = CLUTTER_FIXED_INT (y2);
+      z = CLUTTER_FIXED_TO_INT (z2);
+      y = CLUTTER_FIXED_TO_INT (y2);
     }
 
   if (priv->angle_tilt_y)
@@ -157,8 +157,8 @@ clutter_behaviour_ellipse_advance (ClutterBehaviourEllipse *e,
       z2 = z * clutter_cosi (priv->angle_tilt_y)
         + x * clutter_sini (priv->angle_tilt_y);
 
-      x = CLUTTER_FIXED_INT (x2);
-      z = CLUTTER_FIXED_INT (z2);
+      x = CLUTTER_FIXED_TO_INT (x2);
+      z = CLUTTER_FIXED_TO_INT (z2);
     }
 
   knot->x = x;
@@ -183,75 +183,46 @@ actor_apply_knot_foreach (ClutterBehaviour *behave,
   clutter_actor_set_depth (actor, knot->z);
 }
 
+static inline ClutterAngle
+clamp_angle (ClutterAngle a)
+{
+  ClutterAngle a1, a2;
+  gint rounds;
+
+  /* Need to add the 256 offset here, since the user space 0 maps to our
+   * -256
+   */
+  rounds = (a+256) / 1024;
+  a1 = rounds * 1024;
+  a2 = a - a1;
+
+  return a2;
+}
+
 static void
 clutter_behaviour_ellipse_alpha_notify (ClutterBehaviour *behave,
                                         guint32           alpha)
 {
   ClutterBehaviourEllipse *self = CLUTTER_BEHAVIOUR_ELLIPSE (behave);
   ClutterBehaviourEllipsePrivate *priv = self->priv;
+  ClutterAngle start, end;
   knot3d knot;
   ClutterAngle angle = 0;
 
-  if ((priv->angle_end >= priv->angle_begin &&
-       priv->direction == CLUTTER_ROTATE_CW) ||
-      (priv->angle_end < priv->angle_begin &&
-       priv->direction == CLUTTER_ROTATE_CCW))
+  start = priv->angle_start;
+  end   = priv->angle_end;
+
+  if (priv->direction == CLUTTER_ROTATE_CW && start >= end)
     {
-      angle = (priv->angle_end - priv->angle_begin) * alpha
-              / CLUTTER_ALPHA_MAX_ALPHA
-              + priv->angle_begin;
+      end += 1024;
     }
-  else if (priv->angle_end >= priv->angle_begin &&
-           priv->direction == CLUTTER_ROTATE_CCW)
+  else if (priv->direction == CLUTTER_ROTATE_CCW && start <= end)
     {
-      ClutterAngle diff;
-      ClutterAngle angle_begin = priv->angle_begin + 256;
-      ClutterAngle angle_end   = priv->angle_end + 256;
-      
-      /* Work out the angular length of the arch represented by the
-       * end angle in CCW direction
-       */
-      if (angle_end >= 1024)
-        {
-          gint rounds = angle_end / 1024;
-          ClutterAngle a1 = rounds * 1024;
-          ClutterAngle a2 = - (angle_end - a1);
-
-          diff = a1 + a2 + angle_begin;
-        }
-      else
-        {
-          diff = 1024 - angle_end + angle_begin;
-        }
-      
-      angle = priv->angle_begin - (diff * alpha / CLUTTER_ALPHA_MAX_ALPHA);
+      end -= 1024;
     }
-  else if (priv->angle_end < priv->angle_begin &&
-           priv->direction == CLUTTER_ROTATE_CW)
-    {
-      ClutterAngle diff;
-      ClutterAngle angle_begin = priv->angle_begin + 256;
-      ClutterAngle angle_end   = priv->angle_end + 256;
 
-      /* Work out the angular length of the arch represented by the
-       * begin angle in CW direction
-       */
-      if (angle_begin >= 1024)
-        {
-          gint rounds = angle_begin / 1024;
-          ClutterAngle a1 = rounds * 1024;
-          ClutterAngle a2 = - (angle_begin - a1);
+  angle = (end - start) * alpha / CLUTTER_ALPHA_MAX_ALPHA + start;
 
-          diff = a1 + a2 + angle_end;
-        }
-      else
-        {
-          diff = 1024 - angle_begin + angle_end;
-        }
-
-      angle = priv->angle_begin + (diff * alpha / CLUTTER_ALPHA_MAX_ALPHA);
-    }
-  
   clutter_behaviour_ellipse_advance (self, angle, &knot);
 
   knot.x += priv->center.x;
@@ -271,8 +242,8 @@ clutter_behaviour_ellipse_set_property (GObject      *gobject,
 
   switch (prop_id)
     {
-    case PROP_ANGLE_BEGIN:
-      priv->angle_begin =
+    case PROP_ANGLE_START:
+      priv->angle_start =
         CLUTTER_ANGLE_FROM_DEG (g_value_get_double (value)) - 256;
       break;
     case PROP_ANGLE_END:
@@ -281,15 +252,15 @@ clutter_behaviour_ellipse_set_property (GObject      *gobject,
       break;
     case PROP_ANGLE_TILT_X:
       priv->angle_tilt_x =
-        CLUTTER_ANGLE_FROM_DEG (g_value_get_double (value)) - 256;
+        CLUTTER_ANGLE_FROM_DEG (g_value_get_double (value));
       break;
     case PROP_ANGLE_TILT_Y:
       priv->angle_tilt_y =
-        CLUTTER_ANGLE_FROM_DEG (g_value_get_double (value)) - 256;
+        CLUTTER_ANGLE_FROM_DEG (g_value_get_double (value));
       break;
     case PROP_ANGLE_TILT_Z:
       priv->angle_tilt_z =
-        CLUTTER_ANGLE_FROM_DEG (g_value_get_double (value)) - 256;
+        CLUTTER_ANGLE_FROM_DEG (g_value_get_double (value));
       break;
     case PROP_WIDTH:
       priv->a = g_value_get_int (value) >> 1;
@@ -325,9 +296,9 @@ clutter_behaviour_ellipse_get_property (GObject    *gobject,
 
   switch (prop_id)
     {
-    case PROP_ANGLE_BEGIN:
+    case PROP_ANGLE_START:
       g_value_set_double (value,
-                          CLUTTER_ANGLE_TO_DEG (priv->angle_begin + 256));
+                          CLUTTER_ANGLE_TO_DEG (priv->angle_start + 256));
       break;
     case PROP_ANGLE_END:
       g_value_set_double (value,
@@ -335,15 +306,15 @@ clutter_behaviour_ellipse_get_property (GObject    *gobject,
       break;
     case PROP_ANGLE_TILT_X:
       g_value_set_double (value,
-                          CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_x + 256));
+                          CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_x));
       break;
     case PROP_ANGLE_TILT_Y:
       g_value_set_double (value,
-                          CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_y + 256));
+                          CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_y));
       break;
     case PROP_ANGLE_TILT_Z:
       g_value_set_double (value,
-                          CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_z + 256));
+                          CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_z));
       break;
     case PROP_WIDTH:
       g_value_set_int (value, (priv->a << 1));
@@ -370,7 +341,7 @@ clutter_behaviour_ellipse_applied (ClutterBehaviour *behave,
   ClutterBehaviourEllipse *e = CLUTTER_BEHAVIOUR_ELLIPSE (behave);
   knot3d knot;
 
-  clutter_behaviour_ellipse_advance (e, e->priv->angle_begin, &knot);
+  clutter_behaviour_ellipse_advance (e, e->priv->angle_start, &knot);
 
   clutter_actor_set_position (actor, knot.x, knot.y);
   clutter_actor_set_depth (actor, knot.z);
@@ -397,19 +368,19 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
   behave_class->applied = clutter_behaviour_ellipse_applied;
 
   /**
-   * ClutterBehaviourEllipse:angle-begin:
+   * ClutterBehaviourEllipse:angle-start:
    *
-   * The initial angle from where the rotation should begin.
+   * The initial angle from where the rotation should start.
    *
    * Since: 0.4
    */
   g_object_class_install_property (object_class,
-                                   PROP_ANGLE_BEGIN,
-                                   g_param_spec_double ("angle-begin",
-                                                        "Angle Begin",
+                                   PROP_ANGLE_START,
+                                   g_param_spec_double ("angle-start",
+                                                        "Start Angle",
                                                         "Initial angle",
                                                         0.0,
-                                                        CLUTTER_ANGLE_MAX_DEG,
+                                                        360.0,
                                                         0.0,
                                                         CLUTTER_PARAM_READWRITE));
   /**
@@ -422,11 +393,11 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
   g_object_class_install_property (object_class,
                                    PROP_ANGLE_END,
                                    g_param_spec_double ("angle-end",
-                                                        "Angle End",
+                                                        "End Angle",
                                                         "Final angle",
                                                         0.0,
-                                                        CLUTTER_ANGLE_MAX_DEG,
                                                         360.0,
+                                                        0.0,
                                                         CLUTTER_PARAM_READWRITE));
   /**
    * ClutterBehaviourEllipse:angle-tilt-x:
@@ -539,6 +510,13 @@ clutter_behaviour_ellipse_init (ClutterBehaviourEllipse * self)
   self->priv = priv = CLUTTER_BEHAVIOUR_ELLIPSE_GET_PRIVATE (self);
 
   priv->direction = CLUTTER_ROTATE_CW;
+
+  /* The inital values have to reflect the 90 degree offset between the normal
+   * mathematical space and the clutter clock-based space; the default end
+   * value of 360 is clamped to 0.
+   */
+  priv->angle_start = -256;
+  priv->angle_end   = -256;
 }
 
 /**
@@ -549,12 +527,15 @@ clutter_behaviour_ellipse_init (ClutterBehaviourEllipse * self)
  * @width: width of the ellipse
  * @height: height of the ellipse
  * @direction: #ClutterRotateDirection of rotation
- * @begin: angle in degrees at which movement begins
- * @end: angle in degrees at which movement ends
+ * @start: angle in degrees at which movement starts, between 0 and 360
+ * @end: angle in degrees at which movement ends, between 0 and 360
  *
  * Creates a behaviour that drives actors along an elliptical path with
- * given center, width and height; the movement begins at @angle_begin
- * degrees (with 0 corresponding to 12 o'clock) and ends at @angle_end degrees;
+ * given center, width and height; the movement starts at @angle_start
+ * degrees (with 0 corresponding to 12 o'clock) and ends at @angle_end
+ * degrees. Angles >= 360 degrees get clamped to the canonical interval
+ * <0, 360), if start == end, the behaviour will rotate by exacly 360 degrees.
+ *
  * Return value: the newly created #ClutterBehaviourEllipse
  *
  * Since: 0.4
@@ -566,7 +547,7 @@ clutter_behaviour_ellipse_new (ClutterAlpha          *alpha,
                                gint                   width,
                                gint                   height,
                                ClutterRotateDirection direction,
-                               gdouble                begin,
+                               gdouble                start,
                                gdouble                end)
 {
   ClutterKnot center;
@@ -582,7 +563,7 @@ clutter_behaviour_ellipse_new (ClutterAlpha          *alpha,
                        "width", width,
                        "height", height,
                        "direction", direction,
-                       "angle-begin", begin,
+                       "angle-start", start,
                        "angle-end", end,
                        NULL);
 }
@@ -595,7 +576,7 @@ clutter_behaviour_ellipse_new (ClutterAlpha          *alpha,
  * @width: width of the ellipse
  * @height: height of the ellipse
  * @direction: #ClutterRotateDirection of rotation
- * @begin: #ClutterFixed angle in degrees at which movement begins
+ * @start: #ClutterFixed angle in degrees at which movement starts
  * @end: #ClutterFixed angle in degrees at which movement ends
  *
  * Creates a behaviour that drives actors along an elliptical path. This
@@ -612,7 +593,7 @@ clutter_behaviour_ellipse_newx (ClutterAlpha          * alpha,
                                 gint                    width,
                                 gint                    height,
                                 ClutterRotateDirection  direction,
-                                ClutterFixed            begin,
+                                ClutterFixed            start,
                                 ClutterFixed            end)
 {
   ClutterKnot center;
@@ -628,7 +609,7 @@ clutter_behaviour_ellipse_newx (ClutterAlpha          * alpha,
                        "width", width,
                        "height", height,
                        "direction", direction,
-                       "angle-begin", CLUTTER_ANGLE_FROM_DEGX (begin),
+                       "angle-start", CLUTTER_ANGLE_FROM_DEGX (start),
                        "angle-end", CLUTTER_ANGLE_FROM_DEGX (end),
                        NULL);
 }
@@ -784,94 +765,97 @@ clutter_behaviour_ellipse_get_height (ClutterBehaviourEllipse *self)
 }
 
 /**
- * clutter_behaviour_ellipse_set_angle_begin
+ * clutter_behaviour_ellipse_set_angle_start
  * @self: a #ClutterBehaviourEllipse
- * @angle_begin: angle at which movement begins in degrees
+ * @angle_start: angle at which movement starts in degrees, between 0 and 360.
  *
- * Sets the angle at which movement begins.
+ * Sets the angle at which movement starts; angles >= 360 degress get clamped
+ * to the canonical interval <0, 360).
  *
- * Since: 0.4
+ * Since: 0.6
  */
 void
-clutter_behaviour_ellipse_set_angle_begin (ClutterBehaviourEllipse *self,
-                                           gdouble                  angle_begin)
+clutter_behaviour_ellipse_set_angle_start (ClutterBehaviourEllipse *self,
+                                           gdouble                  angle_start)
 {
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self));
 
-  clutter_behaviour_ellipse_set_angle_beginx (self,
-                                              CLUTTER_ANGLE_FROM_DEG (angle_begin));
+  clutter_behaviour_ellipse_set_angle_startx (self,
+                                        CLUTTER_FLOAT_TO_FIXED (angle_start));
 }
 
 /**
- * clutter_behaviour_ellipse_set_angle_beginx
+ * clutter_behaviour_ellipse_set_angle_startx
  * @self: a #ClutterBehaviourEllipse
- * @angle_begin: #ClutterAngle at which movement begins
+ * @angle_start: An angle, as #ClutterFixed, at which movement starts, in
+ * degrees, between 0 and 360.
  *
- * Sets the angle at which movement begins.
+ * Sets the angle at which movement starts; angles >= 360 degress get clamped
+ * to the canonical interval <0, 360).
  *
- * Since: 0.4
+ * Since: 0.6
  */
 void
-clutter_behaviour_ellipse_set_angle_beginx (ClutterBehaviourEllipse *self,
-                                            ClutterAngle             angle_begin)
+clutter_behaviour_ellipse_set_angle_startx (ClutterBehaviourEllipse *self,
+                                            ClutterFixed             angle_start)
 {
   ClutterBehaviourEllipsePrivate *priv;
   ClutterAngle new_angle;
-
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self));
 
-  new_angle = angle_begin - 256;
+  new_angle = clamp_angle (CLUTTER_ANGLE_FROM_DEGX (angle_start) - 256);
 
   priv = self->priv;
-  if (priv->angle_begin != new_angle)
+  if (priv->angle_start != new_angle)
     {
-      priv->angle_begin = new_angle;
-      g_object_notify (G_OBJECT (self), "angle-begin");
+      priv->angle_start = new_angle;
+      g_object_notify (G_OBJECT (self), "angle-start");
     }
 }
 
 /**
- * clutter_behaviour_ellipse_get_angle_begin
+ * clutter_behaviour_ellipse_get_angle_start
  * @self: a #ClutterBehaviourEllipse
  *
- * Gets the angle at which movements begins.
+ * Gets the angle at which movements starts.
  *
  * Return value: angle in degrees
  *
- * Since: 0.4
+ * Since: 0.6
  */
 gdouble
-clutter_behaviour_ellipse_get_angle_begin (ClutterBehaviourEllipse *self)
+clutter_behaviour_ellipse_get_angle_start (ClutterBehaviourEllipse *self)
 {
   g_return_val_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self), 0.0);
 
-  return CLUTTER_ANGLE_TO_DEG (self->priv->angle_begin + 256);
+  return CLUTTER_ANGLE_TO_DEG (self->priv->angle_start + 256);
 }
 
 /**
- * clutter_behaviour_ellipse_get_angle_beginx
+ * clutter_behaviour_ellipse_get_angle_startx
  * @self: a #ClutterBehaviourEllipse
  *
- * Gets the angle at which movements begins.
+ * Gets the angle at which movements starts.
  *
- * Return value: a #ClutterAngle
+ * Return value: angle, as #ClutterFixed, in degrees.
  *
- * Since: 0.4
+ * Since: 0.6
  */
-ClutterAngle
-clutter_behaviour_ellipse_get_angle_beginx (ClutterBehaviourEllipse *self)
+ClutterFixed
+clutter_behaviour_ellipse_get_angle_startx (ClutterBehaviourEllipse *self)
 {
   g_return_val_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self), 0);
 
-  return self->priv->angle_begin;
+  return CLUTTER_ANGLE_TO_DEGX (self->priv->angle_start);
 }
 
 /**
  * clutter_behaviour_ellipse_set_angle_end
  * @self: a #ClutterBehaviourEllipse
- * @angle_end: angle at which movement ends in degrees.
+ * @angle_end: angle at which movement ends in degrees, between 0 and 360.
  *
- * Sets the angle at which movement ends.
+ * Sets the angle at which movement ends; angles >= 360 degress get clamped
+ * to the canonical interval <0, 360).
  *
  * Since: 0.4
  */
@@ -882,28 +866,30 @@ clutter_behaviour_ellipse_set_angle_end (ClutterBehaviourEllipse *self,
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self));
 
   clutter_behaviour_ellipse_set_angle_endx (self,
-                                            CLUTTER_ANGLE_FROM_DEG (angle_end));
+                                          CLUTTER_FLOAT_TO_FIXED (angle_end));
 }
 
 /**
  * clutter_behaviour_ellipse_set_angle_endx
  * @self: a #ClutterBehaviourEllipse
- * @angle_end: #ClutterAngle at which movement ends
+ * @angle_end: angle, as #ClutterFixed, at which movement ends, in degrees,
+ * between 0 and 360.
  *
- * Sets the angle at which movement ends.
+ * Sets the angle at which movement ends; angles >= 360 degress get clamped
+ * to the canonical interval <0, 360).
  *
  * Since: 0.4
  */
 void
 clutter_behaviour_ellipse_set_angle_endx (ClutterBehaviourEllipse *self,
-                                          ClutterAngle             angle_end)
+                                          ClutterFixed             angle_end)
 {
   ClutterBehaviourEllipsePrivate *priv;
   ClutterAngle new_angle;
 
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self));
 
-  new_angle = angle_end - 256;
+  new_angle = clamp_angle (CLUTTER_ANGLE_FROM_DEGX (angle_end) - 256);
 
   priv = self->priv;
 
@@ -939,16 +925,16 @@ clutter_behaviour_ellipse_get_angle_end (ClutterBehaviourEllipse *self)
  *
  * Gets the angle at which movements ends.
  *
- * Return value: a #ClutterAngle
+ * Return value: angle, as #ClutterFixed, in degrees
  *
  * Since: 0.4
  */
-ClutterAngle
+ClutterFixed
 clutter_behaviour_ellipse_get_angle_endx (ClutterBehaviourEllipse *self)
 {
   g_return_val_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self), 0);
 
-  return self->priv->angle_end;
+  return CLUTTER_ANGLE_TO_DEGX (self->priv->angle_end);
 }
 
 /**
@@ -970,16 +956,16 @@ clutter_behaviour_ellipse_set_angle_tilt (ClutterBehaviourEllipse *self,
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self));
 
   clutter_behaviour_ellipse_set_angle_tiltx (self,
-                                             axis,
-                                             CLUTTER_ANGLE_FROM_DEG (angle_tilt));
+                                         axis,
+                                         CLUTTER_FLOAT_TO_FIXED (angle_tilt));
 }
 
 /**
  * clutter_behaviour_ellipse_set_angle_tiltx
  * @self: a #ClutterBehaviourEllipse
  * @axis: a #ClutterRoateAxis
- * @angle_tilt: #ClutterAngle tilt of the elipse around the center in the given
- * axis
+ * @angle_tilt: tilt, as #ClutterFixed, of the elipse around the center in the given
+ * axis in degrees.
  *
  * Sets the angle at which the ellipse should be tilted around it's center.
  *
@@ -988,14 +974,14 @@ clutter_behaviour_ellipse_set_angle_tilt (ClutterBehaviourEllipse *self,
 void
 clutter_behaviour_ellipse_set_angle_tiltx (ClutterBehaviourEllipse *self,
                                            ClutterRotateAxis        axis,
-                                           ClutterAngle             angle_tilt)
+                                           ClutterFixed             angle_tilt)
 {
   ClutterBehaviourEllipsePrivate *priv;
   ClutterAngle new_angle;
 
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self));
 
-  new_angle = angle_tilt - 256;
+  new_angle = CLUTTER_ANGLE_FROM_DEGX (angle_tilt);
 
   priv = self->priv;
 
@@ -1047,8 +1033,17 @@ clutter_behaviour_ellipse_get_angle_tilt (ClutterBehaviourEllipse *self,
 {
   g_return_val_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self), 0.0);
 
-  return CLUTTER_ANGLE_TO_DEG (clutter_behaviour_ellipse_get_angle_tiltx (self,
-                                                                          axis));
+  switch (axis)
+    {
+    case CLUTTER_X_AXIS:
+      return CLUTTER_ANGLE_TO_DEG (self->priv->angle_tilt_x);
+    case CLUTTER_Y_AXIS:
+      return CLUTTER_ANGLE_TO_DEG (self->priv->angle_tilt_y);
+    case CLUTTER_Z_AXIS:
+      return CLUTTER_ANGLE_TO_DEG (self->priv->angle_tilt_z);
+    }
+
+  return 0;
 }
 
 /**
@@ -1058,11 +1053,11 @@ clutter_behaviour_ellipse_get_angle_tilt (ClutterBehaviourEllipse *self,
  *
  * Gets the tilt of the ellipse around the center in the given axis.
  *
- * Return value: a #ClutterAngle
+ * Return value: angle, as #ClutterFixed, in degrees
  *
  * Since: 0.4
  */
-ClutterAngle
+ClutterFixed
 clutter_behaviour_ellipse_get_angle_tiltx (ClutterBehaviourEllipse *self,
                                            ClutterRotateAxis        axis)
 {
@@ -1071,13 +1066,11 @@ clutter_behaviour_ellipse_get_angle_tiltx (ClutterBehaviourEllipse *self,
   switch (axis)
     {
     case CLUTTER_X_AXIS:
-      return self->priv->angle_tilt_x + 256;
+      return CLUTTER_ANGLE_TO_DEGX (self->priv->angle_tilt_x);
     case CLUTTER_Y_AXIS:
-      return self->priv->angle_tilt_y + 256;
+      return CLUTTER_ANGLE_TO_DEGX (self->priv->angle_tilt_y);
     case CLUTTER_Z_AXIS:
-      return self->priv->angle_tilt_z + 256;
-    default:
-      break;
+      return CLUTTER_ANGLE_TO_DEGX (self->priv->angle_tilt_z);
     }
 
   return 0;
@@ -1099,40 +1092,15 @@ clutter_behaviour_ellipse_set_tilt (ClutterBehaviourEllipse *self,
                                     gdouble                  angle_tilt_x,
                                     gdouble                  angle_tilt_y,
                                     gdouble                  angle_tilt_z)
-{
-  g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self));
-
-  clutter_behaviour_ellipse_set_tiltx (self,
-                                       CLUTTER_ANGLE_FROM_DEG (angle_tilt_x),
-                                       CLUTTER_ANGLE_FROM_DEG (angle_tilt_y),
-                                       CLUTTER_ANGLE_FROM_DEG (angle_tilt_z));
-}
-
-/**
- * clutter_behaviour_ellipse_set_tiltx
- * @self: a #ClutterBehaviourEllipse
- * @angle_tilt_x: #ClutterAngle tilt of the elipse around the center in X axis
- * @angle_tilt_y: #ClutterAngle tilt of the elipse around the center in Y axis
- * @angle_tilt_z: #ClutterAngle tilt of the elipse around the center in Z axis
- *
- * Sets the angle at which the ellipse should be tilted around it's center.
- *
- * Since: 0.4
- */
-void
-clutter_behaviour_ellipse_set_tiltx (ClutterBehaviourEllipse *self,
-                                     ClutterAngle             angle_tilt_x,
-                                     ClutterAngle             angle_tilt_y,
-                                     ClutterAngle             angle_tilt_z)
 {
   ClutterBehaviourEllipsePrivate *priv;
   ClutterAngle new_angle_x, new_angle_y, new_angle_z;
 
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self));
 
-  new_angle_x = angle_tilt_x - 256;
-  new_angle_y = angle_tilt_y - 256;
-  new_angle_z = angle_tilt_z - 256;
+  new_angle_x = CLUTTER_ANGLE_FROM_DEG (angle_tilt_x);
+  new_angle_y = CLUTTER_ANGLE_FROM_DEG (angle_tilt_y);
+  new_angle_z = CLUTTER_ANGLE_FROM_DEG (angle_tilt_z);
 
   priv = self->priv;
 
@@ -1158,23 +1126,73 @@ clutter_behaviour_ellipse_set_tiltx (ClutterBehaviourEllipse *self,
 
       g_object_notify (G_OBJECT (self), "angle-tilt-z");
     }
-  
+
+  g_object_thaw_notify (G_OBJECT (self));
+}
+
+/**
+ * clutter_behaviour_ellipse_set_tiltx
+ * @self: a #ClutterBehaviourEllipse
+ * @angle_tilt_x: tilt of the elipse in degrees, as #ClutterFixed, around the center in X axis
+ * @angle_tilt_y: tilt of the elipse in degrees, as #ClutterFixed, around the center in Y axis
+ * @angle_tilt_z: tilt of the elipse in degrees, as #ClutterFixed, around the center in Z axis
+ *
+ * Sets the angle at which the ellipse should be tilted around it's center.
+ *
+ * Since: 0.4
+ */
+void
+clutter_behaviour_ellipse_set_tiltx (ClutterBehaviourEllipse *self,
+                                     ClutterFixed             angle_tilt_x,
+                                     ClutterFixed             angle_tilt_y,
+                                     ClutterFixed             angle_tilt_z)
+{
+  ClutterBehaviourEllipsePrivate *priv;
+  ClutterAngle new_angle_x, new_angle_y, new_angle_z;
+
+  g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ELLIPSE (self));
+
+  new_angle_x = CLUTTER_ANGLE_FROM_DEGX (angle_tilt_x);
+  new_angle_y = CLUTTER_ANGLE_FROM_DEGX (angle_tilt_y);
+  new_angle_z = CLUTTER_ANGLE_FROM_DEGX (angle_tilt_z);
+
+  priv = self->priv;
+
+  g_object_freeze_notify (G_OBJECT (self));
+
+  if (priv->angle_tilt_x != new_angle_x)
+    {
+      priv->angle_tilt_x = new_angle_x;
+
+      g_object_notify (G_OBJECT (self), "angle-tilt-x");
+    }
+
+  if (priv->angle_tilt_y != new_angle_y)
+    {
+      priv->angle_tilt_y = new_angle_y;
+
+      g_object_notify (G_OBJECT (self), "angle-tilt-y");
+    }
+
+  if (priv->angle_tilt_z != new_angle_z)
+    {
+      priv->angle_tilt_z = new_angle_z;
+
+      g_object_notify (G_OBJECT (self), "angle-tilt-z");
+    }
+
   g_object_thaw_notify (G_OBJECT (self));
 }
 
 /**
  * clutter_behaviour_ellipse_get_tilt
  * @self: a #ClutterBehaviourEllipse
- * @angle_tilt_x: location for tilt of the elipse around the center in X axis in
- * degrees, or NULL.
- * @angle_tilt_y: location for tilt of the elipse around the center in Y axis in
- * degrees, or NULL.
- * @angle_tilt_z: location for tilt of the elipse around the center in Z axis in
- * degrees, or NULL.
+ * @angle_tilt_x: return location for tilt angle on the X axis, or %NULL.
+ * @angle_tilt_y: return location for tilt angle on the Y axis, or %NULL.
+ * @angle_tilt_z: return location for tilt angle on the Z axis, or %NULL.
  *
  * Gets the tilt of the ellipse around the center in Y axis.
  *
- *
  * Since: 0.4
  */
 void
@@ -1190,36 +1208,34 @@ clutter_behaviour_ellipse_get_tilt (ClutterBehaviourEllipse *self,
   priv = self->priv;
 
   if (angle_tilt_x)
-    *angle_tilt_x = CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_x + 256);
+    *angle_tilt_x = CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_x);
 
   if (angle_tilt_y)
-    *angle_tilt_y = CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_y + 256);
+    *angle_tilt_y = CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_y);
 
   if (angle_tilt_z)
-    *angle_tilt_z = CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_z + 256);
+    *angle_tilt_z = CLUTTER_ANGLE_TO_DEG (priv->angle_tilt_z);
 }
 
 /**
  * clutter_behaviour_ellipse_get_tiltx
  * @self: a #ClutterBehaviourEllipse
- * @angle_tilt_x: #ClutterAngle location for tilt of the elipse around the
- * center in X axis, or NULL.
- * @angle_tilt_y: #ClutterAngle location for tilt of the elipse around the
- * center in Y axis, or NULL.
- * @angle_tilt_z: #ClutterAngle location for tilt of the elipse around the
- * center in Z axis, or NULL.
+ * @angle_tilt_x: location for tilt of the elipse, as #ClutterFixed, around the
+ * center in X axis in degrees, or NULL.
+ * @angle_tilt_y: location for tilt of the elipse, as #ClutterFixed, around the
+ * center in Y axis in degress, or NULL.
+ * @angle_tilt_z: location for tilt of the elipse, as #ClutterFixed, around the
+ * center in Z axis in degrees, or NULL.
  *
  * Gets the tilt of the ellipse around the center in Y axis.
  *
- * Return value: a #ClutterAngle
- *
  * Since: 0.4
  */
 void
 clutter_behaviour_ellipse_get_tiltx (ClutterBehaviourEllipse *self,
-                                     ClutterAngle            *angle_tilt_x,
-                                     ClutterAngle            *angle_tilt_y,
-                                     ClutterAngle            *angle_tilt_z)
+                                     ClutterFixed            *angle_tilt_x,
+                                     ClutterFixed            *angle_tilt_y,
+                                     ClutterFixed            *angle_tilt_z)
 {
   ClutterBehaviourEllipsePrivate *priv;
 
@@ -1228,13 +1244,13 @@ clutter_behaviour_ellipse_get_tiltx (ClutterBehaviourEllipse *self,
   priv = self->priv;
 
   if (angle_tilt_x)
-    *angle_tilt_x = priv->angle_tilt_x + 256;
+    *angle_tilt_x = CLUTTER_ANGLE_TO_DEGX (priv->angle_tilt_x);
 
   if (angle_tilt_y)
-    *angle_tilt_y = priv->angle_tilt_y + 256;
+    *angle_tilt_y = CLUTTER_ANGLE_TO_DEGX (priv->angle_tilt_y);
 
   if (angle_tilt_z)
-    *angle_tilt_z = priv->angle_tilt_z + 256;
+    *angle_tilt_z = CLUTTER_ANGLE_TO_DEGX (priv->angle_tilt_z);
 }
 
 /**
