@@ -25,8 +25,7 @@
 
 /**
  * SECTION:clutter-behaviour-path
- * @short_description: A behaviour class interpolating actors along a defined
- * path.
+ * @short_description: A behaviour interpolating position along a path
  *
  * #ClutterBehaviourPath interpolates actors along a defined path.
  *
@@ -36,28 +35,42 @@
  * cleared using clutter_behaviour_path_clear().  Each time the behaviour
  * reaches a knot in the path, the "knot-reached" signal is emitted.
  *
- * Since: 0.2
+ * This first knot in the path is reached with the lower bound value
+ * provided by the #ClutterAlpha objectused by the behaviour; the last
+ * knot in the path is reached with the upper bound value provided by
+ * the #ClutterAlpha object used by the behaviour.
+ *
+ * <note>If the alpha function is a periodic function, i.e. it returns to
+ * 0 after reaching %CLUTTER_ALPHA_MAX_ALPHA, then the actors will walk
+ * the path back to the starting #ClutterKnot.</note>
+ *
+ * #ClutterBehaviourPath is available since Clutter 0.2
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include "clutter-actor.h"
 #include "clutter-behaviour.h"
-#include "clutter-marshal.h"
+#include "clutter-behaviour-path.h"
+#include "clutter-debug.h"
 #include "clutter-enum-types.h"
 #include "clutter-main.h"
-#include "clutter-behaviour-path.h"
+#include "clutter-marshal.h"
 #include "clutter-private.h"
-#include "clutter-debug.h"
+#include "clutter-scriptable.h"
+#include "clutter-script-private.h"
 
 #include <math.h>
 
-G_DEFINE_TYPE (ClutterBehaviourPath, 
-               clutter_behaviour_path,
-	       CLUTTER_TYPE_BEHAVIOUR);
+static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (ClutterBehaviourPath,
+                         clutter_behaviour_path,
+	                 CLUTTER_TYPE_BEHAVIOUR,
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                                clutter_scriptable_iface_init));
 
 struct _ClutterBehaviourPathPrivate
 {
@@ -86,7 +99,7 @@ enum
   PROP_KNOT
 };
 
-static void 
+static void
 clutter_behaviour_path_finalize (GObject *object)
 {
   ClutterBehaviourPath *self = CLUTTER_BEHAVIOUR_PATH(object);
@@ -98,31 +111,41 @@ clutter_behaviour_path_finalize (GObject *object)
 }
 
 static inline void
-interpolate (const ClutterKnot *begin, 
-	     const ClutterKnot *end, 
+interpolate (const ClutterKnot *start,
+	     const ClutterKnot *end,
 	     ClutterKnot       *out,
 	     ClutterFixed       t)
 {
-  out->x = begin->x + CLUTTER_FIXED_INT (t * (end->x - begin->x));
-  out->y = begin->y + CLUTTER_FIXED_INT (t * (end->y - begin->y));
+  out->x = start->x + CLUTTER_FIXED_TO_INT (t * (end->x - start->x));
+  out->y = start->y + CLUTTER_FIXED_TO_INT (t * (end->y - start->y));
 }
 
 static gint
-node_distance (const ClutterKnot *begin,
+node_distance (const ClutterKnot *start,
                const ClutterKnot *end)
 {
-  g_return_val_if_fail (begin != NULL, 0);
+  gint t;
+
+  g_return_val_if_fail (start != NULL, 0);
   g_return_val_if_fail (end != NULL, 0);
 
-  if (clutter_knot_equal (begin, end))
+  if (clutter_knot_equal (start, end))
         return 0;
 
-#if 1
-  return clutter_sqrti ((end->x - begin->x) * (end->x - begin->x) +
-			(end->y - begin->y) * (end->y - begin->y));
+  t = (end->x - start->x) * (end->x - start->x) +
+    (end->y - start->y) * (end->y - start->y);
+
+  /*
+   * If we are using limited precision sqrti implementation, fallback on
+   * clib sqrt if the precission would be less than 10%
+   */
+#if INT_MAX > CLUTTER_SQRTI_ARG_10_PERCENT
+  if (t <= CLUTTER_SQRTI_ARG_10_PERCENT)
+    return clutter_sqrti (t);
+  else
+    return CLUTTER_FLOAT_TO_INT(sqrt(t));
 #else
-  return CLUTTER_FLOAT_TO_INT(sqrt((end->x - begin->x) * (end->x - begin->x) +
-				   (end->y - begin->y) * (end->y - begin->y)));
+    return clutter_sqrti (t);
 #endif
 }
 
@@ -158,9 +181,9 @@ path_alpha_to_position (ClutterBehaviourPath *behave,
   ClutterBehaviourPathPrivate *priv = behave->priv;
   ClutterBehaviour *behaviour = CLUTTER_BEHAVIOUR (behave);
   GSList  *l;
-  gint     total_len, offset, dist_to_next, dist = 0;
+  gint     total_len, offset, dist = 0;
 
-  /* FIXME: Optimise. Much of the data used here can be pre-generated  
+  /* FIXME: Optimise. Much of the data used here can be pre-generated
    *        ( total_len, dist between knots ) when knots are added/removed.
   */
 
@@ -175,29 +198,29 @@ path_alpha_to_position (ClutterBehaviourPath *behave,
   total_len = path_total_length (behave);
   offset = (alpha * total_len) / CLUTTER_ALPHA_MAX_ALPHA;
 
-  CLUTTER_NOTE (BEHAVIOUR, "alpha %i vs %i, len: %i vs %i", 
+  CLUTTER_NOTE (BEHAVIOUR, "alpha %i vs %i, len: %i vs %i",
 		alpha, CLUTTER_ALPHA_MAX_ALPHA,
 		offset, total_len);
 
   if (offset == 0)
     {
       /* first knot */
-      clutter_behaviour_actors_foreach (behaviour, 
+      clutter_behaviour_actors_foreach (behaviour,
 					actor_apply_knot_foreach,
 					priv->knots->data);
-      
+
       priv->last_knot_passed = (ClutterKnot*)priv->knots->data;
       g_signal_emit (behave, path_signals[KNOT_REACHED], 0,
                      priv->knots->data);
       return;
     }
-  
+
   if (offset == total_len)
     {
       /* Special case for last knot */
       ClutterKnot *last_knot = (g_slist_last (priv->knots))->data;
 
-      clutter_behaviour_actors_foreach (behaviour, 
+      clutter_behaviour_actors_foreach (behaviour,
 					actor_apply_knot_foreach,
 					last_knot);
 
@@ -209,8 +232,9 @@ path_alpha_to_position (ClutterBehaviourPath *behave,
 
   for (l = priv->knots; l != NULL; l = l->next)
     {
+      gint dist_to_next = 0;
       ClutterKnot *knot = l->data;
-      
+
       if (l->next)
         {
           ClutterKnot *next = l->next->data;
@@ -221,12 +245,12 @@ path_alpha_to_position (ClutterBehaviourPath *behave,
             {
 	      ClutterKnot new;
 	      ClutterFixed t;
-	    
+
 	      t = CLUTTER_INT_TO_FIXED (offset - dist) / dist_to_next;
 
               interpolate (knot, next, &new, t);
 
-              clutter_behaviour_actors_foreach (behaviour, 
+              clutter_behaviour_actors_foreach (behaviour,
 					        actor_apply_knot_foreach,
 					        &new);
 
@@ -316,7 +340,7 @@ clutter_behaviour_path_class_init (ClutterBehaviourPathClass *klass)
                   CLUTTER_TYPE_KNOT);
 
   behave_class->alpha_notify = clutter_behaviour_path_alpha_notify;
-  
+
   g_type_class_add_private (klass, sizeof (ClutterBehaviourPathPrivate));
 }
 
@@ -328,6 +352,81 @@ clutter_behaviour_path_init (ClutterBehaviourPath *self)
   self->priv = priv = CLUTTER_BEHAVIOUR_PATH_GET_PRIVATE (self);
 }
 
+static void
+clutter_behaviour_path_set_custom_property (ClutterScriptable *scriptable,
+                                            ClutterScript     *script,
+                                            const gchar       *name,
+                                            const GValue      *value)
+{
+  if (strcmp (name, "knots") == 0)
+    {
+      ClutterBehaviourPath *path = CLUTTER_BEHAVIOUR_PATH (scriptable);
+      GSList *knots, *l;
+
+      if (!G_VALUE_HOLDS (value, G_TYPE_POINTER))
+        return;
+
+      knots = g_value_get_pointer (value);
+      for (l = knots; l != NULL; l = l->next)
+        {
+          ClutterKnot *knot = l->data;
+
+          clutter_behaviour_path_append_knot (path, knot);
+          clutter_knot_free (knot);
+        }
+
+      g_slist_free (knots);
+    }
+  else
+    g_object_set_property (G_OBJECT (scriptable), name, value);
+}
+
+static gboolean
+clutter_behaviour_path_parse_custom_node (ClutterScriptable *scriptable,
+                                          ClutterScript     *script,
+                                          GValue            *value,
+                                          const gchar       *name,
+                                          JsonNode          *node)
+{
+  if (strcmp (name, "knots") == 0)
+    {
+      JsonArray *array;
+      guint knots_len, i;
+      GSList *knots = NULL;
+
+      array = json_node_get_array (node);
+      knots_len = json_array_get_length (array);
+
+      for (i = 0; i < knots_len; i++)
+        {
+          JsonNode *val = json_array_get_element (array, i);
+          ClutterKnot knot = { 0, };
+
+          if (clutter_script_parse_knot (script, val, &knot))
+            {
+              CLUTTER_NOTE (SCRIPT, "parsed knot [ x:%d, y:%d ]",
+                            knot.x, knot.y);
+
+              knots = g_slist_prepend (knots, clutter_knot_copy (&knot));
+            }
+        }
+
+      g_value_init (value, G_TYPE_POINTER);
+      g_value_set_pointer (value, g_slist_reverse (knots));
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static void
+clutter_scriptable_iface_init (ClutterScriptableIface *iface)
+{
+  iface->parse_custom_node = clutter_behaviour_path_parse_custom_node;
+  iface->set_custom_property = clutter_behaviour_path_set_custom_property;
+}
+
 /**
  * clutter_behaviour_path_new:
  * @alpha: a #ClutterAlpha, or %NULL
@@ -348,8 +447,8 @@ clutter_behaviour_path_new (ClutterAlpha      *alpha,
 {
   ClutterBehaviourPath *behave;
   gint i;
-     
-  behave = g_object_new (CLUTTER_TYPE_BEHAVIOUR_PATH, 
+
+  behave = g_object_new (CLUTTER_TYPE_BEHAVIOUR_PATH,
                          "alpha", alpha,
 			 NULL);
 
@@ -367,7 +466,7 @@ clutter_behaviour_path_new (ClutterAlpha      *alpha,
  * clutter_behaviour_path_get_knots:
  * @pathb: a #ClutterBehvaiourPath
  *
- * Returns a copy of the list of knots contained by @pathb 
+ * Returns a copy of the list of knots contained by @pathb
  *
  * Return value: a #GSList of the paths knots.
  *
@@ -383,7 +482,7 @@ clutter_behaviour_path_get_knots (ClutterBehaviourPath *pathb)
   retval = NULL;
   for (l = pathb->priv->knots; l != NULL; l = l->next)
     retval = g_slist_prepend (retval, l->data);
-  
+
   return g_slist_reverse (retval);
 }
 
@@ -391,7 +490,7 @@ clutter_behaviour_path_get_knots (ClutterBehaviourPath *pathb)
  * clutter_behaviour_path_append_knot:
  * @pathb: a #ClutterBehvaiourPath
  * @knot:  a #ClutterKnot to append.
- *   
+ *
  * Appends a #ClutterKnot to the path
  *
  * Since: 0.2
@@ -412,9 +511,9 @@ clutter_behaviour_path_append_knot (ClutterBehaviourPath  *pathb,
 /**
  * clutter_behaviour_path_insert_knot:
  * @pathb: a #ClutterBehvaiourPath
- * @offset: position in path to insert knot. 
+ * @offset: position in path to insert knot.
  * @knot:  a #ClutterKnot to append.
- *   
+ *
  * Inserts a #ClutterKnot in the path at specified position. Values greater
  * than total number of knots will append the knot at the end of path.
  *
@@ -438,7 +537,7 @@ clutter_behaviour_path_insert_knot (ClutterBehaviourPath  *pathb,
  * clutter_behaviour_path_remove_knot:
  * @pathb: a #ClutterBehvaiourPath
  * @offset: position in path to remove knot.
- *   
+ *
  * Removes a #ClutterKnot in the path at specified offset.
  *
  * Since: 0.2
@@ -485,7 +584,7 @@ clutter_behaviour_path_append_knots_valist (ClutterBehaviourPath *pathb,
  * @Varargs: additional knots to add to the path
  *
  * Adds a NULL-terminated list of knots to a path.  This function is
- * equivalent to calling clutter_behaviour_path_append_knot() for each 
+ * equivalent to calling clutter_behaviour_path_append_knot() for each
  * member of the list.
  *
  * Since: 0.2
@@ -508,7 +607,7 @@ clutter_behaviour_path_append_knots (ClutterBehaviourPath *pathb,
 /**
  * clutter_behaviour_path_clear:
  * @pathb: a #ClutterBehvaiourPath
- *   
+ *
  * Removes all knots from a path
  *
  * Since: 0.2
