@@ -44,12 +44,14 @@
 #include "clutter-private.h"
 #include "clutter-debug.h"
 
-#include "cogl.h"
+#include "cogl/cogl.h"
 
 enum
 {
   PROP_0,
-  PROP_PARENT_TEXTURE
+  PROP_PARENT_TEXTURE,
+  PROP_REPEAT_Y,
+  PROP_REPEAT_X
 };
 
 G_DEFINE_TYPE (ClutterCloneTexture,
@@ -62,112 +64,76 @@ G_DEFINE_TYPE (ClutterCloneTexture,
 struct _ClutterCloneTexturePrivate
 {
   ClutterTexture      *parent_texture;
+  guint                repeat_x : 1;
+  guint                repeat_y : 1;
 };
 
 static void
-clone_texture_render_to_gl_quad (ClutterCloneTexture *ctexture, 
-				 int                  x_1, 
-				 int                  y_1, 
-				 int                  x_2, 
-				 int                  y_2)
+clutter_clone_texture_get_preferred_width (ClutterActor *self,
+                                           ClutterUnit   for_height,
+                                           ClutterUnit  *min_width_p,
+                                           ClutterUnit  *natural_width_p)
 {
-  gint   qx1 = 0, qx2 = 0, qy1 = 0, qy2 = 0;
-  gint   qwidth = 0, qheight = 0;
-  gint   x, y, i = 0, lastx = 0, lasty = 0;
-  gint   n_x_tiles, n_y_tiles; 
-  gint   pwidth, pheight;
-  float tx, ty;
+  ClutterCloneTexturePrivate *priv = CLUTTER_CLONE_TEXTURE (self)->priv;
+  ClutterActor *parent_texture;
+  ClutterActorClass *parent_texture_class;
 
-  ClutterCloneTexturePrivate *priv = ctexture->priv;
-  ClutterActor *parent_actor = CLUTTER_ACTOR (priv->parent_texture);
-
-  priv = ctexture->priv;
-
-  qwidth  = x_2 - x_1;
-  qheight = y_2 - y_1;
-
-  if (!CLUTTER_ACTOR_IS_REALIZED (parent_actor))
-      clutter_actor_realize (parent_actor);
-
-  /* Only paint if parent is in a state to do so */
-  if (!clutter_texture_has_generated_tiles (priv->parent_texture))
-    return;
-  
-  clutter_texture_get_base_size (priv->parent_texture, &pwidth, &pheight); 
-
-  if (!clutter_texture_is_tiled (priv->parent_texture))
+  /* Note that by calling the get_width_request virtual method directly
+   * and skipping the clutter_actor_get_preferred_width() wrapper, we
+   * are ignoring any size request override set on the parent texture
+   * and just getting the normal size of the parent texture.
+   */
+  parent_texture = CLUTTER_ACTOR (priv->parent_texture);
+  if (!parent_texture)
     {
-      clutter_texture_bind_tile (priv->parent_texture, 0);
+      if (min_width_p)
+        *min_width_p = 0;
 
-      /* NPOTS textures *always* used if extension available
-       */
-      if (clutter_feature_available (CLUTTER_FEATURE_TEXTURE_RECTANGLE))
-	{
-	  tx = (float) pwidth;
-	  ty = (float) pheight;
-	}
-      else
-	{
-	  tx = (float) pwidth / clutter_util_next_p2 (pwidth);  
-	  ty = (float) pheight / clutter_util_next_p2 (pheight);
-	}
+      if (natural_width_p)
+        *natural_width_p = 0;
 
-      cogl_texture_quad (x_1, x_2, y_1, y_2, 
-			 0,
-			 0,
-			 CLUTTER_FLOAT_TO_FIXED (tx),
-			 CLUTTER_FLOAT_TO_FIXED (ty));
       return;
     }
 
-  clutter_texture_get_n_tiles (priv->parent_texture, &n_x_tiles, &n_y_tiles); 
+  parent_texture_class = CLUTTER_ACTOR_GET_CLASS (parent_texture);
+  parent_texture_class->get_preferred_width (parent_texture,
+                                             for_height,
+                                             min_width_p,
+                                             natural_width_p);
+}
 
-  for (x = 0; x < n_x_tiles; x++)
+static void
+clutter_clone_texture_get_preferred_height (ClutterActor *self,
+                                            ClutterUnit   for_width,
+                                            ClutterUnit  *min_height_p,
+                                            ClutterUnit  *natural_height_p)
+{
+  ClutterCloneTexturePrivate *priv = CLUTTER_CLONE_TEXTURE (self)->priv;
+  ClutterActor *parent_texture;
+  ClutterActorClass *parent_texture_class;
+
+  /* Note that by calling the get_height_request virtual method directly
+   * and skipping the clutter_actor_get_preferred_height() wrapper, we
+   * are ignoring any size request override set on the parent texture and
+   * just getting the normal size of the parent texture.
+   */
+  parent_texture = CLUTTER_ACTOR (priv->parent_texture);
+  if (!parent_texture)
     {
-      lasty = 0;
+      if (min_height_p)
+        *min_height_p = 0;
 
-      for (y = 0; y < n_y_tiles; y++)
-	{
-	  gint actual_w, actual_h;
-	  gint xpos, ypos, xsize, ysize, ywaste, xwaste;
-	  
-	  clutter_texture_bind_tile (priv->parent_texture, i);
-	 
-	  clutter_texture_get_x_tile_detail (priv->parent_texture, 
-					     x, &xpos, &xsize, &xwaste);
+      if (natural_height_p)
+        *natural_height_p = 0;
 
-	  clutter_texture_get_y_tile_detail (priv->parent_texture, 
-					     y, &ypos, &ysize, &ywaste);
-
-	  actual_w = xsize - xwaste;
-	  actual_h = ysize - ywaste;
-
-	  tx = (float) actual_w / xsize;
-	  ty = (float) actual_h / ysize;
-
-	  qx1 = x_1 + lastx;
-	  qx2 = qx1 + ((qwidth * actual_w ) / pwidth );
-	  
-	  qy1 = y_1 + lasty;
-	  qy2 = qy1 + ((qheight * actual_h) / pheight );
-
-	  CLUTTER_NOTE (TEXTURE,
-                        "rendering text tile x: %i, y: %i - %ix%i",
-			x, y,
-			actual_w, actual_h);
-
-	  cogl_texture_quad (qx1, qx2, qy1, qy2, 
-			     0,
-			     0,
-			     CLUTTER_FLOAT_TO_FIXED (tx),
-			     CLUTTER_FLOAT_TO_FIXED (ty));
-
-	  lasty += qy2 - qy1;	  
-
-	  i++;
-	}
-      lastx += qx2 - qx1;
+      return;
     }
+
+  parent_texture_class = CLUTTER_ACTOR_GET_CLASS (parent_texture);
+  parent_texture_class->get_preferred_height (parent_texture,
+                                              for_width,
+                                              min_height_p,
+                                              natural_height_p);
 }
 
 static void
@@ -176,8 +142,10 @@ clutter_clone_texture_paint (ClutterActor *self)
   ClutterCloneTexturePrivate  *priv;
   ClutterActor                *parent_texture;
   gint                         x_1, y_1, x_2, y_2;
-  GLenum                       target_type;
   ClutterColor                 col = { 0xff, 0xff, 0xff, 0xff };
+  CoglHandle                   cogl_texture;
+  ClutterFixed                 t_w, t_h;
+  guint                        tex_width, tex_height;
 
   priv = CLUTTER_CLONE_TEXTURE (self)->priv;
 
@@ -199,33 +167,40 @@ clutter_clone_texture_paint (ClutterActor *self)
 
   cogl_push_matrix ();
 
-  /* FIXME: figure out nicer way of getting at this info...  
-   */  
-  if (clutter_feature_available (CLUTTER_FEATURE_TEXTURE_RECTANGLE) &&
-      clutter_texture_is_tiled (CLUTTER_TEXTURE (parent_texture)) == FALSE)
-    {
-      target_type = CGL_TEXTURE_RECTANGLE_ARB;
-      cogl_enable (CGL_ENABLE_TEXTURE_RECT|CGL_ENABLE_BLEND);
-    }
-  else
-    {
-      target_type = CGL_TEXTURE_2D;
-      cogl_enable (CGL_ENABLE_TEXTURE_2D|CGL_ENABLE_BLEND);
-    }
-
-  col.alpha = clutter_actor_get_opacity (self);
+  col.alpha = clutter_actor_get_paint_opacity (self);
   cogl_color (&col);
 
-  clutter_actor_get_coords (self, &x_1, &y_1, &x_2, &y_2);
+  clutter_actor_get_allocation_coords (self, &x_1, &y_1, &x_2, &y_2);
 
   CLUTTER_NOTE (PAINT, "paint to x1: %i, y1: %i x2: %i, y2: %i "
 		"opacity: %i",
 		x_1, y_1, x_2, y_2,
 		clutter_actor_get_opacity (self));
 
+  cogl_texture = clutter_texture_get_cogl_texture (priv->parent_texture);
+
+  if (cogl_texture == COGL_INVALID_HANDLE)
+    return;
+
+  tex_width = cogl_texture_get_width (cogl_texture);
+  tex_height = cogl_texture_get_height (cogl_texture);
+
+  if (priv->repeat_x && tex_width > 0)
+    t_w = CFX_QDIV (CLUTTER_INT_TO_FIXED (x_2 - x_1),
+		    CLUTTER_INT_TO_FIXED (tex_width));
+  else
+    t_w = CFX_ONE;
+  if (priv->repeat_y && tex_height > 0)
+    t_h = CFX_QDIV (CLUTTER_INT_TO_FIXED (y_2 - y_1),
+		    CLUTTER_INT_TO_FIXED (tex_height));
+  else
+    t_h = CFX_ONE;
+
   /* Parent paint translated us into position */
-  clone_texture_render_to_gl_quad (CLUTTER_CLONE_TEXTURE (self), 
-				   0, 0, x_2 - x_1, y_2 - y_1);
+  cogl_texture_rectangle (cogl_texture, 0, 0,
+			  CLUTTER_INT_TO_FIXED (x_2 - x_1),
+			  CLUTTER_INT_TO_FIXED (y_2 - y_1),
+			  0, 0, t_w, t_h);
 
   cogl_pop_matrix ();
 }
@@ -249,14 +224,8 @@ set_parent_texture (ClutterCloneTexture *ctexture,
 
   if (texture) 
     {
-      gint width, height;
-
       priv->parent_texture = g_object_ref (texture);
 
-      /* Sync up the size to parent texture base pixbuf size. */
-      clutter_texture_get_base_size (texture, &width, &height);
-      clutter_actor_set_size (actor, width, height);
-
       /* queue a redraw if the cloned texture is already visible */
       if (CLUTTER_ACTOR_IS_VISIBLE (priv->parent_texture) &&
           was_visible)
@@ -264,6 +233,8 @@ set_parent_texture (ClutterCloneTexture *ctexture,
           clutter_actor_show (actor);
           clutter_actor_queue_redraw (actor);
         }
+
+      clutter_actor_queue_relayout (actor);
     }
       
 }
@@ -294,13 +265,30 @@ clutter_clone_texture_set_property (GObject      *object,
 				    const GValue *value,
 				    GParamSpec   *pspec)
 {
-  ClutterCloneTexture *ctexture = CLUTTER_CLONE_TEXTURE (object);
+  ClutterCloneTexture        *ctexture = CLUTTER_CLONE_TEXTURE (object);
+  ClutterCloneTexturePrivate *priv;
+
+  priv = ctexture->priv;
 
   switch (prop_id)
     {
     case PROP_PARENT_TEXTURE:
       set_parent_texture (ctexture, g_value_get_object (value));
       break;
+    case PROP_REPEAT_X:
+      if (priv->repeat_x != g_value_get_boolean (value))
+	{
+	  priv->repeat_x = !priv->repeat_x;
+	  clutter_actor_queue_redraw (CLUTTER_ACTOR (ctexture));
+	}
+      break;
+    case PROP_REPEAT_Y:
+      if (priv->repeat_y != g_value_get_boolean (value))
+	{
+	  priv->repeat_y = !priv->repeat_y;
+	  clutter_actor_queue_redraw (CLUTTER_ACTOR (ctexture));
+	}
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -314,12 +302,21 @@ clutter_clone_texture_get_property (GObject    *object,
 				    GParamSpec *pspec)
 {
   ClutterCloneTexture *ctexture = CLUTTER_CLONE_TEXTURE (object);
+  ClutterCloneTexturePrivate *priv;
+
+  priv = ctexture->priv;
 
   switch (prop_id)
     {
     case PROP_PARENT_TEXTURE:
       g_value_set_object (value, ctexture->priv->parent_texture);
       break;
+    case PROP_REPEAT_X:
+      g_value_set_boolean (value, priv->repeat_x);
+      break;
+    case PROP_REPEAT_Y:
+      g_value_set_boolean (value, priv->repeat_y);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -332,20 +329,43 @@ clutter_clone_texture_class_init (ClutterCloneTextureClass *klass)
   GObjectClass      *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
-  actor_class->paint = clutter_clone_texture_paint;
+  actor_class->paint =
+    clutter_clone_texture_paint;
+  actor_class->get_preferred_width =
+    clutter_clone_texture_get_preferred_width;
+  actor_class->get_preferred_height =
+    clutter_clone_texture_get_preferred_height;
 
   gobject_class->finalize     = clutter_clone_texture_finalize;
   gobject_class->dispose      = clutter_clone_texture_dispose;
   gobject_class->set_property = clutter_clone_texture_set_property;
   gobject_class->get_property = clutter_clone_texture_get_property;
 
-  g_object_class_install_property (gobject_class,
-		  		   PROP_PARENT_TEXTURE,
-				   g_param_spec_object ("parent-texture",
-					   		"Parent Texture",
-							"The parent texture to clone",
-							CLUTTER_TYPE_TEXTURE,
-							CLUTTER_PARAM_READWRITE));
+  g_object_class_install_property
+    (gobject_class, PROP_PARENT_TEXTURE,
+     g_param_spec_object ("parent-texture",
+			  "Parent Texture",
+			  "The parent texture to clone",
+			  CLUTTER_TYPE_TEXTURE,
+			  CLUTTER_PARAM_READWRITE));
+
+  g_object_class_install_property
+    (gobject_class, PROP_REPEAT_X,
+     g_param_spec_boolean ("repeat-x",
+			   "Tile underlying pixbuf in x direction",
+			   "Reapeat underlying pixbuf rather than scale "
+			   "in x direction.",
+			   FALSE,
+			   CLUTTER_PARAM_READWRITE));
+
+  g_object_class_install_property
+    (gobject_class, PROP_REPEAT_Y,
+     g_param_spec_boolean ("repeat-y",
+			   "Tile underlying pixbuf in y direction",
+			   "Reapeat underlying pixbuf rather than scale "
+			   "in y direction.",
+			   FALSE,
+			   CLUTTER_PARAM_READWRITE));
 
   g_type_class_add_private (gobject_class, sizeof (ClutterCloneTexturePrivate));
 }
