@@ -4,7 +4,7 @@
  * An OpenGL based 'interactive canvas' library.
  *
  * Authored By Matthew Allum  <mallum@openedhand.com>
- *             Neil Jagdish Patel <njp@o-hand.com
+ *             Neil Jagdish Patel <njp@o-hand.com>
  *
  * Copyright (C) 2006 OpenedHand
  *
@@ -30,11 +30,6 @@
  *
  * #ClutterEntry is a #ClutterTexture that allows single line text entry.
  *
- * In order to update the contents of the entry with the text inserted
- * by the user you should connect to the ClutterStage::key-press-event and
- * forward the #ClutterKeyEvent received by the #ClutterStage to the
- * #ClutterEntry you want to update, using clutter_entry_handle_key_event().
- *
  * #ClutterEntry is available since Clutter 0.4.
  */
 
@@ -42,6 +37,8 @@
 #include "config.h"
 #endif
 
+#include <cogl/cogl.h>
+
 #include "clutter-entry.h"
 
 #include "clutter-debug.h"
@@ -61,7 +58,6 @@
 G_DEFINE_TYPE (ClutterEntry, clutter_entry, CLUTTER_TYPE_ACTOR);
 
 /* Probably move into main */
-static PangoClutterFontMap  *_font_map = NULL;
 static PangoContext         *_context  = NULL;
 
 enum
@@ -110,7 +106,8 @@ struct _ClutterEntryPrivate
   gint                  extents_height;
 
   gint                  width;
-  gint                  n_chars;
+  gint                  n_chars; /* number of chars */
+  gint                  n_bytes; /* number of bytes */
 
   guint                 alignment        : 2;
   guint                 wrap             : 1;
@@ -131,7 +128,6 @@ struct _ClutterEntryPrivate
   gint                  width_chars;
 
   ClutterGeometry       cursor_pos;
-  ClutterActor         *cursor;
   gboolean              show_cursor;
 };
 
@@ -276,15 +272,31 @@ clutter_entry_ensure_layout (ClutterEntry *entry, gint width)
       pango_layout_set_font_description (priv->layout, priv->desc);
 
       if (priv->text_visible)
-        pango_layout_set_text (priv->layout, priv->text, priv->n_chars);
+        pango_layout_set_text (priv->layout, priv->text, priv->n_bytes);
       else
         {
-          gint len = g_utf8_strlen (priv->text, -1);
-          gchar *invisible = g_strnfill (len, priv->priv_char);
+          GString *str = g_string_sized_new (priv->n_bytes);
+          gunichar invisible_char;
+          gchar buf[7];
+          gint char_len, i;
 
-          pango_layout_set_text (priv->layout, invisible, len);
-          
-          g_free (invisible);
+          if (priv->priv_char != 0)
+            invisible_char = priv->priv_char;
+          else
+            invisible_char = '*';
+
+          /* we need to convert the string built of invisible characters
+           * into UTF-8 for it to be fed to the Pango layout
+           */
+          memset (buf, 0, sizeof (buf));
+          char_len = g_unichar_to_utf8 (invisible_char, buf);
+
+          for (i = 0; i < priv->n_chars; i++)
+            g_string_append_len (str, buf, char_len);
+
+          pango_layout_set_text (priv->layout, str->str, str->len);
+
+          g_string_free (str, TRUE);
         }
 
       if (priv->wrap)
@@ -294,6 +306,9 @@ clutter_entry_ensure_layout (ClutterEntry *entry, gint width)
 	pango_layout_set_width (priv->layout, width * PANGO_SCALE);
       else
 	pango_layout_set_width (priv->layout, -1);
+
+      /* Prime the cache for the layout */
+      pango_clutter_ensure_glyph_cache_for_layout (priv->layout);
     }
 }
 
@@ -343,13 +358,32 @@ clutter_entry_ensure_cursor_position (ClutterEntry *entry)
   ClutterEntryPrivate  *priv;
   gint                  index_;
   PangoRectangle        rect;
+  gint                  priv_char_bytes;
 
   priv = entry->priv;
 
-  if (priv->position == -1)
-    index_ = strlen (priv->text);
+  /* If characters are invisible, get the byte-length of the invisible
+   * character. If priv_char is 0, we use '*', which is ASCII (1 byte).
+   */
+  if (!priv->text_visible && priv->priv_char)
+    priv_char_bytes = g_unichar_to_utf8 (priv->priv_char, NULL);
   else
-    index_ = offset_to_bytes (priv->text, priv->position);
+    priv_char_bytes = 1;
+  
+  if (priv->position == -1)
+    {
+      if (priv->text_visible)
+        index_ = strlen (priv->text);
+      else
+        index_ = priv->n_chars * priv_char_bytes;
+    }
+  else
+    {
+      if (priv->text_visible)
+        index_ = offset_to_bytes (priv->text, priv->position);
+      else
+        index_ = priv->position * priv_char_bytes;
+    }
 
   pango_layout_get_cursor_pos (priv->layout, index_, &rect, NULL);
   priv->cursor_pos.x = rect.x / PANGO_SCALE;
@@ -375,15 +409,15 @@ clutter_entry_paint_cursor (ClutterEntry *entry)
 
   if (priv->show_cursor)
     {
-      clutter_actor_set_size (CLUTTER_ACTOR (priv->cursor),
-                              priv->cursor_pos.width,
-                              priv->cursor_pos.height);
+      cogl_push_matrix ();
 
-      clutter_actor_set_position (priv->cursor,
-                                  priv->cursor_pos.x,
-                                  priv->cursor_pos.y);
+      cogl_color (&priv->fgcol);
+      cogl_rectangle (priv->cursor_pos.x,
+                      priv->cursor_pos.y,
+                      priv->cursor_pos.width,
+                      priv->cursor_pos.height);
 
-      clutter_actor_paint (priv->cursor);
+      cogl_pop_matrix ();
     }
 }
 
@@ -396,6 +430,7 @@ clutter_entry_paint (ClutterActor *self)
   gint                  width, actor_width;
   gint                  text_width;
   gint                  cursor_x;
+  ClutterColor          color = { 0, };
 
   entry  = CLUTTER_ENTRY(self);
   priv   = entry->priv;
@@ -414,9 +449,8 @@ clutter_entry_paint (ClutterActor *self)
   else
     width = priv->width;
 
-  clutter_actor_set_clip (self, 0, 0,
-                          width,
-                          clutter_actor_get_height (self));
+  cogl_clip_set (0, 0, CLUTTER_INT_TO_FIXED (width),
+		 CLUTTER_INT_TO_FIXED (clutter_actor_get_height (self)));
 
   actor_width = width - (2 * priv->entry_padding);
   clutter_entry_ensure_layout (entry, actor_width);
@@ -465,18 +499,23 @@ clutter_entry_paint (ClutterActor *self)
       priv->cursor_pos.x += priv->entry_padding;
     }
 
-  priv->fgcol.alpha = clutter_actor_get_opacity (self);
+  memcpy (&color, &priv->fgcol, sizeof (ClutterColor));
+  color.alpha = clutter_actor_get_paint_opacity (self);
+
   pango_clutter_render_layout (priv->layout,
                                priv->text_x + priv->entry_padding, 0,
-                               &priv->fgcol, 0);
+                               &color, 0);
 
   if (CLUTTER_ENTRY_GET_CLASS (entry)->paint_cursor)
     CLUTTER_ENTRY_GET_CLASS (entry)->paint_cursor (entry);
+
+  cogl_clip_unset ();
 }
 
 static void
-clutter_entry_request_coords (ClutterActor    *self,
-			      ClutterActorBox *box)
+clutter_entry_allocate (ClutterActor          *self,
+                        const ClutterActorBox *box,
+                        gboolean               absolute_origin_changed)
 {
   ClutterEntry *entry = CLUTTER_ENTRY (self);
   ClutterEntryPrivate *priv = entry->priv;
@@ -492,7 +531,96 @@ clutter_entry_request_coords (ClutterActor    *self,
       priv->width = width;
     }
 
-  CLUTTER_ACTOR_CLASS (clutter_entry_parent_class)->request_coords (self, box);
+  CLUTTER_ACTOR_CLASS (clutter_entry_parent_class)->allocate (self, box, absolute_origin_changed);
+}
+
+static inline void
+clutter_entry_handle_key_event_internal (ClutterEntry    *entry,
+                                         ClutterKeyEvent *event)
+{
+  gunichar key_unichar;
+  ClutterEntryPrivate *priv = entry->priv;
+  gint pos = priv->position;
+  gint len = 0;
+  gint keyval = clutter_key_event_symbol (event);
+
+  if (priv->text)
+    len = g_utf8_strlen (priv->text, -1);
+
+  switch (keyval)
+    {
+      case CLUTTER_Return:
+      case CLUTTER_KP_Enter:
+      case CLUTTER_ISO_Enter:
+        g_signal_emit (entry, entry_signals[ACTIVATE], 0);
+        break;
+
+      case CLUTTER_Escape:
+      case CLUTTER_Up:
+      case CLUTTER_KP_Up:
+      case CLUTTER_Down:
+      case CLUTTER_KP_Down:
+      case CLUTTER_Shift_L:
+      case CLUTTER_Shift_R:
+        break;
+
+      case CLUTTER_BackSpace:
+        if (pos != 0 && len != 0)
+          clutter_entry_delete_chars (entry, 1);
+        break;
+
+      case CLUTTER_Delete:
+      case CLUTTER_KP_Delete:
+        if (len && pos != -1)
+          clutter_entry_delete_text (entry, pos, pos+1);;
+        break;
+
+      case CLUTTER_Left:
+      case CLUTTER_KP_Left:
+        if (pos != 0 && len != 0)
+          {
+            if (pos == -1)
+              clutter_entry_set_cursor_position (entry, len - 1);
+            else
+              clutter_entry_set_cursor_position (entry, pos - 1);
+          }
+        break;
+
+      case CLUTTER_Right:
+      case CLUTTER_KP_Right:
+        if (pos != -1 && len != 0)
+          {
+            if (pos != len)
+              clutter_entry_set_cursor_position (entry, pos + 1);
+          }
+        break;
+
+      case CLUTTER_End:
+      case CLUTTER_KP_End:
+        clutter_entry_set_cursor_position (entry, -1);
+        break;
+
+      case CLUTTER_Begin:
+      case CLUTTER_Home:
+      case CLUTTER_KP_Home:
+        clutter_entry_set_cursor_position (entry, 0);
+        break;
+
+      default:
+        key_unichar = clutter_key_event_unicode (event);
+        if (g_unichar_validate (key_unichar))
+          clutter_entry_insert_unichar (entry, key_unichar);
+        break;
+    }
+}
+
+static gboolean
+clutter_entry_key_press (ClutterActor    *actor,
+                         ClutterKeyEvent *event)
+{
+  clutter_entry_handle_key_event_internal (CLUTTER_ENTRY (actor), event);
+
+  return TRUE;
 }
 
 static void
@@ -535,13 +663,14 @@ clutter_entry_finalize (GObject *object)
 static void
 clutter_entry_class_init (ClutterEntryClass *klass)
 {
-  GObjectClass        *gobject_class = G_OBJECT_CLASS (klass);
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
   klass->paint_cursor = clutter_entry_paint_cursor;
 
   actor_class->paint           = clutter_entry_paint;
-  actor_class->request_coords  = clutter_entry_request_coords;
+  actor_class->allocate        = clutter_entry_allocate;
+  actor_class->key_press_event = clutter_entry_key_press;
 
   gobject_class->finalize     = clutter_entry_finalize;
   gobject_class->dispose      = clutter_entry_dispose;
@@ -741,9 +870,7 @@ clutter_entry_class_init (ClutterEntryClass *klass)
    * @entry: the actor which received the event
    *
    * The ::activate signal is emitted each time the entry is 'activated'
-   * by the user, normally by pressing the 'Enter' key. This signal will
-   * only be emitted when you are adding text to the entry via
-   * clutter_entry_handle_key_event().
+   * by the user, normally by pressing the 'Enter' key.
    *
    * Since: 0.4
    */
@@ -768,16 +895,10 @@ clutter_entry_init (ClutterEntry *self)
 
   self->priv = priv = CLUTTER_ENTRY_GET_PRIVATE (self);
 
-  resolution = clutter_backend_get_resolution (clutter_get_default_backend ());
-  if (resolution < 0)
-    resolution = 96.0; /* fall back */
-
   if (G_UNLIKELY (_context == NULL))
-    {
-      _font_map = PANGO_CLUTTER_FONT_MAP (pango_clutter_font_map_new ());
-      pango_clutter_font_map_set_resolution (_font_map, resolution);
-      _context = pango_clutter_font_map_create_context (_font_map);
-    }
+    _context = _clutter_context_create_pango_context (CLUTTER_CONTEXT ());
+
+  resolution = pango_cairo_context_get_resolution (_context);
 
   priv->alignment     = PANGO_ALIGN_LEFT;
   priv->wrap          = FALSE;
@@ -812,9 +933,6 @@ clutter_entry_init (ClutterEntry *self)
               / 72.0;
   clutter_actor_set_size (CLUTTER_ACTOR (self), font_size * 20, 50);
 
-  priv->cursor        = clutter_rectangle_new_with_color (&priv->fgcol);
-  clutter_actor_set_parent (priv->cursor, CLUTTER_ACTOR (self));
-
   priv->show_cursor   = TRUE;
 }
 
@@ -934,7 +1052,10 @@ clutter_entry_set_text (ClutterEntry *entry,
       if (len < priv->max_length)
         {
            g_free (priv->text);
+
            priv->text = g_strdup (text);
+           priv->n_bytes = strlen (text);
+           priv->n_chars = len;
         }
       else
         {
@@ -944,6 +1065,8 @@ clutter_entry_set_text (ClutterEntry *entry,
           g_free (priv->text);
 
           priv->text = n;
+          priv->n_bytes = strlen (n);
+          priv->n_chars = priv->max_length;
         }
     }
   else
@@ -951,11 +1074,14 @@ clutter_entry_set_text (ClutterEntry *entry,
       g_free (priv->text);
 
       priv->text = g_strdup (text);
+      priv->n_bytes = strlen (text);
       priv->n_chars = g_utf8_strlen (priv->text, -1);
     }
 
   clutter_entry_clear_layout (entry);
   clutter_entry_clear_cursor_position (entry);
+  /* Recreate the layout so the glyph cache will be primed */
+  clutter_entry_ensure_layout (entry, -1);
 
   if (CLUTTER_ACTOR_IS_VISIBLE (entry))
     clutter_actor_queue_redraw (CLUTTER_ACTOR (entry));
@@ -1039,6 +1165,8 @@ clutter_entry_set_font_name (ClutterEntry *entry,
   if (entry->priv->text && entry->priv->text[0] != '\0')
     {
       clutter_entry_clear_layout (entry);
+      /* Recreate the layout so the glyph cache will be primed */
+      clutter_entry_ensure_layout (entry, -1);
 
       if (CLUTTER_ACTOR_IS_VISIBLE (entry))
 	clutter_actor_queue_redraw (CLUTTER_ACTOR (entry));
@@ -1081,8 +1209,6 @@ clutter_entry_set_color (ClutterEntry       *entry,
 
   clutter_actor_set_opacity (actor, priv->fgcol.alpha);
 
-  clutter_rectangle_set_color (CLUTTER_RECTANGLE (priv->cursor), &priv->fgcol);
-
   if (CLUTTER_ACTOR_IS_VISIBLE (actor))
     clutter_actor_queue_redraw (actor);
 
@@ -1266,81 +1392,17 @@ clutter_entry_get_cursor_position (ClutterEntry *entry)
  * ClutterStage::key-press-event or ClutterStage::key-release-event.
  *
  * Since: 0.4
+ *
+ * Deprecated: 0.8: The key events will automatically be handled when
+ *   giving the key focus to an entry using clutter_stage_set_key_focus().
  */
 void
 clutter_entry_handle_key_event (ClutterEntry    *entry,
                                 ClutterKeyEvent *kev)
 {
-  ClutterEntryPrivate *priv;
-  gint pos = 0;
-  gint len = 0;
-  gint keyval = clutter_key_event_symbol (kev);
-
   g_return_if_fail (CLUTTER_IS_ENTRY (entry));
 
-  priv = entry->priv;
-
-  pos = priv->position;
-
-  if (priv->text)
-    len = g_utf8_strlen (priv->text, -1);
-
-  switch (keyval)
-    {
-      case CLUTTER_Return:
-      case CLUTTER_KP_Enter:
-      case CLUTTER_ISO_Enter:
-        g_signal_emit (entry, entry_signals[ACTIVATE], 0);
-        break;
-      case CLUTTER_Escape:
-      case CLUTTER_Up:
-      case CLUTTER_KP_Up:
-      case CLUTTER_Down:
-      case CLUTTER_KP_Down:
-      case CLUTTER_Shift_L:
-      case CLUTTER_Shift_R:
-        break;
-      case CLUTTER_BackSpace:
-        if (pos != 0 && len != 0)
-          clutter_entry_delete_chars (entry, 1);
-        break;
-      case CLUTTER_Delete:
-      case CLUTTER_KP_Delete:
-        if (len && pos != -1)
-          clutter_entry_delete_text (entry, pos, pos+1);;
-        break;
-      case CLUTTER_Left:
-      case CLUTTER_KP_Left:
-        if (pos != 0 && len != 0)
-          {
-            if (pos == -1)
-              clutter_entry_set_cursor_position (entry, len - 1);
-            else
-              clutter_entry_set_cursor_position (entry, pos - 1);
-          }
-        break;
-      case CLUTTER_Right:
-      case CLUTTER_KP_Right:
-        if (pos != -1 && len != 0)
-          {
-            if (pos != len)
-              clutter_entry_set_cursor_position (entry, pos + 1);
-          }
-        break;
-      case CLUTTER_End:
-      case CLUTTER_KP_End:
-        clutter_entry_set_cursor_position (entry, -1);
-        break;
-      case CLUTTER_Begin:
-      case CLUTTER_Home:
-      case CLUTTER_KP_Home:
-        clutter_entry_set_cursor_position (entry, 0);
-        break;
-      default:
-        clutter_entry_insert_unichar (entry,
-                                      clutter_keysym_to_unicode (keyval));
-        break;
-    }
+  clutter_entry_handle_key_event_internal (entry, kev);
 }
 
 /**
@@ -1672,8 +1734,8 @@ clutter_entry_get_invisible_char (ClutterEntry *entry)
 /**
  * clutter_entry_set_max_length:
  * @entry: a #ClutterEntry
- * @max: the maximum number of characters allowed in the entry, or -1
- *   to disable
+ * @max: the maximum number of characters allowed in the entry; 0
+ *   to disable or -1 to set the length of the current string
  *
  * Sets the maximum allowed length of the contents of the actor. If the
  * current contents are longer than the given length, then they will be
