@@ -51,6 +51,9 @@ GType clutter_texture_error_get_type (void) G_GNUC_CONST;
 GType clutter_texture_flags_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_TEXTURE_FLAGS (clutter_texture_flags_get_type())
 
+GType clutter_texture_quality_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_TEXTURE_QUALITY (clutter_texture_quality_get_type())
+
 /* enumerations from "./clutter-timeline.h" */
 GType clutter_timeline_direction_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_TIMELINE_DIRECTION (clutter_timeline_direction_get_type())
@@ -65,6 +68,9 @@ GType clutter_rotate_axis_get_type (void) G_GNUC_CONST;
 GType clutter_rotate_direction_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_ROTATE_DIRECTION (clutter_rotate_direction_get_type())
 
+GType clutter_request_mode_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_REQUEST_MODE (clutter_request_mode_get_type())
+
 G_END_DECLS
 
 #endif /* !__CLUTTER_ENUM_TYPES_H__ */
