@@ -27,6 +27,7 @@
 #include "config.h"
 #endif
 
+#include "clutter-keysyms.h"
 #include "clutter-keysyms-table.h"
 #include "clutter-event.h"
 #include "clutter-private.h"
@@ -180,6 +181,25 @@ clutter_event_get_source (ClutterEvent *event)
   return event->any.source;
 }
 
+/**
+ * clutter_event_get_stage:
+ * @event: a #ClutterEvent
+ *
+ * Retrieves the source #ClutterStage the event originated for, or
+ * NULL if the event has no stage.
+ *
+ * Return value: a #ClutterStage
+ *
+ * Since: 0.8
+ */
+ClutterStage*
+clutter_event_get_stage (ClutterEvent *event)
+{
+  g_return_val_if_fail (event != NULL, NULL);
+
+  return event->any.stage;
+}
+
 /**
  * clutter_button_event_button:
  * @buttev: a #ClutterButtonEvent
@@ -244,8 +264,11 @@ guint32
 clutter_key_event_unicode (ClutterKeyEvent *keyev)
 {
   g_return_val_if_fail (keyev != NULL, 0);
- 
-  return clutter_keysym_to_unicode (keyev->keyval);
+
+  if (keyev->unicode_value)
+    return keyev->unicode_value;
+  else
+    return clutter_keysym_to_unicode (keyev->keyval);
 }
 
 /**
@@ -292,6 +315,54 @@ clutter_keysym_to_unicode (guint keyval)
   return 0;
 }
 
+/**
+ * clutter_event_get_device_id:
+ * @event: a clutter event 
+ * 
+ * Retrieves the events device id if set.
+ * 
+ * Return value: A unique identifier for the device or -1 if the event has
+ *               no specific device set.
+ **/
+gint
+clutter_event_get_device_id (ClutterEvent *event)
+{
+  ClutterInputDevice *device = NULL;
+
+  g_return_val_if_fail (event != NULL, -1);
+
+  switch (event->type)
+    {
+    case CLUTTER_NOTHING:
+    case CLUTTER_STAGE_STATE:
+    case CLUTTER_DESTROY_NOTIFY:
+    case CLUTTER_CLIENT_MESSAGE:
+    case CLUTTER_DELETE:
+    case CLUTTER_ENTER:
+    case CLUTTER_LEAVE:
+      break;
+    case CLUTTER_BUTTON_PRESS:
+    case CLUTTER_BUTTON_RELEASE:
+      device = event->button.device;
+      break;
+    case CLUTTER_MOTION:
+      device = event->motion.device;
+      break;
+    case CLUTTER_SCROLL:
+      device = event->scroll.device;
+      break;
+    case CLUTTER_KEY_PRESS:
+    case CLUTTER_KEY_RELEASE:
+      break;
+    }
+
+  if (device)
+    return device->id;
+  else
+    return -1;
+}
+
+
 GType
 clutter_event_get_type (void)
 {
