@@ -18,9 +18,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _HAVE_CLUTTER_EVENT_H
@@ -182,6 +180,13 @@ typedef struct _ClutterScrollEvent      ClutterScrollEvent;
 typedef struct _ClutterStageStateEvent  ClutterStageStateEvent;
 typedef struct _ClutterCrossingEvent    ClutterCrossingEvent;
 
+/**
+ * ClutterInputDevice:
+ *
+ * Generic representation of an input device. The
+ * actual contents of this structure depend on the
+ * backend used.
+ */
 typedef struct _ClutterInputDevice      ClutterInputDevice;
 
 /**
@@ -200,6 +205,7 @@ struct _ClutterAnyEvent
   ClutterEventType  type;
   guint32           time;
   ClutterEventFlags flags;
+  ClutterStage *stage;
   ClutterActor *source;
 };
 
@@ -208,6 +214,7 @@ struct _ClutterAnyEvent
  * @type: event type
  * @time: event time
  * @flags: event flags
+ * @stage: event source stage
  * @source: event source actor
  * @modifier_state: key modifiers
  * @keyval: raw key value
@@ -223,6 +230,7 @@ struct _ClutterKeyEvent
   ClutterEventType type;
   guint32 time;
   ClutterEventFlags flags;
+  ClutterStage *stage;
   ClutterActor *source;
   ClutterModifierType modifier_state;
   guint keyval;
@@ -235,6 +243,7 @@ struct _ClutterKeyEvent
  * @type: event type
  * @time: event time
  * @flags: event flags
+ * @stage: event source stage
  * @source: event source actor
  * @x: event X coordinate
  * @y: event Y coordinate
@@ -254,6 +263,7 @@ struct _ClutterButtonEvent
   ClutterEventType type;
   guint32 time;
   ClutterEventFlags flags;
+  ClutterStage *stage;
   ClutterActor *source;
   gint x;
   gint y;
@@ -264,23 +274,57 @@ struct _ClutterButtonEvent
   ClutterInputDevice *device; /* Future use */
 };
 
+/**
+ * ClutterCrossingEvent:
+ * @type: event type
+ * @time: event time
+ * @flags: event flags
+ * @stage: event source stage
+ * @source: event source actor
+ * @x: event X coordinate
+ * @y: event Y coordinate
+ * @related: actor related to the crossing
+ *
+ * Event for the movement of the pointer across different actors
+ *
+ * Since: 0.2
+ */
 struct _ClutterCrossingEvent
 {
   ClutterEventType type;
   guint32 time;
   ClutterEventFlags flags;
+  ClutterStage *stage;
   ClutterActor *source;
   gint x;
   gint y;
+  ClutterInputDevice *device; /* future use */
   ClutterActor *related;
 };
 
-
+/**
+ * ClutterMotionEvent:
+ * @type: event type
+ * @time: event time
+ * @flags: event flags
+ * @stage: event source stage
+ * @source: event source actor
+ * @x: event X coordinate
+ * @y: event Y coordinate
+ * @modifier_state: button modifiers
+ * @axes: reserved for future use
+ * @device: reserved for future use
+ *
+ * Event for the pointer motion
+ *
+ * Since: 0.2
+ */
 struct _ClutterMotionEvent
 {
   ClutterEventType type;
   guint32 time;
   ClutterEventFlags flags;
+  ClutterStage *stage;
   ClutterActor *source;
   gint x;
   gint y;
@@ -289,11 +333,30 @@ struct _ClutterMotionEvent
   ClutterInputDevice *device; /* Future use */
 };
 
+/**
+ * ClutterScrollEvent:
+ * @type: event type
+ * @time: event time
+ * @flags: event flags
+ * @stage: event source stage
+ * @source: event source actor
+ * @x: event X coordinate
+ * @y: event Y coordinate
+ * @direction: direction of the scrolling
+ * @modifier_state: button modifiers
+ * @axes: reserved for future use
+ * @device: reserved for future use
+ *
+ * Scroll wheel (or similar device) event
+ *
+ * Since: 0.2
+ */
 struct _ClutterScrollEvent
 {
   ClutterEventType type;
   guint32 time;
   ClutterEventFlags flags;
+  ClutterStage *stage;
   ClutterActor *source;
   gint x;
   gint y;
@@ -303,20 +366,43 @@ struct _ClutterScrollEvent
   ClutterInputDevice *device; /* future use */
 };
 
+/**
+ * ClutterStageStateEvent:
+ * @type: event type
+ * @time: event time
+ * @flags: event flags
+ * @stage: event source stage
+ * @source: event source actor (unused)
+ * @changed_mask: bitwise OR of the changed flags
+ * @new_state: bitwise OR of the current state flags
+ *
+ * Event signalling a change in the #ClutterStage state.
+ *
+ * Since: 0.2
+ */
 struct _ClutterStageStateEvent
 {
   ClutterEventType type;
   guint32 time;
   ClutterEventFlags flags;
+  ClutterStage *stage;
   ClutterActor *source; /* unused XXX: should probably be the stage itself */
   ClutterStageState changed_mask;
   ClutterStageState new_state;
 };
 
+/**
+ * ClutterEvent:
+ * @type: event type
+ *
+ * Generic event wrapper.
+ *
+ * Since: 0.2
+ */
 union _ClutterEvent
 {
   ClutterEventType type;
-  
+
   ClutterAnyEvent any;
   ClutterButtonEvent button;
   ClutterKeyEvent key;
@@ -341,6 +427,7 @@ ClutterModifierType clutter_event_get_state     (ClutterEvent       *event);
 void                clutter_event_get_coords    (ClutterEvent       *event,
                                                  gint               *x,
                                                  gint               *y);
+gint                clutter_event_get_device_id (ClutterEvent *event);
 ClutterActor*       clutter_event_get_source    (ClutterEvent       *event);
 
 guint               clutter_key_event_symbol    (ClutterKeyEvent    *keyev);
@@ -351,6 +438,7 @@ guint32             clutter_button_event_button (ClutterButtonEvent *buttev);
 
 guint32             clutter_keysym_to_unicode   (guint               keyval);
 
+ClutterStage*       clutter_event_get_stage     (ClutterEvent *event);
 
 G_END_DECLS
 
