@@ -27,6 +27,10 @@
 #include "config.h"
 #endif
 
+#include <glib-object.h>
+#include <gobject/gvaluecollector.h>
+
+#define G_IMPLEMENTS_INLINES
 #include "clutter-fixed.h"
 #include "clutter-private.h"
 
@@ -47,8 +51,8 @@
  *
  * <itemizedlist>
  *   <listitem>
- *     <para>Two fixed point numbers can be directly added and
- *     subtracted.</para>
+ *     <para>Two fixed point numbers can be directly added,
+ *     subtracted and have their modulus taken.</para>
  *   </listitem>
  *   <listitem>
  *     <para>To add other numerical type to a fixed point number it has to
@@ -148,6 +152,94 @@ static ClutterFixed sin_tbl [] =
  */
 #define CFX_SIN_STEP 0x00000192
 
+/* <private> */
+const double _magic = 68719476736.0 * 1.5;
+
+/* Where in the 64 bits of double is the mantisa */
+#if (__FLOAT_WORD_ORDER == 1234)
+#define _CFX_MAN			0
+#elif (__FLOAT_WORD_ORDER == 4321)
+#define _CFX_MAN			1
+#else
+#define CFX_NO_FAST_CONVERSIONS
+#endif
+
+/*
+ * clutter_double_to_fixed :
+ * @value: value to be converted
+ *
+ * A fast conversion from double precision floating to fixed point
+ *
+ * Return value: Fixed point representation of the value
+ *
+ * Since: 0.2
+ */
+ClutterFixed
+clutter_double_to_fixed (double val)
+{
+#ifdef CFX_NO_FAST_CONVERSIONS
+    return (ClutterFixed)(val * (double)CFX_ONE);
+#else
+    union
+    {
+	double d;
+	unsigned int i[2];
+    } dbl;
+
+    dbl.d = val;
+    dbl.d = dbl.d + _magic;
+    return dbl.i[_CFX_MAN];
+#endif
+}
+
+/*
+ * clutter_double_to_int :
+ * @value: value to be converted
+ *
+ * A fast conversion from doulbe precision floatint point  to int;
+ * used this instead of casting double/float to int.
+ *
+ * Return value: Integer part of the double
+ *
+ * Since: 0.2
+ */
+gint
+clutter_double_to_int (double val)
+{
+#ifdef CFX_NO_FAST_CONVERSIONS
+    return (gint)(val);
+#else
+    union
+    {
+	double d;
+	unsigned int i[2];
+    } dbl;
+
+    dbl.d = val;
+    dbl.d = dbl.d + _magic;
+    return ((int)dbl.i[_CFX_MAN]) >> 16;
+#endif
+}
+
+guint
+clutter_double_to_uint (double val)
+{
+#ifdef CFX_NO_FAST_CONVERSIONS
+    return (guint)(val);
+#else
+    union
+    {
+	double d;
+	unsigned int i[2];
+    } dbl;
+
+    dbl.d = val;
+    dbl.d = dbl.d + _magic;
+    return (dbl.i[_CFX_MAN]) >> 16;
+#endif
+}
+
+#undef _CFX_MAN
 
 /**
  * clutter_sinx:
@@ -173,11 +265,7 @@ clutter_sinx (ClutterFixed angle)
     }
 
     /* reduce to <0, 2*pi) */
-    if (angle >= CFX_2PI)
-    {
-	ClutterFixed f = CLUTTER_FIXED_DIV (angle, CFX_2PI);
-	angle = angle - f;
-    }
+    angle = angle % CFX_2PI;
 
     /* reduce to first quadrant and sign */
     if (angle > CFX_PI)
@@ -414,6 +502,104 @@ clutter_tani (ClutterAngle angle)
     return result;
 }
 
+/* 257-value table of atan. atan_tbl[0] is atan(0.0) and atan_tbl[256]
+   is atan(1). The angles are radians in ClutterFixed
+   truncated to 16-bit (they're all less than one) */
+static guint16 atan_tbl[] =
+  {
+    0x0000, 0x00FF, 0x01FF, 0x02FF, 0x03FF, 0x04FF, 0x05FF, 0x06FF,
+    0x07FF, 0x08FF, 0x09FE, 0x0AFE, 0x0BFD, 0x0CFD, 0x0DFC, 0x0EFB,
+    0x0FFA, 0x10F9, 0x11F8, 0x12F7, 0x13F5, 0x14F3, 0x15F2, 0x16F0,
+    0x17EE, 0x18EB, 0x19E9, 0x1AE6, 0x1BE3, 0x1CE0, 0x1DDD, 0x1ED9,
+    0x1FD5, 0x20D1, 0x21CD, 0x22C8, 0x23C3, 0x24BE, 0x25B9, 0x26B3,
+    0x27AD, 0x28A7, 0x29A1, 0x2A9A, 0x2B93, 0x2C8B, 0x2D83, 0x2E7B,
+    0x2F72, 0x306A, 0x3160, 0x3257, 0x334D, 0x3442, 0x3538, 0x362D,
+    0x3721, 0x3815, 0x3909, 0x39FC, 0x3AEF, 0x3BE2, 0x3CD4, 0x3DC5,
+    0x3EB6, 0x3FA7, 0x4097, 0x4187, 0x4277, 0x4365, 0x4454, 0x4542,
+    0x462F, 0x471C, 0x4809, 0x48F5, 0x49E0, 0x4ACB, 0x4BB6, 0x4CA0,
+    0x4D89, 0x4E72, 0x4F5B, 0x5043, 0x512A, 0x5211, 0x52F7, 0x53DD,
+    0x54C2, 0x55A7, 0x568B, 0x576F, 0x5852, 0x5934, 0x5A16, 0x5AF7,
+    0x5BD8, 0x5CB8, 0x5D98, 0x5E77, 0x5F55, 0x6033, 0x6110, 0x61ED,
+    0x62C9, 0x63A4, 0x647F, 0x6559, 0x6633, 0x670C, 0x67E4, 0x68BC,
+    0x6993, 0x6A6A, 0x6B40, 0x6C15, 0x6CEA, 0x6DBE, 0x6E91, 0x6F64,
+    0x7036, 0x7108, 0x71D9, 0x72A9, 0x7379, 0x7448, 0x7516, 0x75E4,
+    0x76B1, 0x777E, 0x7849, 0x7915, 0x79DF, 0x7AA9, 0x7B72, 0x7C3B,
+    0x7D03, 0x7DCA, 0x7E91, 0x7F57, 0x801C, 0x80E1, 0x81A5, 0x8269,
+    0x832B, 0x83EE, 0x84AF, 0x8570, 0x8630, 0x86F0, 0x87AF, 0x886D,
+    0x892A, 0x89E7, 0x8AA4, 0x8B5F, 0x8C1A, 0x8CD5, 0x8D8E, 0x8E47,
+    0x8F00, 0x8FB8, 0x906F, 0x9125, 0x91DB, 0x9290, 0x9345, 0x93F9,
+    0x94AC, 0x955F, 0x9611, 0x96C2, 0x9773, 0x9823, 0x98D2, 0x9981,
+    0x9A2F, 0x9ADD, 0x9B89, 0x9C36, 0x9CE1, 0x9D8C, 0x9E37, 0x9EE0,
+    0x9F89, 0xA032, 0xA0DA, 0xA181, 0xA228, 0xA2CE, 0xA373, 0xA418,
+    0xA4BC, 0xA560, 0xA602, 0xA6A5, 0xA746, 0xA7E8, 0xA888, 0xA928,
+    0xA9C7, 0xAA66, 0xAB04, 0xABA1, 0xAC3E, 0xACDB, 0xAD76, 0xAE11,
+    0xAEAC, 0xAF46, 0xAFDF, 0xB078, 0xB110, 0xB1A7, 0xB23E, 0xB2D5,
+    0xB36B, 0xB400, 0xB495, 0xB529, 0xB5BC, 0xB64F, 0xB6E2, 0xB773,
+    0xB805, 0xB895, 0xB926, 0xB9B5, 0xBA44, 0xBAD3, 0xBB61, 0xBBEE,
+    0xBC7B, 0xBD07, 0xBD93, 0xBE1E, 0xBEA9, 0xBF33, 0xBFBC, 0xC046,
+    0xC0CE, 0xC156, 0xC1DD, 0xC264, 0xC2EB, 0xC371, 0xC3F6, 0xC47B,
+    0xC4FF, 0xC583, 0xC606, 0xC689, 0xC70B, 0xC78D, 0xC80E, 0xC88F,
+    0xC90F
+  };
+
+/**
+ * clutter_atani:
+ * @x: The tangent to calculate the angle for
+ *
+ * Fast fixed-point version of the arctangent function.
+ *
+ * Return value: The angle in radians represented as a #ClutterFixed
+ * for which the tangent is @x.
+ */
+ClutterFixed
+clutter_atani (ClutterFixed x)
+{
+  gboolean negative = FALSE;
+  ClutterFixed angle;
+
+  if (x < 0)
+    {
+      negative = TRUE;
+      x = -x;
+    }
+
+  if (x > CFX_ONE)
+    /* if x > 1 then atan(x) = pi/2 - atan(1/x) */
+    angle = CFX_PI / 2 - atan_tbl[CFX_QDIV (CFX_ONE, x) >> 8];
+  else
+    angle = atan_tbl[x >> 8];
+
+  return negative ? -angle : angle;
+}
+
+/**
+ * clutter_atan2i:
+ * @y: Numerator of tangent
+ * @x: Denominator of tangent
+ *
+ * Calculates the arctangent of @y / @x but uses the sign of both
+ * arguments to return the angle in right quadrant.
+ *
+ * Return value: The arctangent of @y / @x
+ */
+ClutterFixed
+clutter_atan2i (ClutterFixed y, ClutterFixed x)
+{
+  ClutterFixed angle;
+
+  if (x == 0)
+    angle = y >= 0 ? CFX_PI_2 : -CFX_PI_2;
+  else
+    {
+      angle = clutter_atani (CFX_QDIV (y, x));
+
+      if (x < 0)
+	angle += y >= 0 ? CFX_PI : -CFX_PI;
+    }
+
+  return angle;
+}
+
 ClutterFixed sqrt_tbl [] =
 {
  0x00000000L, 0x00010000L, 0x00016A0AL, 0x0001BB68L,
@@ -702,7 +888,7 @@ clutter_sqrti (gint number)
 }
 
 /**
- * clutter_fixed_qmulx:
+ * clutter_qmulx:
  * @op1: #ClutterFixed
  * @op2: #ClutterFixed
  *
@@ -742,22 +928,23 @@ clutter_qmulx (ClutterFixed op1, ClutterFixed op2)
 }
 
 /**
- * clutter_fixed_qdivx:
+ * clutter_qdivx:
  * @op1: #ClutterFixed
  * @op2: #ClutterFixed
  *
- * Return value: #ClutterFixed.
- *
  * Divides two fixed values using 64bit arithmetic; this provides
  * significantly better precission than the #CLUTTER_FIXED_DIV macro,
  * but at performance cost.
  *
+ * Return value: #ClutterFixed
+ *
  * Since: 0.4
  */
 ClutterFixed
-clutter_qdivx (ClutterFixed op1, ClutterFixed op2)
+clutter_qdivx (ClutterFixed op1,
+               ClutterFixed op2)
 {
-  return (ClutterFixed)((((gint64)op1) << CFX_Q) / op2);
+  return (ClutterFixed) ((((gint64) op1) << CFX_Q) / op2);
 }
 
 /*
@@ -884,93 +1071,299 @@ clutter_powx (guint x, ClutterFixed y)
   return clutter_pow2x (CFX_MUL (y, clutter_log2x (x)));
 }
 
+static GTypeInfo _info = {
+  0,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  0,
+  0,
+  NULL,
+  NULL,
+};
 
-/* <private> */
-const double _magic = 68719476736.0*1.5;
+static GTypeFundamentalInfo _finfo = { 0, };
 
-/* Where in the 64 bits of double is the mantisa */
-#if (__FLOAT_WORD_ORDER == 1234)
-#define _CFX_MAN			0
-#elif (__FLOAT_WORD_ORDER == 4321)
-#define _CFX_MAN			1
-#else
-#define CFX_NO_FAST_CONVERSIONS
-#endif
+static void
+clutter_value_init_fixed (GValue *value)
+{
+  value->data[0].v_int = 0;
+}
 
-/*
- * clutter_double_to_fixed :
- * @value: value to be converted
+static void
+clutter_value_copy_fixed (const GValue *src,
+                          GValue       *dest)
+{
+  dest->data[0].v_int = src->data[0].v_int;
+}
+
+static gchar *
+clutter_value_collect_fixed (GValue      *value,
+                             guint        n_collect_values,
+                             GTypeCValue *collect_values,
+                             guint        collect_flags)
+{
+  value->data[0].v_int = collect_values[0].v_int;
+
+  return NULL;
+}
+
+static gchar *
+clutter_value_lcopy_fixed (const GValue *value,
+                           guint         n_collect_values,
+                           GTypeCValue  *collect_values,
+                           guint         collect_flags)
+{
+  gint32 *fixed_p = collect_values[0].v_pointer;
+
+  if (!fixed_p)
+    return g_strdup_printf ("value location for `%s' passed as NULL",
+                            G_VALUE_TYPE_NAME (value));
+
+  *fixed_p = value->data[0].v_int;
+
+  return NULL;
+}
+
+static void
+clutter_value_transform_fixed_int (const GValue *src,
+                                   GValue       *dest)
+{
+  dest->data[0].v_int = CLUTTER_FIXED_TO_INT (src->data[0].v_int);
+}
+
+static void
+clutter_value_transform_fixed_double (const GValue *src,
+                                      GValue       *dest)
+{
+  dest->data[0].v_double = CLUTTER_FIXED_TO_DOUBLE (src->data[0].v_int);
+}
+
+static void
+clutter_value_transform_fixed_float (const GValue *src,
+                                     GValue       *dest)
+{
+  dest->data[0].v_float = CLUTTER_FIXED_TO_FLOAT (src->data[0].v_int);
+}
+
+static void
+clutter_value_transform_int_fixed (const GValue *src,
+                                   GValue       *dest)
+{
+  dest->data[0].v_int = CLUTTER_INT_TO_FIXED (src->data[0].v_int);
+}
+
+static void
+clutter_value_transform_double_fixed (const GValue *src,
+                                      GValue       *dest)
+{
+  dest->data[0].v_int = CLUTTER_FLOAT_TO_FIXED (src->data[0].v_double);
+}
+
+static void
+clutter_value_transform_float_fixed (const GValue *src,
+                                     GValue       *dest)
+{
+  dest->data[0].v_int = CLUTTER_FLOAT_TO_FIXED (src->data[0].v_float);
+}
+
+
+static const GTypeValueTable _clutter_fixed_value_table = {
+  clutter_value_init_fixed,
+  NULL,
+  clutter_value_copy_fixed,
+  NULL,
+  "i",
+  clutter_value_collect_fixed,
+  "p",
+  clutter_value_lcopy_fixed
+};
+
+GType
+clutter_fixed_get_type (void)
+{
+  static GType _clutter_fixed_type = 0;
+
+  if (G_UNLIKELY (_clutter_fixed_type == 0))
+    {
+      _info.value_table = & _clutter_fixed_value_table;
+      _clutter_fixed_type =
+        g_type_register_fundamental (g_type_fundamental_next (),
+                                     I_("ClutterFixed"),
+                                     &_info, &_finfo, 0);
+
+      g_value_register_transform_func (_clutter_fixed_type, G_TYPE_INT,
+                                       clutter_value_transform_fixed_int);
+      g_value_register_transform_func (G_TYPE_INT, _clutter_fixed_type,
+                                       clutter_value_transform_int_fixed);
+      g_value_register_transform_func (_clutter_fixed_type, G_TYPE_FLOAT,
+                                       clutter_value_transform_fixed_float);
+      g_value_register_transform_func (G_TYPE_FLOAT, _clutter_fixed_type,
+                                       clutter_value_transform_float_fixed);
+      g_value_register_transform_func (_clutter_fixed_type, G_TYPE_DOUBLE,
+                                       clutter_value_transform_fixed_double);
+      g_value_register_transform_func (G_TYPE_DOUBLE, _clutter_fixed_type,
+                                       clutter_value_transform_double_fixed);
+    }
+
+  return _clutter_fixed_type;
+}
+
+/**
+ * clutter_value_set_fixed:
+ * @value: a #GValue initialized to #CLUTTER_TYPE_FIXED
+ * @fixed_: the fixed point value to set
  *
- * A fast conversion from double precision floating to fixed point
+ * Sets @value to @fixed_.
  *
- * Return value: Fixed point representation of the value
+ * Since: 0.8
+ */
+void
+clutter_value_set_fixed (GValue       *value,
+                         ClutterFixed  fixed_)
+{
+  g_return_if_fail (CLUTTER_VALUE_HOLDS_FIXED (value));
+
+  value->data[0].v_int = fixed_;
+}
+
+/**
+ * clutter_value_get_fixed:
+ * @value: a #GValue initialized to #CLUTTER_TYPE_FIXED
  *
- * Since: 0.2
+ * Gets the fixed point value stored inside @value.
+ *
+ * Return value: the value inside the passed #GValue
+ *
+ * Since: 0.8
  */
 ClutterFixed
-_clutter_double_to_fixed (double val)
+clutter_value_get_fixed (const GValue *value)
 {
-#ifdef CFX_NO_FAST_CONVERSIONS
-    return (ClutterFixed)(val * (double)CFX_ONE);
-#else
-    union
-    {
-	double d;
-	unsigned int i[2];
-    } dbl;
+  g_return_val_if_fail (CLUTTER_VALUE_HOLDS_FIXED (value), 0);
 
-    dbl.d = val;
-    dbl.d = dbl.d + _magic;
-    return dbl.i[_CFX_MAN];
-#endif
+  return value->data[0].v_int;
 }
 
-/*
- * clutter_double_to_int :
- * @value: value to be converted
+static void
+param_fixed_init (GParamSpec *pspec)
+{
+  ClutterParamSpecFixed *fspec = CLUTTER_PARAM_SPEC_FIXED (pspec);
+
+  fspec->minimum = CLUTTER_MINFIXED;
+  fspec->maximum = CLUTTER_MAXFIXED;
+  fspec->default_value = 0;
+}
+
+static void
+param_fixed_set_default (GParamSpec *pspec,
+                         GValue     *value)
+{
+  value->data[0].v_int = CLUTTER_PARAM_SPEC_FIXED (pspec)->default_value;
+}
+
+static gboolean
+param_fixed_validate (GParamSpec *pspec,
+                      GValue     *value)
+{
+  ClutterParamSpecFixed *fspec = CLUTTER_PARAM_SPEC_FIXED (pspec);
+  gint oval = CLUTTER_FIXED_TO_INT (value->data[0].v_int);
+  gint min, max, val;
+
+  g_assert (CLUTTER_IS_PARAM_SPEC_FIXED (pspec));
+
+  /* we compare the integer part of the value because the minimum
+   * and maximum values cover just that part of the representation
+   */
+
+  min = fspec->minimum;
+  max = fspec->maximum;
+  val = CLUTTER_FIXED_TO_INT (value->data[0].v_int);
+
+  val = CLAMP (val, min, max);
+  if (val != oval)
+    {
+      value->data[0].v_int = val;
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static gint
+param_fixed_values_cmp (GParamSpec   *pspec,
+                        const GValue *value1,
+                        const GValue *value2)
+{
+  if (value1->data[0].v_int < value2->data[0].v_int)
+    return -1;
+  else
+    return value1->data[0].v_int > value2->data[0].v_int;
+}
+
+GType
+clutter_param_fixed_get_type (void)
+{
+  static GType pspec_type = 0;
+
+  if (G_UNLIKELY (pspec_type == 0))
+    {
+      const GParamSpecTypeInfo pspec_info = {
+        sizeof (ClutterParamSpecFixed),
+        16,
+        param_fixed_init,
+        CLUTTER_TYPE_FIXED,
+        NULL,
+        param_fixed_set_default,
+        param_fixed_validate,
+        param_fixed_values_cmp,
+      };
+
+      pspec_type = g_param_type_register_static (I_("ClutterParamSpecFixed"),
+                                                 &pspec_info);
+    }
+
+  return pspec_type;
+}
+
+/**
+ * clutter_param_spec_fixed:
+ * @name: name of the property
+ * @nick: short name
+ * @blurb: description (can be translatable)
+ * @minimum: lower boundary
+ * @maximum: higher boundary
+ * @default_value: default value
+ * @flags: flags for the param spec
  *
- * A fast conversion from doulbe precision floatint point  to int;
- * used this instead of casting double/float to int.
+ * Creates a #GParamSpec for properties using #ClutterFixed values
  *
- * Return value: Integer part of the double
+ * Return value: the newly created #GParamSpec
  *
- * Since: 0.2
+ * Since: 0.8
  */
-gint
-_clutter_double_to_int (double val)
+GParamSpec *
+clutter_param_spec_fixed (const gchar *name,
+                          const gchar *nick,
+                          const gchar *blurb,
+                          ClutterUnit  minimum,
+                          ClutterUnit  maximum,
+                          ClutterUnit  default_value,
+                          GParamFlags  flags)
 {
-#ifdef CFX_NO_FAST_CONVERSIONS
-    return (gint)(val);
-#else
-    union
-    {
-	double d;
-	unsigned int i[2];
-    } dbl;
+  ClutterParamSpecFixed *fspec;
 
-    dbl.d = val;
-    dbl.d = dbl.d + _magic;
-    return ((int)dbl.i[_CFX_MAN]) >> 16;
-#endif
+  g_return_val_if_fail (default_value >= minimum && default_value <= maximum,
+                        NULL);
+
+  fspec = g_param_spec_internal (CLUTTER_TYPE_PARAM_FIXED,
+                                 name, nick, blurb,
+                                 flags);
+  fspec->minimum = minimum;
+  fspec->maximum = maximum;
+  fspec->default_value = default_value;
+
+  return G_PARAM_SPEC (fspec);
 }
-
-guint
-_clutter_double_to_uint (double val)
-{
-#ifdef CFX_NO_FAST_CONVERSIONS
-    return (guint)(val);
-#else
-    union
-    {
-	double d;
-	unsigned int i[2];
-    } dbl;
-
-    dbl.d = val;
-    dbl.d = dbl.d + _magic;
-    return (dbl.i[_CFX_MAN]) >> 16;
-#endif
-}
-
-#undef _CFX_MAN
-
