@@ -31,6 +31,11 @@
 
 G_BEGIN_DECLS
 
+/**
+ * CLUTTER_INIT_ERROR:
+ *
+ * #GError domain for #ClutterInitError
+ */
 #define CLUTTER_INIT_ERROR      (clutter_init_error_quark ())
 
 /**
@@ -55,6 +60,24 @@ typedef enum {
 
 GQuark clutter_init_error_quark (void);
 
+/**
+ * CLUTTER_PRIORITY_REDRAW:
+ *
+ * Priority of the redraws.
+ *
+ * Since: 0.8
+ */
+#define CLUTTER_PRIORITY_REDRAW         (G_PRIORITY_DEFAULT + 10)
+
+/**
+ * CLUTTER_PRIORITY_TIMELINE:
+ *
+ * Priority of the timelines.
+ *
+ * Since: 0.8
+ */
+#define CLUTTER_PRIORITY_TIMELINE       (G_PRIORITY_DEFAULT + 30)
+
 /* Initialisation */
 void             clutter_base_init        (void);
 ClutterInitError clutter_init             (int          *argc,
@@ -72,7 +95,7 @@ void             clutter_main                       (void);
 void             clutter_main_quit                  (void);
 gint             clutter_main_level                 (void);
 
-void             clutter_redraw                     (void);
+void             clutter_redraw                     (ClutterStage *stage);
 
 void             clutter_do_event                   (ClutterEvent *event);
 
@@ -101,6 +124,15 @@ guint            clutter_threads_add_timeout_full   (gint           priority,
                                                      GSourceFunc    func,
                                                      gpointer       data,
                                                      GDestroyNotify notify);
+guint            clutter_threads_add_frame_source   (guint          interval,
+						     GSourceFunc    func,
+						     gpointer       data);
+guint            clutter_threads_add_frame_source_full
+                                                    (gint           priority,
+						     guint          interval,
+						     GSourceFunc    func,
+						     gpointer       data,
+						     GDestroyNotify notify);
 
 void             clutter_set_motion_events_enabled   (gboolean enable);
 gboolean         clutter_get_motion_events_enabled   (void);
@@ -118,6 +150,18 @@ void             clutter_grab_keyboard               (ClutterActor *actor);
 void             clutter_ungrab_keyboard             (void);
 ClutterActor *   clutter_get_keyboard_grab           (void);
 
+void             clutter_clear_glyph_cache           (void);
+void             clutter_set_use_mipmapped_text      (gboolean      value);
+gboolean         clutter_get_use_mipmapped_text      (void);
+
+ClutterInputDevice*  clutter_get_input_device_for_id (gint id);
+
+void             clutter_grab_pointer_for_device     (ClutterActor  *actor,
+                                                      gint           id);
+
+void             clutter_ungrab_pointer_for_device   (gint id);
+
+
 G_END_DECLS
 
 #endif /* _HAVE_CLUTTER_MAIN_H */
