@@ -437,9 +437,6 @@ clutter_model_resort (ClutterModel *model)
   g_return_if_fail (CLUTTER_IS_MODEL (model));
   priv = model->priv;
 
-  if (!priv->sort_func)
-    return;
-
   klass = CLUTTER_MODEL_GET_CLASS (model);
 
   if (klass->resort)
@@ -739,6 +736,10 @@ clutter_model_appendv (ClutterModel *model,
   g_object_unref (iter);
 }
 
+/* forward declaration */
+static void clutter_model_iter_set_internal_valist (ClutterModelIter *iter,
+                                                    va_list           args);
+
 /**
  * clutter_model_append:
  * @model: a #ClutterModel
@@ -772,8 +773,9 @@ clutter_model_append (ClutterModel *model,
   iter = CLUTTER_MODEL_GET_CLASS (model)->insert_row (model, -1);
   g_assert (CLUTTER_IS_MODEL_ITER (iter));
 
+  /* do not emit the ::row-changed signal */
   va_start (args, model);
-  clutter_model_iter_set_valist (iter, args);
+  clutter_model_iter_set_internal_valist (iter, args);
   va_end (args);
 
   g_signal_emit (model, model_signals[ROW_ADDED], 0, iter);
@@ -862,7 +864,7 @@ clutter_model_prepend (ClutterModel *model,
   g_assert (CLUTTER_IS_MODEL_ITER (iter));
 
   va_start (args, model);
-  clutter_model_iter_set_valist (iter, args);
+  clutter_model_iter_set_internal_valist (iter, args);
   va_end (args);
 
   g_signal_emit (model, model_signals[ROW_ADDED], 0, iter);
@@ -909,7 +911,7 @@ clutter_model_insert (ClutterModel *model,
    * passed columns matches the model sorting column index
    */
   va_start (args, row);
-  clutter_model_iter_set_valist (iter, args);
+  clutter_model_iter_set_internal_valist (iter, args);
   va_end (args);
 
   g_signal_emit (model, model_signals[ROW_ADDED], 0, iter);
@@ -1226,7 +1228,7 @@ clutter_model_set_sorting_column (ClutterModel *model,
 
   priv->sort_column = column;
 
-  if (priv->sort_column > 0)
+  if (priv->sort_column >= 0)
     clutter_model_resort (model);
 
   g_signal_emit (model, model_signals[SORT_CHANGED], 0);
@@ -1476,6 +1478,16 @@ clutter_model_iter_prev_unimplemented (ClutterModelIter *iter)
   return NULL;
 }
 
+static ClutterModelIter *
+clutter_model_iter_copy_unimplemented (ClutterModelIter *iter)
+{
+  g_warning ("%s: Iterator of type `%s' does not implement the "
+             "ClutterModelIter::copy() virtual function",
+             G_STRLOC,
+             g_type_name (G_OBJECT_TYPE (iter)));
+  return NULL;
+}
+
 static void
 clutter_model_iter_get_property (GObject    *object,
                                  guint       prop_id,
@@ -1538,6 +1550,7 @@ clutter_model_iter_class_init (ClutterModelIterClass *klass)
   klass->prev      = clutter_model_iter_prev_unimplemented;
   klass->get_value = clutter_model_iter_get_value_unimplemented;
   klass->set_value = clutter_model_iter_set_value_unimplemented;
+  klass->copy      = clutter_model_iter_copy_unimplemented;
 
   /* Properties */
 
@@ -1591,27 +1604,15 @@ clutter_model_iter_init (ClutterModelIter *self)
  *  Public functions
  */
 
-/**
- * clutter_model_iter_set_valist:
- * @iter: a #ClutterModelIter
- * @args: va_list of column/value pairs, terminiated by -1
- *
- * See clutter_model_iter_set(); this version takes a va_list for language
- * bindings.
- *
- * Since: 0.6
- */
-void 
-clutter_model_iter_set_valist (ClutterModelIter *iter,
-                               va_list           args)
+static void
+clutter_model_iter_set_internal_valist (ClutterModelIter *iter,
+                                        va_list           args)
 {
   ClutterModel *model;
   ClutterModelIterPrivate *priv;
   guint column = 0;
   gboolean sort = FALSE;
   
-  g_return_if_fail (CLUTTER_IS_MODEL_ITER (iter));
-
   priv = iter->priv;
   model = priv->model;
   g_assert (CLUTTER_IS_MODEL (model));
@@ -1658,6 +1659,32 @@ clutter_model_iter_set_valist (ClutterModelIter *iter,
   priv->ignore_sort = FALSE;
   if (sort)
     clutter_model_resort (model);
+}
+
+/**
+ * clutter_model_iter_set_valist:
+ * @iter: a #ClutterModelIter
+ * @args: va_list of column/value pairs, terminiated by -1
+ *
+ * See clutter_model_iter_set(); this version takes a va_list for language
+ * bindings.
+ *
+ * Since: 0.6
+ */
+void 
+clutter_model_iter_set_valist (ClutterModelIter *iter,
+                               va_list           args)
+{
+  ClutterModelIterPrivate *priv;
+  ClutterModel *model;
+
+  g_return_if_fail (CLUTTER_IS_MODEL_ITER (iter));
+
+  clutter_model_iter_set_internal_valist (iter, args);
+
+  priv = iter->priv;
+  model = priv->model;
+  g_assert (CLUTTER_IS_MODEL (model));
 
   g_signal_emit (model, model_signals[ROW_CHANGED], 0, iter);
 }
@@ -1992,3 +2019,27 @@ clutter_model_iter_get_row (ClutterModelIter *iter)
 
   return 0;
 }
+
+/**
+ * clutter_model_iter_copy:
+ * @iter: a #ClutterModelIter
+ *
+ * Copies the passed iterator.
+ *
+ * Return value: a copy of the iterator, or %NULL
+ *
+ * Since: 0.8
+ */
+ClutterModelIter *
+clutter_model_iter_copy (ClutterModelIter *iter)
+{
+  ClutterModelIterClass *klass;
+
+  g_return_val_if_fail (CLUTTER_IS_MODEL_ITER (iter), NULL);
+
+  klass = CLUTTER_MODEL_ITER_GET_CLASS (iter);
+  if (klass->copy)
+    return klass->copy (iter);
+
+  return NULL;
+}
