@@ -38,7 +38,7 @@
 #include "clutter-private.h"
 #include "clutter-debug.h"
 
-#include "cogl.h"
+#include "cogl/cogl.h"
 
 G_DEFINE_TYPE (ClutterRectangle, clutter_rectangle, CLUTTER_TYPE_ACTOR);
 
@@ -84,9 +84,7 @@ clutter_rectangle_paint (ClutterActor *self)
                                               : "unknown");
   cogl_push_matrix();
 
-  cogl_enable (CGL_ENABLE_BLEND);
-
-  clutter_actor_get_geometry (self, &geom);
+  clutter_actor_get_allocation_geometry (self, &geom);
 
   /* parent paint call will have translated us into position so
    * paint from 0, 0
@@ -96,45 +94,51 @@ clutter_rectangle_paint (ClutterActor *self)
       tmp_col.red   = priv->border_color.red;
       tmp_col.green = priv->border_color.green;
       tmp_col.blue  = priv->border_color.blue;
-      tmp_col.alpha = clutter_actor_get_opacity (self);
+      tmp_col.alpha = clutter_actor_get_paint_opacity (self)
+                      * priv->border_color.alpha
+                      / 255;
 
       cogl_color (&tmp_col);
 
       /* this sucks, but it's the only way to make a border */
       cogl_rectangle (priv->border_width, 0,
-		      geom.width - priv->border_width,
-		      priv->border_width);
+                      geom.width - priv->border_width,
+                      priv->border_width);
 
       cogl_rectangle (geom.width - priv->border_width,
-		      priv->border_width,
-		      priv->border_width,
-		      geom.height - priv->border_width);
+                      priv->border_width,
+                      priv->border_width,
+                      geom.height - priv->border_width);
 
       cogl_rectangle (0, geom.height - priv->border_width,
-		      geom.width - priv->border_width,
-		      priv->border_width);
+                      geom.width - priv->border_width,
+                      priv->border_width);
 
       cogl_rectangle (0, 0,
-		      priv->border_width,
-		      geom.height - priv->border_width);
+                      priv->border_width,
+                      geom.height - priv->border_width);
 
       tmp_col.red   = priv->color.red;
       tmp_col.green = priv->color.green;
       tmp_col.blue  = priv->color.blue;
+      tmp_col.alpha = clutter_actor_get_paint_opacity (self)
+                      * priv->color.alpha
+                      / 255;
 
       cogl_color (&tmp_col);
 
-      cogl_rectangle (priv->border_width,
-		      priv->border_width,
-		      geom.width - priv->border_width * 2,
-		      geom.height - priv->border_width * 2);
+      cogl_rectangle (priv->border_width, priv->border_width,
+                      geom.width - priv->border_width * 2,
+                      geom.height - priv->border_width * 2);
     }
   else
     {
       tmp_col.red   = priv->color.red;
       tmp_col.green = priv->color.green;
       tmp_col.blue  = priv->color.blue;
-      tmp_col.alpha = clutter_actor_get_opacity (self);
+      tmp_col.alpha = clutter_actor_get_paint_opacity (self)
+                      * priv->color.alpha
+                      / 255;
 
       cogl_color (&tmp_col);
 
@@ -392,9 +396,6 @@ clutter_rectangle_set_color (ClutterRectangle   *rectangle,
   priv->color.blue = color->blue;
   priv->color.alpha = color->alpha;
 
-  clutter_actor_set_opacity (CLUTTER_ACTOR (rectangle),
-		  	       priv->color.alpha);
-
 #if 0
   /* FIXME - appears to be causing border to always get drawn */
   if (clutter_color_equal (&priv->color, &priv->border_color))
@@ -403,7 +404,7 @@ clutter_rectangle_set_color (ClutterRectangle   *rectangle,
     priv->has_border = TRUE;
 #endif
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR (rectangle)))
+  if (CLUTTER_ACTOR_IS_VISIBLE (rectangle))
     clutter_actor_queue_redraw (CLUTTER_ACTOR (rectangle));
 
   g_object_notify (G_OBJECT (rectangle), "color");
