@@ -38,7 +38,7 @@
  * For example, this code will start two #ClutterTimeline<!-- -->s after
  * a third timeline terminates:
  *
- * <informalexample><programlisting>
+ * |[
  *   ClutterTimeline *timeline_1, *timeline_2, *timeline_3;
  *   ClutterScore *score;
  *
@@ -47,18 +47,23 @@
  *   timeline_3 = clutter_timeline_new_for_duration (500);
  *
  *   score = clutter_score_new ();
- *   clutter_score_append (score, NULL, timeline_1);
+ *
+ *   clutter_score_append (score, NULL,       timeline_1);
  *   clutter_score_append (score, timeline_1, timeline_2);
  *   clutter_score_append (score, timeline_1, timeline_3);
  *
  *   clutter_score_start ();
- * </programlisting></informalexample>
+ * ]|
  *
- * A #ClutterScore takes a reference on the timelines it manages.
+ * A #ClutterScore takes a reference on the timelines it manages,
+ * so timelines can be safely unreferenced after being appended.
  *
- * New timelines can be added to the #ClutterScore using
+ * New timelines can be appended to the #ClutterScore using
  * clutter_score_append() and removed using clutter_score_remove().
  *
+ * Timelines can also be appended to a specific marker on the
+ * parent timeline, using clutter_score_append_at_marker().
+ *
  * The score can be cleared using clutter_score_remove_all().
  *
  * The list of timelines can be retrieved using
@@ -85,11 +90,18 @@ typedef struct _ClutterScoreEntry       ClutterScoreEntry;
 
 struct _ClutterScoreEntry
 {
-  ClutterTimeline *timeline;
-  guint id;
+  /* the entry unique id */
+  gulong id;
 
-  /* signal handler id */
-  gulong completed_id;
+  ClutterTimeline *timeline;
+  ClutterTimeline *parent;
+
+  /* the optional marker on the parent */
+  gchar *marker;
+
+  /* signal handlers id */
+  gulong complete_id;
+  gulong marker_id;
 
   ClutterScore *score;
 
@@ -105,7 +117,7 @@ struct _ClutterScorePrivate
 
   GHashTable *running_timelines;
 
-  guint       last_id;
+  gulong      last_id;
 
   guint       is_paused : 1;
   guint       loop      : 1;
@@ -130,6 +142,8 @@ enum
   LAST_SIGNAL
 };
 
+static inline void clutter_score_clear (ClutterScore *score);
+
 G_DEFINE_TYPE (ClutterScore, clutter_score, G_TYPE_OBJECT);
 
 static int score_signals[LAST_SIGNAL] = { 0 }; 
@@ -177,26 +191,14 @@ clutter_score_get_property (GObject    *gobject,
 static void
 clutter_score_finalize (GObject *object)
 {
-  ClutterScorePrivate *priv = CLUTTER_SCORE (object)->priv;
+  ClutterScore *score = CLUTTER_SCORE (object);
 
-  if (priv->running_timelines)
-    g_hash_table_destroy (priv->running_timelines);
+  clutter_score_stop (score);
+  clutter_score_clear (score);
 
   G_OBJECT_CLASS (clutter_score_parent_class)->finalize (object);
 }
 
-static void
-clutter_score_dispose (GObject *object)
-{
-  ClutterScore *self = CLUTTER_SCORE(object);
-  ClutterScorePrivate *priv;
-
-  priv = self->priv;
-
-  G_OBJECT_CLASS (clutter_score_parent_class)->dispose (object);
-}
-
-
 static void
 clutter_score_class_init (ClutterScoreClass *klass)
 {
@@ -205,7 +207,6 @@ clutter_score_class_init (ClutterScoreClass *klass)
   gobject_class->set_property = clutter_score_set_property;
   gobject_class->get_property = clutter_score_get_property;
   gobject_class->finalize     = clutter_score_finalize;
-  gobject_class->dispose      = clutter_score_dispose;
 
   g_type_class_add_private (klass, sizeof (ClutterScorePrivate));
 
@@ -411,226 +412,40 @@ clutter_score_is_playing (ClutterScore *score)
   return (g_hash_table_size (score->priv->running_timelines) != 0);
 }
 
-/* forward declaration */
-static void start_entry (ClutterScoreEntry *entry);
-
-static void
-start_children_entries (GNode    *node,
-                        gpointer  data)
+/* destroy_entry:
+ * @node: a #GNode
+ *
+ * Frees the #ClutterScoreEntry attached to @node.
+ */
+static gboolean
+destroy_entry (GNode                  *node,
+               G_GNUC_UNUSED gpointer  data)
 {
   ClutterScoreEntry *entry = node->data;
 
-  start_entry (entry);
-}
-
-static void
-on_timeline_finish (ClutterTimeline   *timeline,
-		    ClutterScoreEntry *entry)
-{
-  ClutterScorePrivate *priv = entry->score->priv;
-
-  g_hash_table_remove (priv->running_timelines,
-		       GINT_TO_POINTER (entry->id));
-
-  g_signal_handler_disconnect (timeline, entry->completed_id);
-  entry->completed_id = 0;
-
-  CLUTTER_NOTE (SCHEDULER, "timeline [%p] (%d) completed", 
-		entry->timeline,
-                entry->id);
-
-  g_signal_emit (entry->score, score_signals[TIMELINE_COMPLETED], 0,
-                 entry->timeline);
-
-  /* start every child */
-  if (entry->node->children)
+  if (G_LIKELY (entry != NULL))
     {
-      g_node_children_foreach (entry->node,
-                               G_TRAVERSE_ALL,
-                               start_children_entries,
-                               NULL);
+      if (entry->marker_id)
+        {
+          g_signal_handler_disconnect (entry->parent, entry->marker_id);
+          entry->marker_id = 0;
+        }
+
+      if (entry->complete_id)
+        {
+          g_signal_handler_disconnect (entry->timeline, entry->complete_id);
+          entry->complete_id = 0;
+        }
+
+      g_object_unref (entry->timeline);
+      g_free (entry->marker);
+      g_slice_free (ClutterScoreEntry, entry);
+
+      node->data = NULL;
     }
 
-  /* score has finished - fire 'completed' signal */
-  if (g_hash_table_size (priv->running_timelines) == 0)
-    {
-      CLUTTER_NOTE (SCHEDULER, "looks like we finished");
-      
-      g_signal_emit (entry->score, score_signals[COMPLETED], 0);
-
-      clutter_score_stop (entry->score);
-      
-      if (priv->loop)
-        clutter_score_start (entry->score);
-    }
-}
-
-static void
-start_entry (ClutterScoreEntry *entry)
-{
-  ClutterScorePrivate *priv = entry->score->priv;
-
-  entry->completed_id =
-    g_signal_connect (entry->timeline,
-                      "completed", G_CALLBACK (on_timeline_finish),
-                      entry);
-
-  CLUTTER_NOTE (SCHEDULER, "timeline [%p] (%d) started",
-                entry->timeline,
-                entry->id);
-
-  if (G_UNLIKELY (priv->running_timelines == NULL))
-    priv->running_timelines = g_hash_table_new (NULL, NULL);
-
-  g_hash_table_insert (priv->running_timelines,
-		       GINT_TO_POINTER (entry->id),
-		       entry);
-
-  clutter_timeline_start (entry->timeline);
-
-  g_signal_emit (entry->score, score_signals[TIMELINE_STARTED], 0,
-                 entry->timeline);
-}
-
-static void
-foreach_running_timeline_start (gpointer key,
-                                gpointer value,
-                                gpointer user_data)
-{
-  ClutterScoreEntry *entry = value;
-
-  clutter_timeline_start (entry->timeline);
-}
-
-/**
- * clutter_score_start:
- * @score: A #ClutterScore
- *
- * Starts the score.
- *
- * Since: 0.6
- */
-void
-clutter_score_start (ClutterScore *score)
-{
-  ClutterScorePrivate *priv;
-
-  g_return_if_fail (CLUTTER_IS_SCORE (score));
-
-  priv = score->priv;
-
-  if (priv->is_paused)
-    {
-      g_hash_table_foreach (priv->running_timelines,
-			    foreach_running_timeline_start,
-			    NULL);
-      priv->is_paused = FALSE;
-    }
-  else
-    {
-      g_node_children_foreach (priv->root,
-                               G_TRAVERSE_ALL,
-                               start_children_entries,
-                               NULL);
-    }
-}
-
-static gboolean
-foreach_running_timeline_stop (gpointer key,
-                               gpointer value,
-                               gpointer user_data)
-{
-  ClutterScoreEntry *entry = value;
-
-  clutter_timeline_stop (entry->timeline);
-
-  return TRUE; 
-}
-
-/**
- * clutter_score_stop:
- * @score: A #ClutterScore
- *
- * Stops and rewinds a playing #ClutterScore instance.
- *
- * Since: 0.6
- */
-void
-clutter_score_stop (ClutterScore *score)
-{
-  ClutterScorePrivate *priv;
-
-  g_return_if_fail (CLUTTER_IS_SCORE (score));
-
-  priv = score->priv;
-
-  g_hash_table_foreach_remove (priv->running_timelines,
-			       foreach_running_timeline_stop,
-			       NULL);
-  g_hash_table_destroy (priv->running_timelines);
-  priv->running_timelines = NULL;
-}
-
-/**
- * clutter_score_rewind:
- * @score: A #ClutterScore
- *
- * Rewinds a #ClutterScore to its initial state.
- *
- * Since: 0.6
- */
-void
-clutter_score_rewind (ClutterScore *score)
-{
-  gboolean was_playing;
-
-  g_return_if_fail (CLUTTER_IS_SCORE (score));
-
-  was_playing = clutter_score_is_playing (score);
-
-  clutter_score_stop (score);
-
-  if (was_playing)
-    clutter_score_start (score);
-}
-
-static void
-foreach_running_timeline_pause (gpointer key,
-                                gpointer value,
-                                gpointer user_data)
-{
-  ClutterScoreEntry *entry = value;
-
-  clutter_timeline_pause (entry->timeline);
-}
-
-/**
- * clutter_score_pause:
- * @score: a #ClutterScore
- *
- * Pauses a playing score @score.
- *
- * Since: 0.6
- */
-void
-clutter_score_pause (ClutterScore *score)
-{
-  ClutterScorePrivate *priv;
-
-  g_return_if_fail (CLUTTER_IS_SCORE (score));
-
-  priv = score->priv;
-
-  if (!clutter_score_is_playing (score)) 
-    return;
-
-  g_hash_table_foreach (priv->running_timelines,
-			foreach_running_timeline_pause,
-			NULL);
-
-  priv->is_paused = TRUE;
-
-  g_signal_emit (score, score_signals[PAUSED], 0);
+  /* continue */
+  return FALSE;
 }
 
 typedef enum {
@@ -648,34 +463,13 @@ typedef struct {
   /* parameters */
   union {
     ClutterTimeline *timeline;
-    guint id;
+    gulong id;
     ClutterScoreEntry *entry;
   } d;
 
   gpointer result;
 } TraverseClosure;
 
-static gboolean
-destroy_entry (GNode                  *node,
-               G_GNUC_UNUSED gpointer  data)
-{
-  ClutterScoreEntry *entry = node->data;
-
-  if (G_LIKELY (entry != NULL))
-    {
-      if (entry->completed_id)
-        g_signal_handler_disconnect (entry->timeline, entry->completed_id);
-
-      g_object_unref (entry->timeline);
-      g_slice_free (ClutterScoreEntry, entry);
-
-      node->data = NULL;
-    }
-
-  /* continue */
-  return FALSE;
-}
-
 /* multi-purpose traversal function for the N-ary tree used by the score */
 static gboolean
 traverse_children (GNode    *node,
@@ -710,8 +504,17 @@ traverse_children (GNode    *node,
     case REMOVE_BY_ID:
       if (closure->d.id == entry->id)
         {
-          if (entry->completed_id)
-            g_signal_handler_disconnect (entry->timeline, entry->completed_id);
+          if (entry->complete_id)
+            {
+              g_signal_handler_disconnect (entry->timeline, entry->complete_id);
+              entry->complete_id = 0;
+            }
+
+          if (entry->marker_id)
+            {
+              g_signal_handler_disconnect (entry->timeline, entry->marker_id);
+              entry->marker_id = 0;
+            }
 
           g_object_unref (entry->timeline);
 
@@ -721,6 +524,7 @@ traverse_children (GNode    *node,
                            -1,
                            destroy_entry, NULL);
 
+          g_free (entry->marker);
           g_slice_free (ClutterScoreEntry, entry);
 
           closure->result = node;
@@ -763,7 +567,7 @@ find_entry_by_timeline (ClutterScore    *score,
 
 static GNode *
 find_entry_by_id (ClutterScore *score,
-                  guint         id)
+                  gulong        id)
 {
   ClutterScorePrivate *priv = score->priv;
   TraverseClosure closure;
@@ -785,10 +589,279 @@ find_entry_by_id (ClutterScore *score,
   return NULL;
 }
 
+/* forward declaration */
+static void start_entry (ClutterScoreEntry *entry);
+
+static void
+start_children_entries (GNode    *node,
+                        gpointer  data)
+{
+  ClutterScoreEntry *entry = node->data;
+
+  start_entry (entry);
+}
+
+static void
+on_timeline_marker (ClutterTimeline   *timeline,
+                    const gchar       *marker_name,
+                    gint               frame_num,
+                    ClutterScoreEntry *entry)
+{
+  GNode *parent;
+  CLUTTER_NOTE (SCHEDULER, "timeline [%p] marker ('%s') reached",
+		entry->timeline,
+                entry->marker);
+
+  parent = find_entry_by_timeline (entry->score, timeline);
+  if (!parent)
+    return;
+
+  /* start every child */
+  if (parent->children)
+    {
+      g_node_children_foreach (parent,
+                               G_TRAVERSE_ALL,
+                               start_children_entries,
+                               NULL);
+    }
+}
+
+static void
+on_timeline_completed (ClutterTimeline   *timeline,
+                       ClutterScoreEntry *entry)
+{
+  ClutterScorePrivate *priv = entry->score->priv;
+
+  g_hash_table_remove (priv->running_timelines,
+                       GUINT_TO_POINTER (entry->id));
+
+  g_signal_handler_disconnect (timeline, entry->complete_id);
+  entry->complete_id = 0;
+
+  CLUTTER_NOTE (SCHEDULER, "timeline [%p] ('%lu') completed", 
+		entry->timeline,
+                entry->id);
+
+  g_signal_emit (entry->score, score_signals[TIMELINE_COMPLETED], 0,
+                 entry->timeline);
+
+  /* start every child */
+  if (entry->node->children)
+    {
+      g_node_children_foreach (entry->node,
+                               G_TRAVERSE_ALL,
+                               start_children_entries,
+                               NULL);
+    }
+
+  /* score has finished - fire 'completed' signal */
+  if (g_hash_table_size (priv->running_timelines) == 0)
+    {
+      CLUTTER_NOTE (SCHEDULER, "looks like we finished");
+      
+      g_signal_emit (entry->score, score_signals[COMPLETED], 0);
+
+      clutter_score_stop (entry->score);
+      
+      if (priv->loop)
+        clutter_score_start (entry->score);
+    }
+}
+
+static void
+start_entry (ClutterScoreEntry *entry)
+{
+  ClutterScorePrivate *priv = entry->score->priv;
+
+  /* timelines attached to a marker might already be playing when we
+   * end up here from the ::completed handler, so we need to perform
+   * this check to avoid restarting those timelines
+   */
+  if (clutter_timeline_is_playing (entry->timeline))
+    return;
+
+  entry->complete_id = g_signal_connect (entry->timeline,
+                                         "completed",
+                                         G_CALLBACK (on_timeline_completed),
+                                         entry);
+
+  CLUTTER_NOTE (SCHEDULER, "timeline [%p] ('%lu') started",
+                entry->timeline,
+                entry->id);
+
+  if (G_UNLIKELY (priv->running_timelines == NULL))
+    priv->running_timelines = g_hash_table_new (NULL, NULL);
+
+  g_hash_table_insert (priv->running_timelines,
+                       GUINT_TO_POINTER (entry->id),
+                       entry);
+
+  clutter_timeline_start (entry->timeline);
+
+  g_signal_emit (entry->score, score_signals[TIMELINE_STARTED], 0,
+                 entry->timeline);
+}
+
+enum
+{
+  ACTION_START,
+  ACTION_PAUSE,
+  ACTION_STOP
+};
+
+static void
+foreach_running_timeline (gpointer key,
+                          gpointer value,
+                          gpointer user_data)
+{
+  ClutterScoreEntry *entry = value;
+  gint action = GPOINTER_TO_INT (user_data);
+
+  switch (action)
+    {
+    case ACTION_START:
+      clutter_timeline_start (entry->timeline);
+      break;
+
+    case ACTION_PAUSE:
+      clutter_timeline_pause (entry->timeline);
+      break;
+
+    case ACTION_STOP:
+      clutter_timeline_stop (entry->timeline);
+      break;
+    }
+}
+
+/**
+ * clutter_score_start:
+ * @score: A #ClutterScore
+ *
+ * Starts the score.
+ *
+ * Since: 0.6
+ */
+void
+clutter_score_start (ClutterScore *score)
+{
+  ClutterScorePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_SCORE (score));
+
+  priv = score->priv;
+
+  if (priv->is_paused)
+    {
+      g_hash_table_foreach (priv->running_timelines,
+			    foreach_running_timeline,
+			    GINT_TO_POINTER (ACTION_START));
+      priv->is_paused = FALSE;
+    }
+  else
+    {
+      g_node_children_foreach (priv->root,
+                               G_TRAVERSE_ALL,
+                               start_children_entries,
+                               NULL);
+    }
+}
+
+/**
+ * clutter_score_stop:
+ * @score: A #ClutterScore
+ *
+ * Stops and rewinds a playing #ClutterScore instance.
+ *
+ * Since: 0.6
+ */
+void
+clutter_score_stop (ClutterScore *score)
+{
+  ClutterScorePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_SCORE (score));
+
+  priv = score->priv;
+
+  if (priv->running_timelines)
+    {
+      g_hash_table_foreach (priv->running_timelines,
+                            foreach_running_timeline,
+                            GINT_TO_POINTER (ACTION_STOP));
+      g_hash_table_destroy (priv->running_timelines);
+      priv->running_timelines = NULL;
+    }
+}
+
+/**
+ * clutter_score_pause:
+ * @score: a #ClutterScore
+ *
+ * Pauses a playing score @score.
+ *
+ * Since: 0.6
+ */
+void
+clutter_score_pause (ClutterScore *score)
+{
+  ClutterScorePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_SCORE (score));
+
+  priv = score->priv;
+
+  if (!clutter_score_is_playing (score)) 
+    return;
+
+  g_hash_table_foreach (priv->running_timelines,
+			foreach_running_timeline,
+			GINT_TO_POINTER (ACTION_PAUSE));
+
+  priv->is_paused = TRUE;
+
+  g_signal_emit (score, score_signals[PAUSED], 0);
+}
+
+/**
+ * clutter_score_rewind:
+ * @score: A #ClutterScore
+ *
+ * Rewinds a #ClutterScore to its initial state.
+ *
+ * Since: 0.6
+ */
+void
+clutter_score_rewind (ClutterScore *score)
+{
+  gboolean was_playing;
+
+  g_return_if_fail (CLUTTER_IS_SCORE (score));
+
+  was_playing = clutter_score_is_playing (score);
+
+  clutter_score_stop (score);
+
+  if (was_playing)
+    clutter_score_start (score);
+}
+
+static inline void
+clutter_score_clear (ClutterScore *score)
+{
+  ClutterScorePrivate *priv = score->priv;
+
+  g_node_traverse (priv->root,
+                   G_POST_ORDER,
+                   G_TRAVERSE_ALL,
+                   -1,
+                   destroy_entry, NULL);
+  g_node_destroy (priv->root);
+}
+
 /**
  * clutter_score_append:
  * @score: a #ClutterScore
- * @parent: a #ClutterTimeline in the score or %NULL
+ * @parent: a #ClutterTimeline in the score, or %NULL
  * @timeline: a #ClutterTimeline
  *
  * Appends a timeline to another one existing in the score; the newly
@@ -799,17 +872,19 @@ find_entry_by_id (ClutterScore *score,
  *
  * #ClutterScore will take a reference on @timeline.
  *
- * Return value: the id of the newly added timeline, to be used with
- *   clutter_score_get_timeline() and clutter_score_remove().
+ * Return value: the id of the #ClutterTimeline inside the score, or
+ *   0 on failure. The returned id can be used with clutter_score_remove()
+ *   or clutter_score_get_timeline().
  *
  * Since: 0.6
  */
-guint
+gulong
 clutter_score_append (ClutterScore    *score,
 		      ClutterTimeline *parent,
 		      ClutterTimeline *timeline)
 {
   ClutterScorePrivate *priv;
+  ClutterScoreEntry *entry;
 
   g_return_val_if_fail (CLUTTER_IS_SCORE (score), 0);
   g_return_val_if_fail (parent == NULL || CLUTTER_IS_TIMELINE (parent), 0);
@@ -819,25 +894,20 @@ clutter_score_append (ClutterScore    *score,
 
   if (!parent)
     {
-      ClutterScoreEntry *entry;
-
       entry = g_slice_new (ClutterScoreEntry);
-
       entry->timeline = g_object_ref (timeline);
+      entry->parent = NULL;
       entry->id = priv->last_id;
-      entry->completed_id = 0;
+      entry->marker = NULL;
+      entry->marker_id = 0;
+      entry->complete_id = 0;
       entry->score = score;
 
       entry->node = g_node_append_data (priv->root, entry);
-
-      priv->last_id += 1;
-
-      return entry->id;
     }
   else
     {
       GNode *node;
-      ClutterScoreEntry *entry;
 
       node = find_entry_by_timeline (score, parent);
       if (G_UNLIKELY (!node))
@@ -848,18 +918,93 @@ clutter_score_append (ClutterScore    *score,
 
       entry = g_slice_new (ClutterScoreEntry);
       entry->timeline = g_object_ref (timeline);
+      entry->parent = parent;
       entry->id = priv->last_id;
-      entry->completed_id = 0;
+      entry->marker = NULL;
+      entry->marker_id = 0;
+      entry->complete_id = 0;
       entry->score = score;
 
       entry->node = g_node_append_data (node, entry);
-
-      priv->last_id += 1;
-
-      return entry->id;
     }
 
-  return 0;
+  priv->last_id += 1;
+
+  return entry->id;
+}
+
+/**
+ * clutter_score_append_at_marker:
+ * @score: a #ClutterScore
+ * @parent: the parent #ClutterTimeline
+ * @marker_name: the name of the marker to use
+ * @timeline: the #ClutterTimeline to append
+ *
+ * Appends @timeline at the given @marker_name on the @parent
+ * #ClutterTimeline.
+ *
+ * If you want to append @timeline at the end of @parent, use
+ * clutter_score_append().
+ *
+ * The #ClutterScore will take a reference on @timeline.
+ *
+ * Return value: the id of the #ClutterTimeline inside the score, or
+ *   0 on failure. The returned id can be used with clutter_score_remove()
+ *   or clutter_score_get_timeline().
+ *
+ * Since: 0.8
+ */
+gulong
+clutter_score_append_at_marker (ClutterScore    *score,
+                                ClutterTimeline *parent,
+                                const gchar     *marker_name,
+                                ClutterTimeline *timeline)
+{
+  ClutterScorePrivate *priv;
+  GNode *node;
+  ClutterScoreEntry *entry;
+  gchar *marker_reached_signal;
+
+  g_return_val_if_fail (CLUTTER_IS_SCORE (score), 0);
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (parent), 0);
+  g_return_val_if_fail (marker_name != NULL, 0);
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), 0);
+
+  if (!clutter_timeline_has_marker (parent, marker_name))
+    {
+      g_warning ("The parent timeline has no marker `%s'", marker_name);
+      return 0;
+    }
+
+  priv = score->priv;
+
+  node = find_entry_by_timeline (score, parent);
+  if (G_UNLIKELY (!node))
+    {
+      g_warning ("Unable to find the parent timeline inside the score.");
+      return 0;
+    }
+
+  entry = g_slice_new (ClutterScoreEntry);
+  entry->timeline = g_object_ref (timeline);
+  entry->parent = parent;
+  entry->marker = g_strdup (marker_name);
+  entry->id = priv->last_id;
+  entry->score = score;
+
+  marker_reached_signal = g_strdup_printf ("marker-reached::%s", marker_name);
+  entry->marker_id = g_signal_connect (entry->parent,
+                                       marker_reached_signal,
+                                       G_CALLBACK (on_timeline_marker),
+                                       entry);
+
+  entry->node = g_node_append_data (node, entry);
+
+  g_free (marker_reached_signal);
+
+  priv->last_id += 1;
+
+  return entry->id;
 }
 
 /**
@@ -875,7 +1020,7 @@ clutter_score_append (ClutterScore    *score,
  */
 void
 clutter_score_remove (ClutterScore *score,
-                      guint         id)
+                      gulong        id)
 {
   ClutterScorePrivate *priv;
   TraverseClosure closure;
@@ -915,17 +1060,13 @@ clutter_score_remove_all (ClutterScore *score)
 
   g_return_if_fail (CLUTTER_IS_SCORE (score));
 
+  priv = score->priv;
+
   /* this will take care of the running timelines */
   clutter_score_stop (score);
 
-  priv = score->priv;
-
-  g_node_traverse (priv->root,
-                   G_POST_ORDER,
-                   G_TRAVERSE_ALL,
-                   -1,
-                   destroy_entry, NULL);
-  g_node_destroy (priv->root);
+  /* destroy all the contents of the tree */
+  clutter_score_clear (score);
 
   /* recreate the sentinel */
   priv->root = g_node_new (NULL);
@@ -945,7 +1086,7 @@ clutter_score_remove_all (ClutterScore *score)
  */
 ClutterTimeline *
 clutter_score_get_timeline (ClutterScore *score,
-                            guint         id)
+                            gulong        id)
 {
   GNode *node;
   ClutterScoreEntry *entry;
