@@ -47,7 +47,7 @@
  *
  * A simple object might be defined as:
  *
- * <programlisting>
+ * |[
  * {
  *   "id"     : "red-button",
  *   "type"   : "ClutterRectangle",
@@ -55,18 +55,20 @@
  *   "height" : 100,
  *   "color"  : "0xff0000ff"
  * }
- * </programlisting>
+ * ]|
  *
  * This will produce a red #ClutterRectangle, 100x100 pixels wide, and
- * with a name of "red-button"; it can be retrieved by calling:
+ * with a ClutterScript id of "red-button"; it can be retrieved by calling:
  *
- * <programlisting>
+ * |[
  * ClutterActor *red_button;
  *
  * red_button = CLUTTER_ACTOR (clutter_script_get_object (script, "red-button"));
- * </programlisting>
+ * ]|
  *
- * and then manipulated with the Clutter API.
+ * and then manipulated with the Clutter API. For every object created
+ * using ClutterScript it is possible to check the id by calling
+ * clutter_get_script_id().
  *
  * Packing can be represented using the "children" member, and passing an
  * array of objects or ids of objects already defined (but not packed: the
@@ -76,7 +78,7 @@
  * Behaviours and timelines can also be defined inside a UI definition
  * buffer:
  *
- * <programlisting>
+ * |[
  * {
  *   "id"          : "rotate-behaviour",
  *   "type"        : "ClutterBehaviourRotate",
@@ -88,19 +90,19 @@
  *     "function" : "sine"
  *   }
  * }
- * </programlisting>
+ * ]|
  *
  * And then to apply a defined behaviour to an actor defined inside the
  * definition of an actor, the "behaviour" member can be used:
  *
- * <programlisting>
+ * |[
  * {
  *   "id" : "my-rotating-actor",
  *   "type" : "ClutterTexture",
  *   ...
  *   "behaviours" : [ "rotate-behaviour" ]
  * }
- * </programlisting>
+ * ]|
  *
  * A #ClutterAlpha belonging to a #ClutterBehaviour can only be defined
  * implicitely. A #ClutterTimeline belonging to a #ClutterAlpha can be
@@ -115,7 +117,7 @@
  * then autoconnected to their respective signals using the
  * clutter_script_connect_signals() function:
  *
- * <programlisting>
+ * |[
  *   ...
  *   "signals" : [
  *     { "name" : "button-press-event", "handler" : "on_button_press" },
@@ -124,8 +126,9 @@
  *       "handler" : "after_foo",
  *       "after" : true
  *     },
- *   ]
- * </programlisting>
+ *   ],
+ *   ...
+ * ]|
  *
  * Signal handler definitions must have a "name" and a "handler" members;
  * they can also have the "after" and "swapped" boolean members (for the
@@ -160,6 +163,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <gmodule.h>
 
 #include "clutter-actor.h"
 #include "clutter-alpha.h"
@@ -197,6 +201,8 @@ struct _ClutterScriptPrivate
 
   JsonParser *parser;
 
+  gchar **search_paths;
+
   gchar *filename;
   guint is_filename : 1;
 };
@@ -281,7 +287,8 @@ get_id_from_node (JsonNode *node)
 }
 
 static GList *
-parse_children (JsonNode *node)
+parse_children (ObjectInfo *oinfo,
+                JsonNode   *node)
 {
   JsonArray *array;
   GList *retval;
@@ -290,7 +297,7 @@ parse_children (JsonNode *node)
   if (JSON_NODE_TYPE (node) != JSON_NODE_ARRAY)
     return NULL;
 
-  retval = NULL;
+  retval = oinfo->children;
 
   array = json_node_get_array (node);
   array_len = json_array_get_length (array);
@@ -309,7 +316,8 @@ parse_children (JsonNode *node)
 }
 
 static GList *
-parse_signals (JsonNode *node)
+parse_signals (ObjectInfo *oinfo,
+               JsonNode   *node)
 {
   JsonArray *array;
   GList *retval;
@@ -321,7 +329,7 @@ parse_signals (JsonNode *node)
       return NULL;
     }
 
-  retval = NULL;
+  retval = oinfo->signals;
   array = json_node_get_array (node);
   array_len = json_array_get_length (array);
 
@@ -428,7 +436,8 @@ parse_signals (JsonNode *node)
 }
 
 static GList *
-parse_behaviours (JsonNode *node)
+parse_behaviours (ObjectInfo *oinfo,
+                  JsonNode   *node)
 {
   JsonArray *array;
   GList *retval;
@@ -437,7 +446,7 @@ parse_behaviours (JsonNode *node)
   if (JSON_NODE_TYPE (node) != JSON_NODE_ARRAY)
     return NULL;
 
-  retval = NULL;
+  retval = oinfo->behaviours;
 
   array = json_node_get_array (node);
   array_len = json_array_get_length (array);
@@ -659,6 +668,7 @@ json_object_end (JsonParser *parser,
   ClutterScriptPrivate *priv = script->priv;
   ObjectInfo *oinfo;
   JsonNode *val;
+  const gchar *id;
   GList *members, *l;
 
   if (!json_object_has_member (object, "id"))
@@ -687,27 +697,32 @@ json_object_end (JsonParser *parser,
       return;
     }
 
-  oinfo = g_slice_new0 (ObjectInfo);
-  oinfo->merge_id = priv->last_merge_id;
-  
   val = json_object_get_member (object, "id");
-  oinfo->id = json_node_dup_string (val);
+  id = json_node_get_string (val);
 
-  val = json_object_get_member (object, "type");
-  oinfo->class_name = json_node_dup_string (val);
-
-  if (json_object_has_member (object, "type_func"))
+  oinfo = g_hash_table_lookup (priv->objects, id);
+  if (G_LIKELY (!oinfo))
     {
-      val = json_object_get_member (object, "type_func");
-      oinfo->type_func = json_node_dup_string (val);
+      oinfo = g_slice_new0 (ObjectInfo);
+      oinfo->merge_id = priv->last_merge_id;
+      oinfo->id = g_strdup (id);
 
-      json_object_remove_member (object, "type_func");
+      val = json_object_get_member (object, "type");
+      oinfo->class_name = json_node_dup_string (val);
+  
+      if (json_object_has_member (object, "type_func"))
+        {
+          val = json_object_get_member (object, "type_func");
+          oinfo->type_func = json_node_dup_string (val);
+
+          json_object_remove_member (object, "type_func");
+        }
     }
 
   if (json_object_has_member (object, "children"))
     {
       val = json_object_get_member (object, "children");
-      oinfo->children = parse_children (val);
+      oinfo->children = parse_children (oinfo, val);
 
       json_object_remove_member (object, "children");
     }
@@ -715,7 +730,7 @@ json_object_end (JsonParser *parser,
   if (json_object_has_member (object, "behaviours"))
     {
       val = json_object_get_member (object, "behaviours");
-      oinfo->behaviours = parse_behaviours (val);
+      oinfo->behaviours = parse_behaviours (oinfo, val);
 
       json_object_remove_member (object, "behaviours");
     }
@@ -723,11 +738,22 @@ json_object_end (JsonParser *parser,
   if (json_object_has_member (object, "signals"))
     {
       val = json_object_get_member (object, "signals");
-      oinfo->signals = parse_signals (val);
+      oinfo->signals = parse_signals (oinfo, val);
 
       json_object_remove_member (object, "signals");
     }
 
+  if (strcmp (oinfo->class_name, "ClutterStage") == 0 &&
+      json_object_has_member (object, "is-default"))
+    {
+      val = json_object_get_member (object, "is-default");
+      oinfo->is_stage_default = json_node_get_boolean (val);
+
+      json_object_remove_member (object, "is-default");
+    }
+  else
+    oinfo->is_stage_default = FALSE;
+
   oinfo->is_toplevel = FALSE;
   oinfo->is_unmerged = FALSE;
   oinfo->has_unresolved = TRUE;
@@ -764,7 +790,8 @@ json_object_end (JsonParser *parser,
                 g_list_length (oinfo->properties),
                 g_list_length (oinfo->signals));
 
-  g_hash_table_replace (priv->objects, g_strdup (oinfo->id), oinfo);
+  g_hash_table_steal (priv->objects, oinfo->id);
+  g_hash_table_insert (priv->objects, oinfo->id, oinfo);
 
   oinfo->object = clutter_script_construct_object (script, oinfo);
 }
@@ -927,6 +954,7 @@ clutter_script_parse_node (ClutterScript *script,
           break;
 
         case G_TYPE_OBJECT:
+#ifdef USE_GDKPIXBUF
           if (G_VALUE_HOLDS (value, GDK_TYPE_PIXBUF))
             {
               if (G_VALUE_HOLDS (&node_value, G_TYPE_STRING))
@@ -958,7 +986,7 @@ clutter_script_parse_node (ClutterScript *script,
                       g_warning ("Unable to open image at path `%s': %s",
                                  path,
                                  error->message);
-                                 g_error_free (error);
+                      g_error_free (error);
                     }
                   else
                     {
@@ -983,6 +1011,7 @@ clutter_script_parse_node (ClutterScript *script,
                 }
             }
           break;
+#endif
 
         default:
           retval = FALSE;
@@ -1211,6 +1240,22 @@ add_children (ClutterScript    *script,
   oinfo->children = unresolved;
 }
 
+static const struct
+{
+  const gchar *type_name;
+  guint is_toplevel : 1;
+} clutter_toplevels[] = {
+  { "ClutterActor",          FALSE },
+  { "ClutterAlpha",          FALSE },
+  { "ClutterBehaviour",      TRUE  },
+  { "ClutterEffectTemplate", TRUE  },
+  { "ClutterModel",          TRUE  },
+  { "ClutterScore",          TRUE  },
+  { "ClutterTimeline",       TRUE  }
+};
+
+static guint n_clutter_toplevels = G_N_ELEMENTS (clutter_toplevels);
+
 GObject *
 clutter_script_construct_object (ClutterScript *script,
                                  ObjectInfo    *oinfo)
@@ -1243,9 +1288,9 @@ clutter_script_construct_object (ClutterScript *script,
 
   if (oinfo->object)
     object = oinfo->object;
-  else if (oinfo->gtype == CLUTTER_TYPE_STAGE)
+  else if (oinfo->gtype == CLUTTER_TYPE_STAGE && oinfo->is_stage_default)
     {
-      /* the stage is a complex beast: we cannot create it using
+      /* the default stage is a complex beast: we cannot create it using
        * g_object_newv() but we need clutter_script_construct_parameters()
        * to add the GParamSpec to the PropertyInfo pspec member, so
        * that we don't have to implement every complex property (like
@@ -1339,6 +1384,19 @@ clutter_script_construct_object (ClutterScript *script,
     }
   g_array_free (params, TRUE);
 
+  for (i = 0; i < n_clutter_toplevels; i++)
+    {
+      const gchar *t_name = clutter_toplevels[i].type_name;
+      GType t_type;
+      
+      t_type = clutter_script_get_type_from_name (script, t_name);
+      if (g_type_is_a (oinfo->gtype, t_type))
+        {
+          oinfo->is_toplevel = clutter_toplevels[i].is_toplevel;
+          break;
+        }
+    }
+
   if (oinfo->children && CLUTTER_IS_CONTAINER (object))
     add_children (script, CLUTTER_CONTAINER (object), oinfo);
 
@@ -1450,15 +1508,30 @@ object_info_free (gpointer data)
       g_list_foreach (oinfo->behaviours, (GFunc) g_free, NULL);
       g_list_free (oinfo->behaviours);
 
-      if (oinfo->is_toplevel && oinfo->object)
+      /* we unref top-level objects and leave the actors alone,
+       * unless we are unmerging in which case we have to destroy
+       * the actor to unparent them
+       */
+      if (oinfo->object)
         {
-          g_object_unref (oinfo->object);
-          oinfo->object = NULL;
-        }
+          if (oinfo->is_unmerged)
+            {
+              if (oinfo->is_toplevel)
+                g_object_unref (oinfo->object);
+              else
+                {
+                  /* destroy every actor, unless it's the default stage */
+                  if (oinfo->is_stage_default == FALSE &&
+                      CLUTTER_IS_ACTOR (oinfo->object))
+                    clutter_actor_destroy (CLUTTER_ACTOR (oinfo->object));
+                }
+            }
+          else
+            {
+              if (oinfo->is_toplevel)
+                g_object_unref (oinfo->object);
+            }
 
-      if (oinfo->is_unmerged && oinfo->object)
-        {
-          clutter_actor_destroy (CLUTTER_ACTOR (oinfo->object));
           oinfo->object = NULL;
         }
 
@@ -1473,6 +1546,7 @@ clutter_script_finalize (GObject *gobject)
 
   g_object_unref (priv->parser);
   g_hash_table_destroy (priv->objects);
+  g_strfreev (priv->search_paths);
   g_free (priv->filename);
 
   G_OBJECT_CLASS (clutter_script_parent_class)->finalize (gobject);
@@ -1564,7 +1638,7 @@ clutter_script_init (ClutterScript *script)
   priv->last_merge_id = 0;
 
   priv->objects = g_hash_table_new_full (g_str_hash, g_str_equal,
-                                         g_free,
+                                         NULL,
                                          object_info_free);
 }
 
@@ -2085,3 +2159,126 @@ clutter_script_error_quark (void)
 {
   return g_quark_from_static_string ("clutter-script-error");
 }
+
+/**
+ * clutter_script_add_search_paths:
+ * @script: a #ClutterScript
+ * @paths: an array of strings containing different search paths
+ * @n_paths: the length of the passed array
+ *
+ * Adds @paths to the list of search paths held by @script.
+ *
+ * The search paths are used by clutter_script_lookup_filename(), which
+ * can be used to define search paths for the textures source file name
+ * or other custom, file-based properties.
+ *
+ * Since: 0.8
+ */
+void
+clutter_script_add_search_paths (ClutterScript       *script,
+                                 const gchar * const  paths[],
+                                 gsize                n_paths)
+{
+  ClutterScriptPrivate *priv;
+  gchar **old_paths, **new_paths;
+  gsize old_paths_len, i;
+  gsize iter = 0;
+
+  g_return_if_fail (CLUTTER_IS_SCRIPT (script));
+  g_return_if_fail (paths != NULL);
+  g_return_if_fail (n_paths > 0);
+
+  priv = script->priv;
+
+  if (priv->search_paths)
+    {
+      old_paths     = priv->search_paths;
+      old_paths_len = g_strv_length (old_paths);
+    }
+  else
+    {
+      old_paths     = NULL;
+      old_paths_len = 0;
+    }
+
+  new_paths = g_new0 (gchar*, old_paths_len + n_paths + 1);
+
+  for (i = 0, iter = 0; i < old_paths_len; i++, iter++)
+    new_paths[iter] = g_strdup (old_paths[i]);
+
+  for (i = 0; i < n_paths; i++, iter++)
+    new_paths[iter] = g_strdup (paths[i]);
+
+  CLUTTER_NOTE (SCRIPT, "Added %d new search paths (new size: %d)",
+                n_paths,
+                g_strv_length (new_paths));
+
+  priv->search_paths = new_paths;
+  g_strfreev (old_paths);
+}
+
+/**
+ * clutter_script_lookup_filename:
+ * @script: a #ClutterScript
+ * @filename: the name of the file to lookup
+ *
+ * Looks up @filename inside the search paths of @script. If @filename
+ * is found, its full path will be returned .
+ *
+ * Return value: the full path of @filename or %NULL if no path was
+ *   found.
+ *
+ * Since: 0.8
+ */
+gchar *
+clutter_script_lookup_filename (ClutterScript *script,
+                                const gchar   *filename)
+{
+  ClutterScriptPrivate *priv;
+  gchar *dirname;
+  gchar *retval;
+
+  g_return_val_if_fail (CLUTTER_IS_SCRIPT (script), FALSE);
+  g_return_val_if_fail (filename != NULL, FALSE);
+
+  if (g_path_is_absolute (filename))
+    return g_strdup (filename);
+
+  priv = script->priv;
+
+  if (priv->search_paths)
+    {
+      gsize paths_len, i;
+
+      paths_len = g_strv_length (priv->search_paths);
+      for (i = 0; i < paths_len; i++)
+        {
+          retval = g_build_filename (priv->search_paths[i], filename, NULL);
+          if (g_file_test (retval, G_FILE_TEST_EXISTS))
+            return retval;
+          else
+            {
+              g_free (retval);
+              retval = NULL;
+            }
+        }
+    }
+
+  /* Fall back to assuming relative to our script */
+  if (priv->is_filename)
+    dirname = g_path_get_dirname (script->priv->filename);
+  else
+    dirname = g_get_current_dir ();
+  
+  retval = g_build_filename (dirname, filename, NULL);
+  if (!g_file_test (retval, G_FILE_TEST_EXISTS))
+    {
+      g_free (retval);
+      retval = NULL;
+    }
+  
+  g_free (dirname);
+
+  return retval;
+}
+
