@@ -49,7 +49,7 @@
 
 #include <glib.h>
 
-#include <cogl/cogl.h>
+#include "cogl/cogl.h"
 
 #include "clutter-debug.h"
 #include "clutter-private.h"
@@ -68,7 +68,7 @@ typedef enum {
 
 struct _ClutterShaderPrivate
 {
-  guint       bound            : 1; /* Shader is bound to the GL context */
+  guint       compiled         : 1; /* Shader is bound to the GL context */
   guint       is_enabled       : 1;
 
   guint       vertex_is_glsl   : 1;
@@ -77,10 +77,10 @@ struct _ClutterShaderPrivate
   gchar      *vertex_source;        /* GLSL source for vertex shader */
   gchar      *fragment_source;      /* GLSL source for fragment shader */
 
-  COGLhandle  program;
+  CoglHandle  program;
 
-  COGLhandle  vertex_shader;
-  COGLhandle  fragment_shader;
+  CoglHandle  vertex_shader;
+  CoglHandle  fragment_shader;
 };
 
 enum 
@@ -89,7 +89,7 @@ enum
 
   PROP_VERTEX_SOURCE,
   PROP_FRAGMENT_SOURCE,
-  PROP_BOUND,
+  PROP_COMPILED,
   PROP_ENABLED
 };
 
@@ -164,8 +164,8 @@ clutter_shader_get_property (GObject    *object,
     case PROP_FRAGMENT_SOURCE:
       g_value_set_string (value, priv->fragment_source);
       break;
-    case PROP_BOUND:
-      g_value_set_boolean (value, priv->bound);
+    case PROP_COMPILED:
+      g_value_set_boolean (value, priv->compiled);
       break;
     case PROP_ENABLED:
       g_value_set_boolean (value, priv->is_enabled);
@@ -237,19 +237,19 @@ clutter_shader_class_init (ClutterShaderClass *klass)
 			  NULL,
 			  CLUTTER_PARAM_READWRITE));
   /**
-   * ClutterShader:bound:
+   * ClutterShader:compiled:
    *
-   * Whether the shader is bound (compiled and linked, ready for use
-   * in the GL context).
+   * Whether the shader is compiled and linked, ready for use
+   * in the GL context.
    *
-   * Since: 0.6
+   * Since: 0.8
    */
   g_object_class_install_property 
     (object_class,
-     PROP_BOUND,
-     g_param_spec_boolean ("bound",
-			   "Bound",
-			   "Whether the shader is bound",
+     PROP_COMPILED,
+     g_param_spec_boolean ("compiled",
+			   "Compiled",
+			   "Whether the shader is compiled and linked",
 			   FALSE,
 			   CLUTTER_PARAM_READABLE));
   /**
@@ -276,14 +276,14 @@ clutter_shader_init (ClutterShader *self)
 
   priv = self->priv = CLUTTER_SHADER_GET_PRIVATE (self);
 
-  priv->bound = FALSE;
+  priv->compiled = FALSE;
 
   priv->vertex_source = NULL;
   priv->fragment_source = NULL;
 
-  priv->program = 0;
-  priv->vertex_shader = 0;
-  priv->fragment_shader = 0;
+  priv->program = COGL_INVALID_HANDLE;
+  priv->vertex_shader = COGL_INVALID_HANDLE;
+  priv->fragment_shader = COGL_INVALID_HANDLE;
 }
 
 /**
@@ -334,7 +334,7 @@ clutter_shader_set_fragment_source (ClutterShader      *shader,
   /* release shader if bound when changing the source, the shader will
    * automatically be rebound on the next use.
    */
-  if (clutter_shader_is_bound (shader))
+  if (clutter_shader_is_compiled (shader))
     clutter_shader_release (shader);
 
   is_glsl = !g_str_has_prefix (data, "!!ARBfp");
@@ -380,7 +380,7 @@ clutter_shader_set_vertex_source (ClutterShader      *shader,
   /* release shader if bound when changing the source, the shader will
    * automatically be rebound on the next use.
    */
-  if (clutter_shader_is_bound (shader))
+  if (clutter_shader_is_compiled (shader))
     clutter_shader_release (shader);
 
   is_glsl = !g_str_has_prefix (data, "!!ARBvp");
@@ -403,21 +403,38 @@ bind_glsl_shader (ClutterShader  *self,
   ClutterShaderPrivate *priv;
   priv = self->priv;
 
-  cogl_enable (CGL_FRAGMENT_SHADER);
-  cogl_enable (CGL_VERTEX_SHADER);
-
   priv->program = cogl_create_program ();
 
-  if (priv->vertex_is_glsl && priv->vertex_source)
+  if (priv->vertex_is_glsl && priv->vertex_source != COGL_INVALID_HANDLE)
     {
-     priv->vertex_shader = cogl_create_shader (CGL_VERTEX_SHADER);
+      GLint compiled = CGL_FALSE;
 
-     cogl_shader_source (priv->vertex_shader, priv->vertex_source);
-     cogl_shader_compile (priv->vertex_shader);
-     cogl_program_attach_shader (priv->program, priv->vertex_shader);
+      priv->vertex_shader = cogl_create_shader (CGL_VERTEX_SHADER);
+
+      cogl_shader_source (priv->vertex_shader, priv->vertex_source);
+      cogl_shader_compile (priv->vertex_shader);
+
+      cogl_shader_get_parameteriv (priv->vertex_shader,
+                                   CGL_OBJECT_COMPILE_STATUS,
+                                   &compiled);
+      if (compiled != CGL_TRUE)
+        {
+          gchar error_buf[512];
+
+          cogl_shader_get_info_log (priv->vertex_shader, 512, error_buf);
+
+          g_set_error (error, CLUTTER_SHADER_ERROR,
+                       CLUTTER_SHADER_ERROR_COMPILE,
+                       "Vertex shader compilation failed: %s",
+                       error_buf);
+
+          return FALSE;
+        }
+      else
+        cogl_program_attach_shader (priv->program, priv->vertex_shader);
     }
 
-  if (priv->fragment_is_glsl && priv->fragment_source)
+  if (priv->fragment_is_glsl && priv->fragment_source != COGL_INVALID_HANDLE)
     {
       GLint compiled = CGL_FALSE;
 
@@ -437,7 +454,7 @@ bind_glsl_shader (ClutterShader  *self,
 
           g_set_error (error, CLUTTER_SHADER_ERROR,
                        CLUTTER_SHADER_ERROR_COMPILE,
-                       "Shader compilation failed: %s",
+                       "Fragment shader compilation failed: %s",
                        error_buf);
 
           return FALSE;
@@ -452,21 +469,21 @@ bind_glsl_shader (ClutterShader  *self,
 }
 
 /**
- * clutter_shader_bind:
+ * clutter_shader_compile:
  * @shader: a #ClutterShader
  * @error: return location for a #GError, or %NULL
  *
- * Compile and link GLSL sources set for vertex and fragment shaders for
+ * Compiles and links GLSL sources set for vertex and fragment shaders for
  * a #ClutterShader. If the compilation fails and a #GError return location is
  * provided the error will contain the errors from the compiler, if any.
  *
- * Return value: returns TRUE if the shader was succesfully bound.
+ * Return value: returns TRUE if the shader was succesfully compiled.
  *
- * Since: 0.6
+ * Since: 0.8
  */
 gboolean
-clutter_shader_bind (ClutterShader  *shader,
-                     GError        **error)
+clutter_shader_compile (ClutterShader  *shader,
+                        GError        **error)
 {
   ClutterShaderPrivate *priv;
 
@@ -474,19 +491,19 @@ clutter_shader_bind (ClutterShader  *shader,
 
   priv = shader->priv;
 
-  if (priv->bound)
-    return priv->bound;
+  if (priv->compiled)
+    return priv->compiled;
 
-  if ((priv->vertex_source   && !priv->vertex_is_glsl) ||
-      (priv->fragment_source && !priv->fragment_is_glsl))
+  if ((priv->vertex_source != COGL_INVALID_HANDLE && !priv->vertex_is_glsl) ||
+      (priv->fragment_source != COGL_INVALID_HANDLE && !priv->fragment_is_glsl))
     {
       /* XXX: Could remove this check, since we only advertise support for GLSL
        * shaders anyways. */
       g_set_error (error, CLUTTER_SHADER_ERROR,
                    CLUTTER_SHADER_ERROR_NO_ASM,
                    "ASM shaders not supported");
-      priv->bound = FALSE;
-      return priv->bound;
+      priv->compiled = FALSE;
+      return priv->compiled;
     }
 
   if (!clutter_feature_available (CLUTTER_FEATURE_SHADERS_GLSL))
@@ -494,23 +511,21 @@ clutter_shader_bind (ClutterShader  *shader,
       g_set_error (error, CLUTTER_SHADER_ERROR,
                    CLUTTER_SHADER_ERROR_NO_GLSL,
                    "GLSL shaders not supported");
-      priv->bound = FALSE;
-      return priv->bound;
+      priv->compiled = FALSE;
+      return priv->compiled;
     }
 
-  priv->bound = bind_glsl_shader (shader, error);
+  priv->compiled = bind_glsl_shader (shader, error);
+  g_object_notify (G_OBJECT (shader), "compiled");
 
-  if (priv->bound)
-    g_object_notify (G_OBJECT (shader), "bound");
-
-  return priv->bound;
+  return priv->compiled;
 }
 
 /**
  * clutter_shader_release:
  * @shader: a #ClutterShader
  *
- * Free up any GL context resources held by the shader.
+ * Frees up any GL context resources held by the shader.
  *
  * Since: 0.6
  */
@@ -523,30 +538,30 @@ clutter_shader_release (ClutterShader *shader)
 
   priv = shader->priv;
 
-  if (!priv->bound)
+  if (!priv->compiled)
     return;
 
-  g_assert (priv->program);
+  g_assert (priv->program != COGL_INVALID_HANDLE);
 
-  if (priv->vertex_is_glsl && priv->vertex_shader)
-    cogl_shader_destroy (priv->vertex_shader);
+  if (priv->vertex_is_glsl && priv->vertex_shader != COGL_INVALID_HANDLE)
+    cogl_shader_unref (priv->vertex_shader);
 
-  if (priv->fragment_is_glsl && priv->fragment_shader)
-    cogl_shader_destroy (priv->fragment_shader);
+  if (priv->fragment_is_glsl && priv->fragment_shader != COGL_INVALID_HANDLE)
+    cogl_shader_unref (priv->fragment_shader);
 
-  if (priv->program)
-    cogl_program_destroy (priv->program);
+  if (priv->program != COGL_INVALID_HANDLE)
+    cogl_program_unref (priv->program);
 
-  priv->vertex_shader = 0;
-  priv->fragment_shader = 0;
-  priv->program = 0;
-  priv->bound = FALSE;
+  priv->vertex_shader = COGL_INVALID_HANDLE;
+  priv->fragment_shader = COGL_INVALID_HANDLE;
+  priv->program = COGL_INVALID_HANDLE;
+  priv->compiled = FALSE;
 
-  g_object_notify (G_OBJECT (shader), "bound");
+  g_object_notify (G_OBJECT (shader), "compiled");
 }
 
 /**
- * clutter_shader_is_bound:
+ * clutter_shader_is_compiled:
  * @shader: a #ClutterShader
  *
  * Checks whether @shader is is currently compiled, linked and bound
@@ -554,14 +569,14 @@ clutter_shader_release (ClutterShader *shader)
  *
  * Return value: %TRUE if the shader is compiled, linked and ready for use.
  *
- * Since: 0.6
+ * Since: 0.8
  */
 gboolean
-clutter_shader_is_bound (ClutterShader *shader)
+clutter_shader_is_compiled (ClutterShader *shader)
 {
   g_return_val_if_fail (CLUTTER_IS_SHADER (shader), FALSE);
 
-  return shader->priv->bound;
+  return shader->priv->compiled;
 }
 
 /**
@@ -569,9 +584,11 @@ clutter_shader_is_bound (ClutterShader *shader)
  * @shader: a #ClutterShader
  * @enabled: The new state of the shader.
  *
- * Enable a shader, will attempt to bind the shader if it isn't already bound.
- * When FALSE is passed in the default state of the GL pipeline will be used
- * instead.
+ * Enables a shader. This function will attempt to compile and link
+ * the shader, if it isn't already.
+ *
+ * When @enabled is %FALSE the default state of the GL pipeline will be
+ * used instead.
  *
  * Since: 0.6
  */
@@ -590,7 +607,7 @@ clutter_shader_set_is_enabled (ClutterShader *shader,
       GError *error = NULL;
       gboolean res;
 
-      res = clutter_shader_bind (shader, &error);
+      res = clutter_shader_compile (shader, &error);
       if (!res)
         {
           g_warning ("Unable to bind the shader: %s",
@@ -606,7 +623,7 @@ clutter_shader_set_is_enabled (ClutterShader *shader,
       if (priv->is_enabled)
         cogl_program_use (priv->program);
       else
-        cogl_program_use (0);
+        cogl_program_use (COGL_INVALID_HANDLE);
 
       g_object_notify (G_OBJECT (shader), "enabled");
     }
@@ -658,8 +675,8 @@ clutter_shader_set_uniform_1f (ClutterShader *shader,
   cogl_program_uniform_1f (location, foo);
 }
 
-/**
- * clutter_shader_release_all:
+/*
+ * _clutter_shader_release_all:
  *
  * Iterate through all #ClutterShaders and tell them to release GL context
  * related sources.
@@ -667,7 +684,7 @@ clutter_shader_set_uniform_1f (ClutterShader *shader,
  * Since: 0.6
  */
 void
-clutter_shader_release_all (void)
+_clutter_shader_release_all (void)
 {
   g_list_foreach (clutter_shaders_list,
                   (GFunc) clutter_shader_release,
