@@ -29,6 +29,24 @@
  *
  * #ClutterStage is a top level 'window' on which child actors are placed
  * and manipulated.
+ *
+ * Clutter creates a default stage upon initialization, which can be retrieved
+ * using clutter_stage_get_default(). Clutter always provides the default
+ * stage, unless the backend is unable to create one. The stage returned
+ * by clutter_stage_get_default() is guaranteed to always be the same.
+ *
+ * Backends might provide support for multiple stages. The support for this
+ * feature can be checked at run-time using the clutter_feature_available()
+ * function and the %CLUTTER_FEATURE_STAGE_MULTIPLE flag. If the backend used
+ * supports multiple stages, new #ClutterStage instances can be created
+ * using clutter_stage_new(). These stages must be managed by the developer
+ * using clutter_actor_destroy(), which will take care of destroying all the
+ * actors contained inside them.
+ *
+ * #ClutterStage is a proxy actor, wrapping the backend-specific
+ * implementation of the windowing system. It is possible to subclass
+ * #ClutterStage, as long as every overridden virtual function chains up to
+ * the parent class corresponding function.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,29 +62,37 @@
 #include "clutter-enum-types.h"
 #include "clutter-private.h"
 #include "clutter-debug.h"
+#include "clutter-stage-manager.h"
+#include "clutter-stage-window.h"
 #include "clutter-version.h" 	/* For flavour */
+#include "clutter-id-pool.h"
 
-#include "cogl.h"
+#include "cogl/cogl.h"
 
-G_DEFINE_ABSTRACT_TYPE (ClutterStage, clutter_stage, CLUTTER_TYPE_GROUP);
+G_DEFINE_TYPE (ClutterStage, clutter_stage, CLUTTER_TYPE_GROUP);
 
 #define CLUTTER_STAGE_GET_PRIVATE(obj) \
 (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_STAGE, ClutterStagePrivate))
 
 struct _ClutterStagePrivate
 {
+  /* the stage implementation */
+  ClutterActor       *impl;
+
   ClutterColor        color;
   ClutterPerspective  perspective;
   ClutterFog          fog;
 
+  gchar              *title;
+  ClutterActor       *key_focused_actor;
+
+  guint               update_idle;	       /* repaint idler id */
+
   guint is_fullscreen     : 1;
   guint is_offscreen      : 1;
   guint is_cursor_visible : 1;
   guint is_user_resizable : 1;
   guint use_fog           : 1;
-
-  gchar              *title;
-  ClutterActor       *key_focused_actor;
 };
 
 enum
@@ -94,11 +120,100 @@ enum
 
 static guint stage_signals[LAST_SIGNAL] = { 0, };
 
+static void
+clutter_stage_get_preferred_width (ClutterActor *self,
+                                   ClutterUnit   for_height,
+                                   ClutterUnit  *min_width_p,
+                                   ClutterUnit  *natural_width_p)
+{
+  ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+
+  g_assert (priv->impl != NULL);
+
+  CLUTTER_ACTOR_GET_CLASS (priv->impl)->get_preferred_width (priv->impl,
+                                                             for_height,
+                                                             min_width_p,
+                                                             natural_width_p);
+}
+
+static void
+clutter_stage_get_preferred_height (ClutterActor *self,
+                                    ClutterUnit   for_width,
+                                    ClutterUnit  *min_height_p,
+                                    ClutterUnit  *natural_height_p)
+{
+  ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+
+  g_assert (priv->impl != NULL);
+
+  CLUTTER_ACTOR_GET_CLASS (priv->impl)->get_preferred_height (priv->impl,
+                                                              for_width,
+                                                              min_height_p,
+                                                              natural_height_p);
+}
+static void
+clutter_stage_allocate (ClutterActor          *self,
+                        const ClutterActorBox *box,
+                        gboolean               origin_changed)
+{
+  ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+
+  g_assert (priv->impl != NULL);
+
+  /* if the stage is fixed size (for instance, it's using a frame-buffer)
+   * then we simply ignore any allocation request and override the
+   * allocation chain.
+   */
+  if (G_LIKELY (!clutter_feature_available (CLUTTER_FEATURE_STAGE_STATIC)))
+    {
+      ClutterActorClass *klass;
+
+      CLUTTER_NOTE (ACTOR, "Following allocation to %dx%d (origin %s)",
+                    CLUTTER_UNITS_TO_DEVICE (box->x2 - box->x1),
+                    CLUTTER_UNITS_TO_DEVICE (box->y2 - box->y1),
+                    origin_changed ? "changed" : "not changed");
+
+      klass = CLUTTER_ACTOR_CLASS (clutter_stage_parent_class);
+      klass->allocate (self, box, origin_changed);
+
+      klass = CLUTTER_ACTOR_GET_CLASS (priv->impl);
+      klass->allocate (priv->impl, box, origin_changed);
+    }
+  else
+    {
+      ClutterActorBox override = { 0, };
+      ClutterActorClass *klass;
+      ClutterUnit natural_width, natural_height;
+
+      /* propagate the allocation */
+      klass = CLUTTER_ACTOR_GET_CLASS (priv->impl);
+      klass->allocate (self, box, origin_changed);
+
+      /* get the preferred size from the backend */
+      clutter_actor_get_preferred_size (priv->impl,
+                                        NULL, NULL,
+                                        &natural_width, &natural_height);
+
+      override.x1 = 0;
+      override.y1 = 0;
+      override.x2 = natural_width;
+      override.y2 = natural_height;
+
+      /* and store the overridden allocation */
+      klass = CLUTTER_ACTOR_CLASS (clutter_stage_parent_class);
+      klass->allocate (self, &override, origin_changed);
+    }
+}
+
 static void
 clutter_stage_paint (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
 
+  CLUTTER_SET_PRIVATE_FLAGS (self, CLUTTER_ACTOR_IN_PAINT);
+
+  CLUTTER_NOTE (PAINT, "Initializing stage paint");
+
   cogl_paint_init (&priv->color);
 
   if (priv->use_fog)
@@ -109,6 +224,12 @@ clutter_stage_paint (ClutterActor *self)
                     priv->fog.z_far);
     }
 
+  CLUTTER_NOTE (PAINT, "Proxying the paint to the stage implementation");
+  clutter_actor_paint (priv->impl);
+
+  CLUTTER_UNSET_PRIVATE_FLAGS (self, CLUTTER_ACTOR_IN_PAINT);
+
+  /* this will take care of painting every child */
   CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->paint (self);
 }
 
@@ -124,6 +245,92 @@ clutter_stage_pick (ClutterActor       *self,
   CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->paint (self);
 }
 
+static void
+clutter_stage_realize (ClutterActor *self)
+{
+  ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+
+  CLUTTER_ACTOR_SET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
+
+  g_assert (priv->impl != NULL);
+  CLUTTER_ACTOR_GET_CLASS (priv->impl)->realize (priv->impl);
+
+  /* ensure that the stage is using the context if the
+   * realization sequence was successful
+   */
+  if (CLUTTER_ACTOR_IS_REALIZED (priv->impl))
+    clutter_stage_ensure_current (CLUTTER_STAGE (self));
+  else
+    CLUTTER_ACTOR_UNSET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
+}
+
+static void
+clutter_stage_unrealize (ClutterActor *self)
+{
+  ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+
+  /* unset the flag */
+  CLUTTER_ACTOR_UNSET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
+
+  /* and then unrealize the implementation */
+  g_assert (priv->impl != NULL);
+  CLUTTER_ACTOR_GET_CLASS (priv->impl)->unrealize (priv->impl);
+
+  clutter_stage_ensure_current (CLUTTER_STAGE (self));
+}
+
+static void
+clutter_stage_show (ClutterActor *self)
+{
+  ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+
+  g_assert (priv->impl != NULL);
+
+  if (!CLUTTER_ACTOR_IS_REALIZED (priv->impl))
+    clutter_actor_realize (priv->impl);
+
+  clutter_actor_show (priv->impl);
+
+  CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->show (self);
+}
+
+static void
+clutter_stage_hide (ClutterActor *self)
+{
+  ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+
+  g_assert (priv->impl != NULL);
+  clutter_actor_hide (priv->impl);
+
+  CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->hide (self);
+}
+
+static void
+clutter_stage_real_fullscreen (ClutterStage *stage)
+{
+  ClutterStagePrivate *priv = stage->priv;
+  ClutterUnit natural_width, natural_height;
+  ClutterActorBox box;
+
+  /* we need to force an allocation here because the size
+   * of the stage might have been changed by the backend
+   *
+   * this is a really bad solution to the issues caused by
+   * the fact that fullscreening the stage on the X11 backends
+   * is really an asynchronous operation
+   */
+  clutter_actor_get_preferred_size (CLUTTER_ACTOR (priv->impl),
+                                    NULL, NULL,
+                                    &natural_width, &natural_height);
+
+  box.x1 = 0;
+  box.y1 = 0;
+  box.x2 = natural_width;
+  box.y2 = natural_height;
+
+  clutter_actor_allocate (CLUTTER_ACTOR (stage), &box, FALSE);
+}
+
 static void
 clutter_stage_set_property (GObject      *object,
 			    guint         prop_id,
@@ -241,6 +448,40 @@ clutter_stage_get_property (GObject    *object,
     }
 }
 
+static void
+clutter_stage_dispose (GObject *object)
+{
+  ClutterStage        *stage = CLUTTER_STAGE (object);
+  ClutterStagePrivate *priv = stage->priv;
+  ClutterStageManager *stage_manager = clutter_stage_manager_get_default ();
+
+  if (priv->update_idle)
+    {
+      g_source_remove (priv->update_idle);
+      priv->update_idle = 0;
+    }
+
+  clutter_actor_unrealize (CLUTTER_ACTOR (object));
+  
+  _clutter_stage_manager_remove_stage (stage_manager, stage);
+
+  if (priv->impl)
+    {
+      CLUTTER_NOTE (MISC, "Disposing of the stage implementation");
+      g_object_unref (priv->impl);
+      priv->impl = NULL;
+    }
+
+  G_OBJECT_CLASS (clutter_stage_parent_class)->dispose (object);
+}
+
+static void
+clutter_stage_finalize (GObject *object)
+{
+  G_OBJECT_CLASS (clutter_stage_parent_class)->finalize (object);
+}
+
+
 static void
 clutter_stage_class_init (ClutterStageClass *klass)
 {
@@ -249,9 +490,18 @@ clutter_stage_class_init (ClutterStageClass *klass)
 
   gobject_class->set_property = clutter_stage_set_property;
   gobject_class->get_property = clutter_stage_get_property;
+  gobject_class->dispose = clutter_stage_dispose;
+  gobject_class->finalize = clutter_stage_finalize;
 
+  actor_class->allocate = clutter_stage_allocate;
+  actor_class->get_preferred_width = clutter_stage_get_preferred_width;
+  actor_class->get_preferred_height = clutter_stage_get_preferred_height;
   actor_class->paint = clutter_stage_paint;
   actor_class->pick = clutter_stage_pick;
+  actor_class->realize = clutter_stage_realize;
+  actor_class->unrealize = clutter_stage_unrealize;
+  actor_class->show = clutter_stage_show;
+  actor_class->hide = clutter_stage_hide;
 
   /**
    * ClutterStage:fullscreen:
@@ -358,7 +608,7 @@ clutter_stage_class_init (ClutterStageClass *klass)
   stage_signals[FULLSCREEN] =
     g_signal_new ("fullscreen",
 		  G_TYPE_FROM_CLASS (gobject_class),
-		  G_SIGNAL_RUN_LAST,
+		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (ClutterStageClass, fullscreen),
 		  NULL, NULL,
 		  clutter_marshal_VOID__VOID,
@@ -415,6 +665,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
 		  clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
 
+  klass->fullscreen = clutter_stage_real_fullscreen;
+
   g_type_class_add_private (gobject_class, sizeof (ClutterStagePrivate));
 }
 
@@ -422,12 +674,28 @@ static void
 clutter_stage_init (ClutterStage *self)
 {
   ClutterStagePrivate *priv;
+  ClutterBackend *backend;
 
   /* a stage is a top-level object */
   CLUTTER_SET_PRIVATE_FLAGS (self, CLUTTER_ACTOR_IS_TOPLEVEL);
 
   self->priv = priv = CLUTTER_STAGE_GET_PRIVATE (self);
 
+  CLUTTER_NOTE (BACKEND, "Creating stage from the default backend");
+  backend = clutter_get_default_backend ();
+  priv->impl = _clutter_backend_create_stage (backend, self, NULL);
+  if (!priv->impl)
+    {
+      g_warning ("Unable to create a new stage, falling back to the "
+                 "default stage.");
+      priv->impl = CLUTTER_ACTOR (_clutter_stage_get_default_window ());
+
+      /* at this point we must have a default stage, or we're screwed */
+      g_assert (priv->impl != NULL);
+    }
+  else
+    g_object_ref_sink (priv->impl);
+
   priv->is_offscreen      = FALSE;
   priv->is_fullscreen     = FALSE;
   priv->is_user_resizable = FALSE;
@@ -449,7 +717,6 @@ clutter_stage_init (ClutterStage *self)
   priv->fog.z_near  = CLUTTER_FLOAT_TO_FIXED (1.0);
   priv->fog.z_far   = CLUTTER_FLOAT_TO_FIXED (2.0);
 
-  clutter_actor_set_size (CLUTTER_ACTOR (self), 640, 480);
   clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);
   clutter_stage_set_key_focus (self, NULL);
 }
@@ -457,11 +724,12 @@ clutter_stage_init (ClutterStage *self)
 /**
  * clutter_stage_get_default:
  *
- * Returns the main stage.  #ClutterStage is a singleton, so
- * the stage will be created the first time this function is
+ * Returns the main stage. The default #ClutterStage is a singleton,
+ * so the stage will be created the first time this function is
  * called (typically, inside clutter_init()); all the subsequent
- * calls to clutter_stage_get_default() will return the same
- * instance.
+ * calls to clutter_stage_get_default() will return the same instance.
+ *
+ * Clutter guarantess the existence of the default stage.
  *
  * Return value: the main #ClutterStage.  You should never
  *   destroy or unref the returned actor.
@@ -469,12 +737,18 @@ clutter_stage_init (ClutterStage *self)
 ClutterActor *
 clutter_stage_get_default (void)
 {
-  ClutterMainContext *context;
+  ClutterStageManager *stage_manager = clutter_stage_manager_get_default ();
+  ClutterStage *stage;
 
-  context = clutter_context_get_default ();
-  g_assert (context != NULL);
+  stage = clutter_stage_manager_get_default_stage (stage_manager);
+  if (G_UNLIKELY (stage == NULL))
+    /* This will take care of automatically adding the stage to the
+     * stage manager and setting it as the default. Its floating
+     * reference will be claimed by the stage manager.
+     */
+    stage = g_object_new (CLUTTER_TYPE_STAGE, NULL);
 
-  return _clutter_backend_get_stage (context->backend);
+  return CLUTTER_ACTOR (stage);
 }
 
 /**
@@ -494,10 +768,8 @@ clutter_stage_set_color (ClutterStage       *stage,
   g_return_if_fail (color != NULL);
 
   priv = stage->priv;
-  priv->color.red = color->red;
-  priv->color.green = color->green;
-  priv->color.blue = color->blue;
-  priv->color.alpha = color->alpha;
+
+  priv->color = *color;
 
   if (CLUTTER_ACTOR_IS_VISIBLE (stage))
     clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
@@ -523,10 +795,7 @@ clutter_stage_get_color (ClutterStage *stage,
 
   priv = stage->priv;
 
-  color->red = priv->color.red;
-  color->green = priv->color.green;
-  color->blue = priv->color.blue;
-  color->alpha = priv->color.alpha;
+  *color = priv->color;
 }
 
 /**
@@ -550,6 +819,9 @@ clutter_stage_set_perspectivex (ClutterStage       *stage,
 
   priv->perspective = *perspective;
 
+  /* this will cause the viewport to be reset; see
+   * clutter_maybe_setup_viewport() inside clutter-main.c
+   */
   CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_SYNC_MATRICES);
 }
 
@@ -582,7 +854,7 @@ clutter_stage_get_perspectivex (ClutterStage       *stage,
  * @z_far: the  distance from the viewer to the far clipping
  *   plane (always positive)
  *
- * Set the stage perspective.
+ * Sets the stage perspective.
  *
  * Since: 0.4
  */
@@ -599,11 +871,14 @@ clutter_stage_set_perspective (ClutterStage *stage,
 
   priv = stage->priv;
 
-  priv->perspective.fovy   = CLUTTER_FLOAT_TO_FIXED(fovy);
-  priv->perspective.aspect = CLUTTER_FLOAT_TO_FIXED(aspect);
-  priv->perspective.z_near = CLUTTER_FLOAT_TO_FIXED(z_near);
-  priv->perspective.z_far  = CLUTTER_FLOAT_TO_FIXED(z_far);
+  priv->perspective.fovy   = CLUTTER_FLOAT_TO_FIXED (fovy);
+  priv->perspective.aspect = CLUTTER_FLOAT_TO_FIXED (aspect);
+  priv->perspective.z_near = CLUTTER_FLOAT_TO_FIXED (z_near);
+  priv->perspective.z_far  = CLUTTER_FLOAT_TO_FIXED (z_far);
 
+  /* this will cause the viewport to be reset; see
+   * clutter_maybe_setup_viewport() inside clutter-main.c
+   */
   CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_SYNC_MATRICES);
 }
 
@@ -666,12 +941,17 @@ clutter_stage_fullscreen (ClutterStage *stage)
   priv = stage->priv;
   if (!priv->is_fullscreen)
     {
+      ClutterStageWindow *impl = CLUTTER_STAGE_WINDOW (priv->impl);
+      ClutterStageWindowIface *iface;
+
+      iface = CLUTTER_STAGE_WINDOW_GET_IFACE (impl);
+
       /* Only set if backend implements.
        * Also see clutter_stage_event() for setting priv->is_fullscreen
        * on state change event.
       */
-      if (CLUTTER_STAGE_GET_CLASS (stage)->set_fullscreen)
-	CLUTTER_STAGE_GET_CLASS (stage)->set_fullscreen (stage, TRUE);
+      if (iface->set_fullscreen)
+	iface->set_fullscreen (impl, TRUE);
     }
 }
 
@@ -695,9 +975,17 @@ clutter_stage_unfullscreen (ClutterStage *stage)
   priv = stage->priv;
   if (priv->is_fullscreen)
     {
-      /* Only set if backend implements */
-      if (CLUTTER_STAGE_GET_CLASS (stage)->set_fullscreen)
-	CLUTTER_STAGE_GET_CLASS (stage)->set_fullscreen (stage, FALSE);
+      ClutterStageWindow *impl = CLUTTER_STAGE_WINDOW (priv->impl);
+      ClutterStageWindowIface *iface;
+
+      iface = CLUTTER_STAGE_WINDOW_GET_IFACE (impl);
+
+      /* Only set if backend implements.
+       * Also see clutter_stage_event() for setting priv->is_fullscreen
+       * on state change event.
+      */
+      if (iface->set_fullscreen)
+	iface->set_fullscreen (impl, FALSE);
     }
 }
 
@@ -722,14 +1010,20 @@ clutter_stage_set_user_resizable (ClutterStage *stage,
   priv = stage->priv;
 
   if (clutter_feature_available (CLUTTER_FEATURE_STAGE_USER_RESIZE)
-      && priv->is_user_resizable != resizable
-      && CLUTTER_STAGE_GET_CLASS (stage)->set_user_resize)
+      && priv->is_user_resizable != resizable)
     {
-      priv->is_user_resizable = resizable;
+      ClutterStageWindow *impl = CLUTTER_STAGE_WINDOW (priv->impl);
+      ClutterStageWindowIface *iface;
 
-      CLUTTER_STAGE_GET_CLASS (stage)->set_user_resize (stage, resizable);
+      iface = CLUTTER_STAGE_WINDOW_GET_IFACE (impl);
+      if (iface->set_user_resizable)
+        {
+          priv->is_user_resizable = resizable;
 
-      g_object_notify (G_OBJECT (stage), "user-resizable");
+          iface->set_user_resizable (impl, resizable);
+
+          g_object_notify (G_OBJECT (stage), "user-resizable");
+        }
     }
 }
 
@@ -767,12 +1061,18 @@ clutter_stage_show_cursor (ClutterStage *stage)
   priv = stage->priv;
   if (!priv->is_cursor_visible)
     {
-      priv->is_cursor_visible = TRUE;
+      ClutterStageWindow *impl = CLUTTER_STAGE_WINDOW (priv->impl);
+      ClutterStageWindowIface *iface;
 
-      if (CLUTTER_STAGE_GET_CLASS (stage)->set_cursor_visible)
-        CLUTTER_STAGE_GET_CLASS (stage)->set_cursor_visible (stage, TRUE);
+      iface = CLUTTER_STAGE_WINDOW_GET_IFACE (impl);
+      if (iface->set_cursor_visible)
+        {
+          priv->is_cursor_visible = TRUE;
 
-      g_object_notify (G_OBJECT (stage), "cursor-visible");
+          iface->set_cursor_visible (impl, TRUE);
+
+          g_object_notify (G_OBJECT (stage), "cursor-visible");
+        }
     }
 }
 
@@ -794,17 +1094,23 @@ clutter_stage_hide_cursor (ClutterStage *stage)
   priv = stage->priv;
   if (priv->is_cursor_visible)
     {
-      priv->is_cursor_visible = FALSE;
+      ClutterStageWindow *impl = CLUTTER_STAGE_WINDOW (priv->impl);
+      ClutterStageWindowIface *iface;
 
-      if (CLUTTER_STAGE_GET_CLASS (stage)->set_cursor_visible)
-	CLUTTER_STAGE_GET_CLASS (stage)->set_cursor_visible (stage, FALSE);
+      iface = CLUTTER_STAGE_WINDOW_GET_IFACE (impl);
+      if (iface->set_cursor_visible)
+        {
+          priv->is_cursor_visible = TRUE;
 
-      g_object_notify (G_OBJECT (stage), "cursor-visible");
+          iface->set_cursor_visible (impl, FALSE);
+
+          g_object_notify (G_OBJECT (stage), "cursor-visible");
+        }
     }
 }
 
 /**
- * clutter_stage_snapshot
+ * clutter_stage_read_pixels:
  * @stage: A #ClutterStage
  * @x: x coordinate of the first pixel that is read from stage
  * @y: y coordinate of the first pixel that is read from stage
@@ -813,27 +1119,78 @@ clutter_stage_hide_cursor (ClutterStage *stage)
  * @height: Height dimention of pixels to be read, or -1 for the
  *   entire stage height
  *
- * Gets a pixel based representation of the current rendered stage.
+ * Makes a screenshot of the stage in RGBA 8bit data, returns a
+ * linear buffer with @width * 4 as rowstride.
  *
- * Return value: pixel representation as a  #GdkPixbuf
- **/
-GdkPixbuf *
-clutter_stage_snapshot (ClutterStage *stage,
-			gint          x,
-			gint          y,
-			gint          width,
-			gint          height)
+ * The alpha data contained in the returned buffer is driver-dependent, 
+ * and not guaranteed to hold any sensible value.
+ *
+ * Return value: a pointer to newly allocated memory with the buffer
+ *   or %NULL if the read failed. Use g_free() on the returned data
+ *   to release the resources it has allocated.
+ */
+guchar *
+clutter_stage_read_pixels (ClutterStage *stage,
+                           gint          x,
+                           gint          y,
+                           gint          width,
+                           gint          height)
 {
-  ClutterStageClass *klass;
+  guchar *pixels;
+  guchar *temprow;
+  GLint   viewport[4];
+  gint    rowstride;
+  gint    stage_x, stage_y, stage_width, stage_height;
 
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), NULL);
+
+  /* according to glReadPixels documentation pixels outside the viewport are
+   * undefined, but no error should be provoked, thus this is probably unnneed.
+   */
   g_return_val_if_fail (x >= 0 && y >= 0, NULL);
 
-  klass = CLUTTER_STAGE_GET_CLASS (stage);
-  if (klass->draw_to_pixbuf)
-    return klass->draw_to_pixbuf (stage, x, y, width, height);
+  /* Force a redraw of the stage before reading back pixels */
+  clutter_stage_paint (CLUTTER_ACTOR (stage));
+  clutter_stage_ensure_current (stage);
 
-  return NULL;
+  glGetIntegerv (GL_VIEWPORT, viewport);
+  stage_x      = viewport[0];
+  stage_y      = viewport[1];
+  stage_width  = viewport[2];
+  stage_height = viewport[3];
+
+  if (width < 0 || width > stage_width)
+    width = stage_width;
+
+  if (height < 0 || height > stage_height)
+    height = stage_height;
+
+  rowstride = width * 4;
+
+  pixels  = g_malloc (height * rowstride);
+  temprow = g_malloc (rowstride);
+
+  /* check whether we need to read into a smaller temporary buffer */
+  glReadPixels (x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
+
+  /* vertically flip the buffer in-place */
+  for (y = 0; y < height / 2; y++)
+    {
+      if (y != height - y - 1) /* skip center row */
+        {
+          memcpy (temprow,
+                  pixels + y * rowstride, rowstride);
+          memcpy (pixels + y * rowstride, 
+                  pixels + (height - y - 1) * rowstride, rowstride);
+          memcpy (pixels + (height - y - 1) * rowstride,
+                  temprow,
+                  rowstride);
+        }
+    }
+
+  g_free (temprow);
+
+  return pixels;
 }
 
 /**
@@ -861,8 +1218,9 @@ clutter_stage_get_actor_at_pos (ClutterStage *stage,
  * @event: a #ClutterEvent
  *
  * This function is used to emit an event on the main stage.
+ *
  * You should rarely need to use this function, except for
- * synthetising events.
+ * synthetised events.
  *
  * Return value: the return value from the signal emission
  *
@@ -938,6 +1296,7 @@ clutter_stage_set_title (ClutterStage       *stage,
 			 const gchar        *title)
 {
   ClutterStagePrivate *priv;
+  ClutterStageWindow *impl;
 
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
@@ -946,8 +1305,9 @@ clutter_stage_set_title (ClutterStage       *stage,
   g_free (priv->title);
   priv->title = g_strdup (title);
 
-  if (CLUTTER_STAGE_GET_CLASS (stage)->set_title)
-    CLUTTER_STAGE_GET_CLASS (stage)->set_title (stage, priv->title);
+  impl = CLUTTER_STAGE_WINDOW (priv->impl);
+  if (CLUTTER_STAGE_WINDOW_GET_IFACE(impl)->set_title != NULL)
+    CLUTTER_STAGE_WINDOW_GET_IFACE (impl)->set_title (impl, priv->title);
 
   g_object_notify (G_OBJECT (stage), "title");
 }
@@ -1349,3 +1709,163 @@ clutter_fog_get_type (void)
 
   return our_type;
 }
+
+/**
+ * clutter_stage_new:
+ *
+ * Creates a new, non-default stage. A non-default stage is a new
+ * top-level actor which can be used as another container. It works
+ * exactly like the default stage, but while clutter_stage_get_default()
+ * will always return the same instance, you will have to keep a pointer
+ * to any #ClutterStage returned by clutter_stage_create().
+ *
+ * The ability to support multiple stages depends on the current
+ * backend. Use clutter_feature_available() and
+ * %CLUTTER_FEATURE_STAGE_MULTIPLE to check at runtime whether a
+ * backend supports multiple stages.
+ *
+ * Return value: a new stage, or %NULL if the default backend does
+ *   not support multiple stages. Use clutter_actor_destroy() to
+ *   programmatically close the returned stage.
+ *
+ * Since: 0.8
+ */
+ClutterActor *
+clutter_stage_new (void)
+{
+  if (!clutter_feature_available (CLUTTER_FEATURE_STAGE_MULTIPLE))
+    {
+      g_warning ("Unable to create a new stage: the %s backend does not "
+                 "support multiple stages.",
+                 CLUTTER_FLAVOUR);
+      return NULL;
+    }
+
+  /* The stage manager will grab the floating reference when the stage
+     is added to it in the constructor */
+  return g_object_new (CLUTTER_TYPE_STAGE, NULL);
+}
+
+/**
+ * clutter_stage_ensure_current:
+ * @stage: the #ClutterStage
+ *
+ * This function essentially makes sure the right GL context is
+ * current for the passed stage. It is not intended to
+ * be used by applications.
+ *
+ * Since: 0.8
+ */
+void
+clutter_stage_ensure_current (ClutterStage *stage)
+{
+  ClutterMainContext *ctx = clutter_context_get_default ();
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  _clutter_backend_ensure_context (ctx->backend, stage);
+}
+
+static gboolean
+redraw_update_idle (gpointer user_data)
+{
+  ClutterStage *stage = user_data;
+  ClutterStagePrivate *priv = stage->priv;
+
+  if (priv->update_idle)
+    {
+      g_source_remove (priv->update_idle);
+      priv->update_idle = 0;
+    }
+
+  CLUTTER_NOTE (MULTISTAGE, "redrawing via idle for stage:%p", stage);
+  clutter_redraw (stage);
+
+  return FALSE;
+}
+
+/**
+ * clutter_stage_queue_redraw:
+ * @stage: the #ClutterStage
+ *
+ * Queues a redraw for the passed stage.
+ *
+ * <note>Applications should call clutter_actor_queue_redraw() and not
+ * this function.</note>
+ *
+ * Since: 0.8
+ */
+void
+clutter_stage_queue_redraw (ClutterStage *stage)
+{
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  if (!stage->priv->update_idle)
+    {
+      CLUTTER_TIMESTAMP (SCHEDULER, "Adding idle source for stage: %p", stage);
+
+      /* FIXME: weak_ref self in case we dissapear before paint? */
+      stage->priv->update_idle =
+        clutter_threads_add_idle_full (CLUTTER_PRIORITY_REDRAW,
+                                       redraw_update_idle,
+                                       stage,
+                                       NULL);
+    }
+}
+
+/**
+ * clutter_stage_is_default:
+ * @stage: a #ClutterStage
+ *
+ * Checks if @stage is the default stage, or an instance created using
+ * clutter_stage_new() but internally using the same implementation.
+ *
+ * Return value: %TRUE if the passed stage is the default one
+ *
+ * Since: 0.8
+ */
+gboolean
+clutter_stage_is_default (ClutterStage *stage)
+{
+  ClutterStageWindow *impl;
+
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
+
+  if (CLUTTER_ACTOR (stage) == clutter_stage_get_default ())
+    return TRUE;
+
+  impl = _clutter_stage_get_window (stage);
+  if (impl == _clutter_stage_get_default_window ())
+    return TRUE;
+
+  return FALSE;
+}
+
+void
+_clutter_stage_set_window (ClutterStage       *stage,
+                           ClutterStageWindow *stage_window)
+{
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+  g_return_if_fail (CLUTTER_IS_STAGE_WINDOW (stage_window));
+
+  if (stage->priv->impl)
+    g_object_unref (stage->priv->impl);
+
+  stage->priv->impl = CLUTTER_ACTOR (stage_window);
+}
+
+ClutterStageWindow *
+_clutter_stage_get_window (ClutterStage *stage)
+{
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), NULL);
+
+  return CLUTTER_STAGE_WINDOW (stage->priv->impl);
+}
+
+ClutterStageWindow *
+_clutter_stage_get_default_window (void)
+{
+  ClutterActor *stage = clutter_stage_get_default ();
+
+  return _clutter_stage_get_window (CLUTTER_STAGE (stage));
+}
