@@ -25,19 +25,58 @@
 
 /**
  * SECTION:clutter-timeline
- * @short_description: A class for based events
+ * @short_description: A class for time-based events
  *
  * #ClutterTimeline is a base class for managing time based events such
  * as animations.
  *
  * Every timeline shares the same #ClutterTimeoutPool to decrease the
- * possibility of starvating the main loop when using many timelines
+ * possibility of starving the main loop when using many timelines
  * at the same time; this might cause problems if you are also using
  * a library making heavy use of threads with no GLib main loop integration.
+ *
  * In that case you might disable the common timeline pool by setting
  * the %CLUTTER_TIMELINE=no-pool environment variable prior to launching
  * your application.
  *
+ * One way to visualise a timeline is as a path with marks along its length.
+ * When creating a timeline of @n_frames via clutter_timeline_new(), then the
+ * number of frames can be seen as the paths length, and each unit of length
+ * (each frame) is delimited by a mark.
+ *
+ * For a non looping timeline there will be (n_frames + 1) marks along its
+ * length. For a looping timeline, the two ends are joined with one mark.
+ * Technically this mark represents two discrete frame numbers, but for a
+ * looping timeline the start and end frame numbers are considered equivalent.
+ *
+ * When you create a timeline it starts with
+ * clutter_timeline_get_current_frame() == 0.
+ *
+ * After starting a timeline, the first timeout is for current_frame_num == 1
+ * (Notably it isn't 0 since there is a delay before the first timeout signals
+ * so re-asserting the starting frame (0) wouldn't make sense.)
+ * Notably, this implies that actors you intend to be affected by the
+ * timeline's progress, should be manually primed/positioned for frame 0 which
+ * will be displayed before the first timeout. (If you are not careful about
+ * this point you will likely see flashes of incorrect actor state in your
+ * program)
+ *
+ * For a non looping timeline the last timeout would be for
+ * current_frame_num == @n_frames
+ *
+ * For a looping timeline the timeout for current_frame_num == @n_frames would
+ * be followed by a timeout for current_frame_num == 1 (remember frame 0 is
+ * considered == frame (@n_frames)).
+ *
+ * There may be times when a system is not able to meet the frame rate
+ * requested for a timeline, and in this case the frame number will be
+ * interpolated at the next timeout event. The interpolation is calculated from
+ * the time that the timeline was started, not from the time of the last
+ * timeout, so a given timeline should basically elapse in the same - real
+ * world - time on any given system. An invariable here though is that
+ * current_frame_num == @n_frames will always be signaled, but notably frame 1
+ * can be interpolated past and so never signaled.
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,7 +94,6 @@
 G_DEFINE_TYPE (ClutterTimeline, clutter_timeline, G_TYPE_OBJECT);
 
 #define FPS_TO_INTERVAL(f) (1000 / (f))
-#define CLUTTER_TIMELINE_PRIORITY       (G_PRIORITY_DEFAULT + 30)
 
 struct _ClutterTimelinePrivate
 {
@@ -73,13 +111,21 @@ struct _ClutterTimelinePrivate
 
   gint skipped_frames;
 
-  gulong last_frame_msecs;
-  gulong start_frame_secs;
+  GTimeVal prev_frame_timeval;
   guint  msecs_delta;
 
+  GHashTable *markers_by_frame;
+  GHashTable *markers_by_name;
+
   guint loop : 1;
 };
 
+typedef struct {
+  gchar *name;
+  guint frame_num;
+  GQuark quark;
+} TimelineMarker;
+
 enum
 {
   PROP_0,
@@ -98,6 +144,7 @@ enum
   STARTED,
   PAUSED,
   COMPLETED,
+  MARKER_REACHED,
 
   LAST_SIGNAL
 };
@@ -121,7 +168,7 @@ timeline_pool_init (void)
         }
       else
         {
-          timeline_pool = clutter_timeout_pool_new (CLUTTER_TIMELINE_PRIORITY);
+          timeline_pool = clutter_timeout_pool_new (CLUTTER_PRIORITY_TIMELINE);
           timeline_use_pool = TRUE;
         }
     }
@@ -144,9 +191,9 @@ timeout_add (guint          interval,
     }
   else
     {
-      res = clutter_threads_add_timeout_full (CLUTTER_TIMELINE_PRIORITY,
-                                              interval,
-                                              func, data, notify);
+      res = clutter_threads_add_frame_source_full (CLUTTER_PRIORITY_TIMELINE,
+						   interval,
+						   func, data, notify);
     }
 
   return res;
@@ -164,6 +211,31 @@ timeout_remove (guint tag)
     g_source_remove (tag);
 }
 
+static TimelineMarker *
+timeline_marker_new (const gchar *name,
+                     guint        frame_num)
+{
+  TimelineMarker *marker = g_slice_new0 (TimelineMarker);
+
+  marker->name = g_strdup (name);
+  marker->quark = g_quark_from_string (marker->name);
+  marker->frame_num = frame_num;
+
+  return marker;
+}
+
+static void
+timeline_marker_free (gpointer data)
+{
+  if (G_LIKELY (data))
+    {
+      TimelineMarker *marker = data;
+
+      g_free (marker->name);
+      g_slice_free (TimelineMarker, marker);
+    }
+}
+
 /* Object */
 
 static void
@@ -201,7 +273,7 @@ clutter_timeline_set_property (GObject      *object,
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
-  }
+    }
 }
 
 static void
@@ -245,6 +317,11 @@ clutter_timeline_get_property (GObject    *object,
 static void
 clutter_timeline_finalize (GObject *object)
 {
+  ClutterTimelinePrivate *priv = CLUTTER_TIMELINE (object)->priv;
+
+  g_hash_table_destroy (priv->markers_by_frame);
+  g_hash_table_destroy (priv->markers_by_name);
+
   G_OBJECT_CLASS (clutter_timeline_parent_class)->finalize (object);
 }
 
@@ -358,7 +435,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                                                       CLUTTER_PARAM_READWRITE));
   /**
    * ClutterTimeline:direction:
-   * 
+   *
    * The direction of the timeline, either %CLUTTER_TIMELINE_FORWARD or
    * %CLUTTER_TIMELINE_BACKWARD.
    *
@@ -376,7 +453,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
   /**
    * ClutterTimeline::new-frame:
    * @timeline: the timeline which received the signal
-   * @frame_num: the number of the new frame between 0 and 
+   * @frame_num: the number of the new frame between 0 and
    * ClutterTimeline:num-frames
    *
    * The ::new-frame signal is emitted each time a new frame in the
@@ -437,18 +514,66 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
 		  NULL, NULL,
 		  clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
+  /**
+   * ClutterTimeline::marker-reached:
+   * @timeline: the #ClutterTimeline which received the signal
+   * @marker_name: the name of the marker reached
+   * @frame_num: the frame number
+   *
+   * The ::marker-reached signal is emitted each time a timeline
+   * reaches a marker set with clutter_timeline_add_marker_at_frame()
+   * or clutter_timeline_add_marker_at_time(). This signal is
+   * detailed with the name of the marker as well, so it is
+   * possible to connect a callback to the ::marker-reached signal
+   * for a specific marker with:
+   *
+   * <informalexample><programlisting>
+   *   clutter_timeline_add_marker_at_frame (timeline, "foo", 24);
+   *   clutter_timeline_add_marker_at_frame (timeline, "bar", 48);
+   *
+   *   g_signal_connect (timeline, "marker-reached",
+   *                     G_CALLBACK (each_marker_reached), NULL);
+   *   g_signal_connect (timeline, "marker-reached::foo",
+   *                     G_CALLBACK (foo_marker_reached), NULL);
+   *   g_signal_connect (timeline, "marker-reached::bar",
+   *                     G_CALLBACK (bar_marker_reached), NULL);
+   * </programlisting></informalexample>
+   *
+   * In the example, the first callback will be invoked for both
+   * the "foo" and "bar" marker, while the second and third callbacks
+   * will be invoked for the "foo" or "bar" markers, respectively.
+   *
+   * Since: 0.8
+   */
+  timeline_signals[MARKER_REACHED] =
+    g_signal_new ("marker-reached",
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED | G_SIGNAL_NO_HOOKS,
+                  G_STRUCT_OFFSET (ClutterTimelineClass, marker_reached),
+                  NULL, NULL,
+                  clutter_marshal_VOID__STRING_INT,
+                  G_TYPE_NONE, 2,
+                  G_TYPE_STRING,
+                  G_TYPE_INT);
 }
 
 static void
 clutter_timeline_init (ClutterTimeline *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-					    CLUTTER_TYPE_TIMELINE,
-					    ClutterTimelinePrivate);
+  ClutterTimelinePrivate *priv;
 
-  self->priv->fps = clutter_get_default_frame_rate ();
-  self->priv->n_frames = 0;
-  self->priv->msecs_delta = 0;
+  self->priv = priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+                                                   CLUTTER_TYPE_TIMELINE,
+                                                   ClutterTimelinePrivate);
+
+  priv->fps = clutter_get_default_frame_rate ();
+  priv->n_frames = 0;
+  priv->msecs_delta = 0;
+
+  priv->markers_by_frame = g_hash_table_new (NULL, NULL);
+  priv->markers_by_name = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                 NULL,
+                                                 timeline_marker_free);
 }
 
 static gboolean
@@ -457,29 +582,11 @@ timeline_timeout_func (gpointer data)
   ClutterTimeline        *timeline = data;
   ClutterTimelinePrivate *priv;
   GTimeVal                timeval;
-  gint                    n_frames;
+  guint                   n_frames;
   gulong                  msecs;
-  gboolean                retval = TRUE;
 
   priv = timeline->priv;
 
-  if (!timeline)
-    {
-      CLUTTER_NOTE (SCHEDULER,
-                    "The timeline [%p] has been disposed",
-                    timeline);
-      return FALSE;
-    }
-
-  if (!priv->timeout_id)
-    {
-      CLUTTER_NOTE (SCHEDULER,
-                    "The timeline [%p] has been removed",
-                    timeline);
-
-      return FALSE;
-    }
-
   g_object_ref (timeline);
 
   /* Figure out potential frame skips */
@@ -489,57 +596,32 @@ timeline_timeout_func (gpointer data)
                      timeline,
                      priv->current_frame_num);
 
-  /* Fire off signal */
-  g_signal_emit (timeline, timeline_signals[NEW_FRAME], 0,
-                 priv->current_frame_num);
-
-  /* Signal removes source ? */
-  if (!priv->timeout_id)
+  if (!priv->prev_frame_timeval.tv_sec)
     {
-      g_object_unref (timeline);
-      return FALSE;
+      CLUTTER_NOTE (SCHEDULER,
+                    "Timeline [%p] recieved timeout before being initialised!",
+                    timeline);
+      priv->prev_frame_timeval = timeval;
     }
 
-  if (priv->last_frame_msecs)
-    {
-      /* Check time diff from out last call and adjust number
-       * of frames to advance accordingly.
-      */
-      msecs = ((timeval.tv_sec - priv->start_frame_secs) * 1000)
-              + (timeval.tv_usec / 1000);
-      n_frames = (msecs - priv->last_frame_msecs)
-                 / (1000 / priv->fps);
-      priv->msecs_delta = msecs - priv->last_frame_msecs;
+  /* Interpolate the current frame based on the timeval of the
+   * previous frame */
+  msecs = (timeval.tv_sec - priv->prev_frame_timeval.tv_sec) * 1000;
+  msecs += (timeval.tv_usec - priv->prev_frame_timeval.tv_usec) / 1000;
+  priv->msecs_delta = msecs;
+  n_frames = msecs / (1000 / priv->fps);
+  if (n_frames == 0)
+    n_frames = 1;
 
-      if (n_frames <= 0)
-        {
-          n_frames = 1;
-          priv->skipped_frames = 0;
-        }
-      else if (n_frames > 1)
-	{
-	  CLUTTER_TIMESTAMP (SCHEDULER,
-			     "Timeline [%p], skipping %d frames\n",
-			     timeline,
-                             n_frames);
+  priv->skipped_frames = n_frames - 1;
 
-          priv->skipped_frames = n_frames - 1;
-	}
-      else
-        priv->skipped_frames = 0;
-    }
-  else
-    {
-      /* First frame, set up timings.*/
-      priv->start_frame_secs = timeval.tv_sec;
-      priv->skipped_frames   = 0;
-      priv->msecs_delta      = 0;
+  if (priv->skipped_frames)
+    CLUTTER_TIMESTAMP (SCHEDULER,
+                       "Timeline [%p], skipping %d frames\n",
+                       timeline,
+                       priv->skipped_frames);
 
-      msecs     = timeval.tv_usec / 1000;
-      n_frames  = 1;
-    }
-
-  priv->last_frame_msecs = msecs;
+  priv->prev_frame_timeval = timeval;
 
   /* Advance frames */
   if (priv->direction == CLUTTER_TIMELINE_FORWARD)
@@ -547,21 +629,79 @@ timeline_timeout_func (gpointer data)
   else
     priv->current_frame_num -= n_frames;
 
-  /* Handle loop or stop */
-  if (((priv->direction == CLUTTER_TIMELINE_FORWARD) &&
-       (priv->current_frame_num > priv->n_frames)) ||
+  /* If we have not reached the end of the timeline: */
+  if (!(
+      ((priv->direction == CLUTTER_TIMELINE_FORWARD) &&
+       (priv->current_frame_num >= priv->n_frames)) ||
       ((priv->direction == CLUTTER_TIMELINE_BACKWARD) &&
-       (priv->current_frame_num < 0)))
+       (priv->current_frame_num <= 0))
+       ))
     {
-      if (priv->direction == CLUTTER_TIMELINE_FORWARD)
+      gint i;
+
+      /* Fire off signal */
+      g_signal_emit (timeline, timeline_signals[NEW_FRAME], 0,
+                     priv->current_frame_num);
+
+      for (i = priv->skipped_frames; i >= 0; i--)
         {
-          priv->current_frame_num = priv->n_frames;
-        }
-      else if (priv->direction == CLUTTER_TIMELINE_BACKWARD)
-        {
-          priv->current_frame_num = 0;
+          gint frame_num = priv->current_frame_num - i;
+          GSList *markers, *l;
+
+          markers = g_hash_table_lookup (priv->markers_by_frame,
+                                         GUINT_TO_POINTER (frame_num));
+          for (l = markers; l; l = l->next)
+            {
+              TimelineMarker *marker = l->data;
+
+              CLUTTER_NOTE (SCHEDULER, "Marker `%s' reached", marker->name);
+
+              g_signal_emit (timeline, timeline_signals[MARKER_REACHED],
+                             marker->quark,
+                             marker->name,
+                             marker->frame_num);
+            }
         }
 
+      /* Signal pauses timeline ? */
+      if (!priv->timeout_id)
+        {
+          g_object_unref (timeline);
+          return FALSE;
+        }
+
+      g_object_unref (timeline);
+      return TRUE;
+    }
+  else
+    {
+      /* Handle loop or stop */
+      ClutterTimelineDirection saved_direction = priv->direction;
+      guint overflow_frame_num = priv->current_frame_num;
+      gint end_frame;
+
+      /* In case the signal handlers want to take a peek... */
+      if (priv->direction == CLUTTER_TIMELINE_FORWARD)
+        priv->current_frame_num = priv->n_frames;
+      else if (priv->direction == CLUTTER_TIMELINE_BACKWARD)
+        priv->current_frame_num = 0;
+
+      end_frame = priv->current_frame_num;
+
+      /* Fire off signal */
+      g_signal_emit (timeline, timeline_signals[NEW_FRAME], 0,
+                     priv->current_frame_num);
+
+      /* Did the signal handler modify the current_frame_num */
+      if (priv->current_frame_num != end_frame)
+        {
+          g_object_unref (timeline);
+          return TRUE;
+        }
+
+      /* Note: If the new-frame signal handler paused the timeline
+       * on the last frame we will still go ahead and send the
+       * completed signal */
       CLUTTER_NOTE (SCHEDULER,
                     "Timeline [%p] completed (cur: %d, tot: %d, drop: %d)",
                     timeline,
@@ -569,40 +709,86 @@ timeline_timeout_func (gpointer data)
                     priv->n_frames,
                     n_frames - 1);
 
-      /* if we skipped some frame to get here let's see whether we still need
-       * to emit the last new-frame signal with the last frame
-       */
-      if (n_frames > 1)
-	g_signal_emit (timeline, timeline_signals[NEW_FRAME], 0,
-                       priv->current_frame_num);
+      if (!priv->loop && priv->timeout_id)
+        {
+          /* We remove the timeout now, so that the completed signal handler
+           * may choose to re-start the timeline
+           *
+           * ** Perhaps we should remove this earlier, and regardless
+           * of priv->loop. Are we limiting the things that could be done in
+           * the above new-frame signal handler */
+          timeout_remove (priv->timeout_id);
+          priv->timeout_id = 0;
+        }
+
+      g_signal_emit (timeline, timeline_signals[COMPLETED], 0);
+
+      /* Again check to see if the user has manually played with
+       * current_frame_num, before we finally stop or loop the timeline */
+
+      if (priv->current_frame_num != end_frame &&
+          !(/* Except allow moving from frame 0 -> n_frame (or vica-versa)
+               since these are considered equivalent */
+            (priv->current_frame_num == 0 && end_frame == priv->n_frames) ||
+            (priv->current_frame_num == priv->n_frames && end_frame == 0)
+          ))
+        {
+          g_object_unref (timeline);
+          return TRUE;
+        }
 
       if (priv->loop)
-	{
-	  g_signal_emit (timeline, timeline_signals[COMPLETED], 0);
-	  clutter_timeline_rewind (timeline);
+        {
+          /* We try and interpolate smoothly around a loop */
+          if (saved_direction == CLUTTER_TIMELINE_FORWARD)
+            priv->current_frame_num = overflow_frame_num - priv->n_frames;
+          else
+            priv->current_frame_num = priv->n_frames + overflow_frame_num;
 
-          retval = TRUE;
-	}
-      else
-	{
-          if (priv->timeout_id)
+          /* Or if the direction changed, we try and bounce */
+          if (priv->direction != saved_direction)
             {
-              timeout_remove (priv->timeout_id);
-              priv->timeout_id = 0;
+              priv->current_frame_num = priv->n_frames
+                                        - priv->current_frame_num;
             }
 
-          priv->last_frame_msecs = 0;
-
-          g_signal_emit (timeline, timeline_signals[COMPLETED], 0);
+          g_object_unref (timeline);
+          return TRUE;
+        }
+      else
+        {
           clutter_timeline_rewind (timeline);
 
-	  retval = FALSE;
-	}
+          priv->prev_frame_timeval.tv_sec = 0;
+          priv->prev_frame_timeval.tv_usec = 0;
+
+          g_object_unref (timeline);
+          return FALSE;
+        }
     }
+}
 
-  g_object_unref (timeline);
+static guint
+timeline_timeout_add (ClutterTimeline *timeline,
+                      guint          interval,
+                      GSourceFunc    func,
+                      gpointer       data,
+                      GDestroyNotify notify)
+{
+  ClutterTimelinePrivate *priv;
+  GTimeVal timeval;
 
-  return retval;
+  priv = timeline->priv;
+
+  if (priv->prev_frame_timeval.tv_sec == 0)
+    {
+      g_get_current_time (&timeval);
+      priv->prev_frame_timeval = timeval;
+    }
+  priv->skipped_frames   = 0;
+  priv->msecs_delta      = 0;
+
+  return timeout_add (interval, func, data, notify);
 }
 
 static gboolean
@@ -613,9 +799,10 @@ delay_timeout_func (gpointer data)
 
   priv->delay_id = 0;
 
-  priv->timeout_id = timeout_add (FPS_TO_INTERVAL (priv->fps),
-                                  timeline_timeout_func,
-                                  timeline, NULL);
+  priv->timeout_id = timeline_timeout_add (timeline,
+                                           FPS_TO_INTERVAL (priv->fps),
+                                           timeline_timeout_func,
+                                           timeline, NULL);
 
   g_signal_emit (timeline, timeline_signals[STARTED], 0);
 
@@ -651,9 +838,10 @@ clutter_timeline_start (ClutterTimeline *timeline)
     }
   else
     {
-      priv->timeout_id = timeout_add (FPS_TO_INTERVAL (priv->fps),
-                                      timeline_timeout_func,
-                                      timeline, NULL);
+      priv->timeout_id = timeline_timeout_add (timeline,
+                                               FPS_TO_INTERVAL (priv->fps),
+                                               timeline_timeout_func,
+                                               timeline, NULL);
 
       g_signal_emit (timeline, timeline_signals[STARTED], 0);
     }
@@ -686,7 +874,8 @@ clutter_timeline_pause (ClutterTimeline *timeline)
       priv->timeout_id = 0;
     }
 
-  priv->last_frame_msecs = 0;
+  priv->prev_frame_timeval.tv_sec = 0;
+  priv->prev_frame_timeval.tv_usec = 0;
 
   g_signal_emit (timeline, timeline_signals[PAUSED], 0);
 }
@@ -904,9 +1093,10 @@ clutter_timeline_set_speed (ClutterTimeline *timeline,
         {
           timeout_remove (priv->timeout_id);
 
-          priv->timeout_id = timeout_add (FPS_TO_INTERVAL (priv->fps),
-                                          timeline_timeout_func,
-                                          timeline, NULL);
+          priv->timeout_id = timeline_timeout_add (timeline,
+                                                   FPS_TO_INTERVAL (priv->fps),
+                                                   timeline_timeout_func,
+                                                   timeline, NULL);
         }
 
       g_object_notify (G_OBJECT (timeline), "fps");
@@ -983,7 +1173,7 @@ clutter_timeline_clone (ClutterTimeline *timeline)
  *
  * Creates a new #ClutterTimeline with a duration of @msecs using
  * the value of the ClutterTimeline:fps property to compute the
- * equivalent number of frames. 
+ * equivalent number of frames.
  *
  * Return value: the newly created #ClutterTimeline
  *
@@ -1053,7 +1243,6 @@ clutter_timeline_set_delay (ClutterTimeline *timeline,
   ClutterTimelinePrivate *priv;
 
   g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
-  g_return_if_fail (msecs > 0);
 
   priv = timeline->priv;
 
@@ -1153,17 +1342,19 @@ ClutterFixed
 clutter_timeline_get_progressx (ClutterTimeline *timeline)
 {
   ClutterTimelinePrivate *priv;
+  ClutterFixed progress;
 
   g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), 0);
 
   priv = timeline->priv;
 
-  if (priv->direction == CLUTTER_TIMELINE_FORWARD)
-    return CLUTTER_FIXED_DIV (CLUTTER_INT_TO_FIXED (priv->current_frame_num),
-                              CLUTTER_INT_TO_FIXED (priv->n_frames));
-  else
-    return CLUTTER_FIXED_DIV (CLUTTER_INT_TO_FIXED (priv->n_frames),
-                              CLUTTER_INT_TO_FIXED (priv->current_frame_num));
+  progress = clutter_qdivx (CLUTTER_INT_TO_FIXED (priv->current_frame_num),
+			    CLUTTER_INT_TO_FIXED (priv->n_frames));
+
+  if (priv->direction == CLUTTER_TIMELINE_BACKWARD)
+    progress = CFX_ONE - progress;
+
+  return progress;
 }
 
 /**
@@ -1256,3 +1447,268 @@ clutter_timeline_get_delta (ClutterTimeline *timeline,
 
   return priv->skipped_frames + 1;
 }
+
+static inline void
+clutter_timeline_add_marker_internal (ClutterTimeline *timeline,
+                                      const gchar     *marker_name,
+                                      guint            frame_num)
+{
+  ClutterTimelinePrivate *priv = timeline->priv;
+  TimelineMarker *marker;
+  GSList *markers;
+
+  marker = g_hash_table_lookup (priv->markers_by_name, marker_name);
+  if (G_UNLIKELY (marker))
+    {
+      g_warning ("A marker named `%s' already exists on frame %d",
+                 marker->name,
+                 marker->frame_num);
+      return;
+    }
+
+  marker = timeline_marker_new (marker_name, frame_num);
+  g_hash_table_insert (priv->markers_by_name, marker->name, marker);
+
+  markers = g_hash_table_lookup (priv->markers_by_frame,
+                                 GUINT_TO_POINTER (frame_num));
+  if (!markers)
+    {
+      markers = g_slist_prepend (NULL, marker);
+      g_hash_table_insert (priv->markers_by_frame,
+                           GUINT_TO_POINTER (frame_num),
+                           markers);
+    }
+  else
+    {
+      markers = g_slist_prepend (markers, marker);
+      g_hash_table_replace (priv->markers_by_frame,
+                            GUINT_TO_POINTER (frame_num),
+                            markers);
+    }
+}
+
+/**
+ * clutter_timeline_add_marker_at_frame:
+ * @timeline: a #ClutterTimeline
+ * @marker_name: the unique name for this marker
+ * @frame_num: the marker's frame
+ *
+ * Adds a named marker at @frame_num. Markers are unique string identifiers
+ * for a specific frame. Once @timeline reaches @frame_num, it will emit
+ * a ::marker-reached signal for each marker attached to that frame.
+ *
+ * A marker can be removed with clutter_timeline_remove_marker(). The
+ * timeline can be advanced to a marker using
+ * clutter_timeline_advance_to_marker().
+ *
+ * Since: 0.8
+ */
+void
+clutter_timeline_add_marker_at_frame (ClutterTimeline *timeline,
+                                      const gchar     *marker_name,
+                                      guint            frame_num)
+{
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+  g_return_if_fail (marker_name != NULL);
+  g_return_if_fail (frame_num <= clutter_timeline_get_n_frames (timeline));
+
+  clutter_timeline_add_marker_internal (timeline, marker_name, frame_num);
+}
+
+/**
+ * clutter_timeline_add_marker_at_time:
+ * @timeline: a #ClutterTimeline
+ * @marker_name: the unique name for this marker
+ * @msecs: position of the marker in milliseconds
+ *
+ * Time-based variant of clutter_timeline_add_marker_at_frame().
+ *
+ * Adds a named marker at @msecs.
+ *
+ * Since: 0.8
+ */
+void
+clutter_timeline_add_marker_at_time (ClutterTimeline *timeline,
+                                     const gchar     *marker_name,
+                                     guint            msecs)
+{
+  guint frame_num;
+
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+  g_return_if_fail (marker_name != NULL);
+  g_return_if_fail (msecs <= clutter_timeline_get_duration (timeline));
+
+  frame_num = msecs * timeline->priv->fps / 1000;
+
+  clutter_timeline_add_marker_internal (timeline, marker_name, frame_num);
+}
+
+/**
+ * clutter_timeline_list_markers:
+ * @timeline: a #ClutterTimeline
+ * @frame_num: the frame number to check, or -1
+ * @n_markers: the number of markers returned
+ *
+ * Retrieves the list of markers at @frame_num. If @frame_num is a
+ * negative integer, all the markers attached to @timeline will be
+ * returned.
+ *
+ * Return value: a newly allocated, %NULL terminated string array
+ *   containing the names of the markers. Use g_strfreev() when
+ *   done.
+ *
+ * Since: 0.8
+ */
+gchar **
+clutter_timeline_list_markers (ClutterTimeline *timeline,
+                               gint             frame_num,
+                               gsize           *n_markers)
+{
+  ClutterTimelinePrivate *priv;
+  gchar **retval = NULL;
+  gsize i;
+
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), NULL);
+
+  priv = timeline->priv;
+
+  if (frame_num < 0)
+    {
+      GList *markers, *l;
+
+      markers = g_hash_table_get_keys (priv->markers_by_name);
+      retval = g_new0 (gchar*, g_list_length (markers) + 1);
+
+      for (i = 0, l = markers; l != NULL; i++, l = l->next)
+        retval[i] = g_strdup (l->data);
+
+      g_list_free (markers);
+    }
+  else
+    {
+      GSList *markers, *l;
+
+      markers = g_hash_table_lookup (priv->markers_by_frame,
+                                     GUINT_TO_POINTER (frame_num));
+      retval = g_new0 (gchar*, g_slist_length (markers) + 1);
+
+      for (i = 0, l = markers; l != NULL; i++, l = l->next)
+        retval[i] = g_strdup (l->data);
+    }
+
+  if (n_markers)
+    *n_markers = i;
+
+  return retval;
+}
+
+/**
+ * clutter_timeline_advance_to_marker:
+ * @timeline: a #ClutterTimeline
+ * @marker_name: the name of the marker
+ *
+ * Advances @timeline to the frame of the given @marker_name.
+ *
+ * Since: 0.8
+ */
+void
+clutter_timeline_advance_to_marker (ClutterTimeline *timeline,
+                                    const gchar     *marker_name)
+{
+  ClutterTimelinePrivate *priv;
+  TimelineMarker *marker;
+
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+  g_return_if_fail (marker_name != NULL);
+
+  priv = timeline->priv;
+
+  marker = g_hash_table_lookup (priv->markers_by_name, marker_name);
+  if (!marker)
+    {
+      g_warning ("No marker named `%s' found.", marker_name);
+      return;
+    }
+
+  clutter_timeline_advance (timeline, marker->frame_num);
+}
+
+/**
+ * clutter_timeline_remove_marker:
+ * @timeline: a #ClutterTimeline
+ * @marker_name: the name of the marker to remove
+ *
+ * Removes @marker_name, if found, from @timeline.
+ *
+ * Since: 0.8
+ */
+void
+clutter_timeline_remove_marker (ClutterTimeline *timeline,
+                                const gchar     *marker_name)
+{
+  ClutterTimelinePrivate *priv;
+  TimelineMarker *marker;
+  GSList *markers;
+
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+  g_return_if_fail (marker_name != NULL);
+
+  priv = timeline->priv;
+
+  marker = g_hash_table_lookup (priv->markers_by_name, marker_name);
+  if (!marker)
+    {
+      g_warning ("No marker named `%s' found.", marker_name);
+      return;
+    }
+
+  /* remove from the list of markers at the same frame */
+  markers = g_hash_table_lookup (priv->markers_by_frame,
+                                 GUINT_TO_POINTER (marker->frame_num));
+  if (G_LIKELY (markers))
+    {
+      markers = g_slist_remove (markers, marker);
+      if (!markers)
+        {
+          /* no markers left, remove the slot */
+          g_hash_table_remove (priv->markers_by_frame,
+                               GUINT_TO_POINTER (marker->frame_num));
+        }
+      else
+        g_hash_table_replace (priv->markers_by_frame,
+                              GUINT_TO_POINTER (marker->frame_num),
+                              markers);
+    }
+  else
+    {
+      /* uh-oh, dangling marker; this should never happen */
+      g_warning ("Dangling marker %s at frame %d",
+                 marker->name,
+                 marker->frame_num);
+    }
+
+  /* this will take care of freeing the marker as well */
+  g_hash_table_remove (priv->markers_by_name, marker_name);
+}
+
+/**
+ * clutter_timeline_has_marker:
+ * @timeline: a #ClutterTimeline
+ * @marker_name: the name of the marker
+ *
+ * Checks whether @timeline has a marker set with the given name.
+ *
+ * Return value: %TRUE if the marker was found
+ *
+ * Since: 0.8
+ */
+gboolean
+clutter_timeline_has_marker (ClutterTimeline *timeline,
+                             const gchar     *marker_name)
+{
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), FALSE);
+  g_return_val_if_fail (marker_name != NULL, FALSE);
+
+  return NULL != g_hash_table_lookup (timeline->priv->markers_by_name,
+                                      marker_name);
+}
