@@ -38,7 +38,10 @@ G_BEGIN_DECLS
 #define CLUTTER_TYPE_VERTEX     (clutter_vertex_get_type ())
 
 /* Forward delarations to avoid header catch 22's */
-typedef struct _ClutterActor         ClutterActor;
+typedef struct _ClutterActor            ClutterActor;
+typedef struct _ClutterStage            ClutterStage;
+typedef struct _ClutterContainer        ClutterContainer; /* dummy */
+typedef struct _ClutterChildMeta        ClutterChildMeta;
 
 /**
  * ClutterGravity:
@@ -167,6 +170,20 @@ typedef enum { /*< prefix=CLUTTER_ROTATE >*/
   CLUTTER_ROTATE_CCW
 } ClutterRotateDirection;
 
+/**
+ * ClutterRequestMode:
+ * @CLUTTER_REQUEST_HEIGHT_FOR_WIDTH: Height for width requests
+ * @CLUTTER_REQUEST_WIDTH_FOR_HEIGHT: Width for height requests
+ *
+ * Specifies the type of requests for a #ClutterActor.
+ *
+ * Since: 0.8
+ */
+typedef enum {
+  CLUTTER_REQUEST_HEIGHT_FOR_WIDTH,
+  CLUTTER_REQUEST_WIDTH_FOR_HEIGHT
+} ClutterRequestMode;
+
 G_END_DECLS
 
 #endif /* __CLUTTER_TYPES_H__ */
