@@ -46,7 +46,7 @@
  *
  * The minor version of the Clutter library (2, if %CLUTTER_VERSION is 1.2.3)
  */
-#define CLUTTER_MINOR_VERSION   (6)
+#define CLUTTER_MINOR_VERSION   (8)
 
 /**
  * CLUTTER_MICRO_VERSION:
@@ -60,7 +60,7 @@
  *
  * The full version of the Clutter library, like 1.2.3
  */
-#define CLUTTER_VERSION         0.6.0
+#define CLUTTER_VERSION         0.8.0
 
 /**
  * CLUTTER_VERSION_S:
@@ -68,7 +68,7 @@
  * The full version of the Clutter library, in string form (suited for
  * string concatenation)
  */
-#define CLUTTER_VERSION_S       "0.6.0"
+#define CLUTTER_VERSION_S       "0.8.0"
 
 /**
  * CLUTTER_VERSION_HEX:
@@ -112,6 +112,15 @@
  */
 #define CLUTTER_COGL            "gl"
 
+/**
+ * CLUTTER_STAGE_TYPE:
+ *
+ * The default GObject type for the Clutter stage. 
+ *
+ * Since 0.8
+ */
+#define CLUTTER_STAGE_TYPE CLUTTER_TYPE_STAGE_GLX
+
 /**
  * CLUTTER_NO_FPU:
  *
@@ -122,4 +131,5 @@
  */
 #define CLUTTER_NO_FPU          CLUTTER_NO_FPU_MACRO_WAS_REMOVED
 
+
 #endif /* __CLUTTER_VERSION_H__ */
