@@ -32,8 +32,7 @@
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
 
-#include <GLES/gl.h>
-#include <GLES/egl.h>
+#include "clutter-egl-headers.h"
 
 #include <clutter/clutter-stage.h>
 
