@@ -45,8 +45,9 @@
 #include "../clutter-main.h"
 #include "../clutter-debug.h"
 #include "../clutter-private.h"
+#include "../clutter-version.h"
 
-#include "cogl.h"
+#include "cogl/cogl.h"
 
 G_DEFINE_TYPE (ClutterBackendGLX, clutter_backend_glx, CLUTTER_TYPE_BACKEND_X11);
 
@@ -172,6 +173,18 @@ clutter_backend_glx_finalize (GObject *gobject)
 static void
 clutter_backend_glx_dispose (GObject *gobject)
 {
+  ClutterBackendGLX *backend_glx = CLUTTER_BACKEND_GLX (gobject);
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (gobject);
+
+  /* Unrealize all shaders, since the GL context is going away */
+  _clutter_shader_release_all ();
+
+  if (backend_glx->gl_context)
+    {
+      glXDestroyContext (backend_x11->xdpy, backend_glx->gl_context);
+      backend_glx->gl_context = None;
+    }
+
   G_OBJECT_CLASS (clutter_backend_glx_parent_class)->dispose (gobject);
 }
 
@@ -213,9 +226,16 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
 {
   ClutterBackendGLX  *backend_glx = CLUTTER_BACKEND_GLX (backend);
   const gchar        *glx_extensions = NULL;
-  ClutterFeatureFlags flags = 0;
+  ClutterFeatureFlags flags;
+  
+  flags = clutter_backend_x11_get_features (backend);
+  flags |= CLUTTER_FEATURE_STAGE_MULTIPLE;
 
-  /* FIXME: we really need to check if gl context is set */
+  /* this will make sure that the GL context exists and
+   * it's bound to a drawable
+   */
+  g_assert (backend_glx->gl_context != None);
+  g_assert (glXGetCurrentDrawable () != None);
 
   CLUTTER_NOTE (BACKEND, "Checking features\n"
                 "GL_VENDOR: %s\n"
@@ -327,27 +347,106 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
 
   CLUTTER_NOTE (MISC, "backend features checked");
 
-  return flags|clutter_backend_x11_get_features (backend);
+  return flags;
 }
 
 static void
-clutter_backend_glx_redraw (ClutterBackend *backend)
+clutter_backend_glx_ensure_context (ClutterBackend *backend, 
+                                    ClutterStage   *stage)
 {
-  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
-  ClutterStageGLX   *stage_glx;
-  ClutterStageX11   *stage_x11;
+  if (stage == NULL)
+    {
+      ClutterBackendX11 *backend_x11;
 
-  stage_x11 = CLUTTER_STAGE_X11(backend_x11->stage);
-  stage_glx = CLUTTER_STAGE_GLX(backend_x11->stage);
+      backend_x11 = CLUTTER_BACKEND_X11 (backend);
+      CLUTTER_NOTE (MULTISTAGE, "Clearing all context");
 
-  clutter_actor_paint (CLUTTER_ACTOR (stage_glx));
+      glXMakeCurrent (backend_x11->xdpy, None, NULL);
+    }
+  else
+    {
+      ClutterBackendGLX  *backend_glx;
+      ClutterStageGLX    *stage_glx;
+      ClutterStageX11    *stage_x11;
+      ClutterStageWindow *impl;
+
+      impl = _clutter_stage_get_window (stage);
+      g_assert (impl != NULL);
+
+      CLUTTER_NOTE (MULTISTAGE, "Setting context for stage of type %s [%p]",
+                    g_type_name (G_OBJECT_TYPE (impl)),
+                    impl);
+
+      stage_glx = CLUTTER_STAGE_GLX (impl);
+      stage_x11 = CLUTTER_STAGE_X11 (impl);
+      backend_glx = CLUTTER_BACKEND_GLX (backend);
+
+      /* no GL context to set */
+      if (backend_glx->gl_context == None)
+        return;
+
+      clutter_x11_trap_x_errors ();
+
+      /* we might get here inside the final dispose cycle, so we
+       * need to handle this gracefully
+       */
+      if (stage_x11->xwin == None)
+        {
+          ClutterBackendX11 *backend_x11;
+
+          backend_x11 = CLUTTER_BACKEND_X11 (backend);
+          CLUTTER_NOTE (MULTISTAGE,
+                        "Received a stale stage, clearing all context");
+
+          glXMakeCurrent (backend_x11->xdpy, None, NULL);
+        }
+      else
+        {
+          CLUTTER_NOTE (BACKEND,
+                        "MakeCurrent dpy: %p, window: 0x%x (%s), context: %p",
+                        stage_x11->xdpy,
+                        (int) stage_x11->xwin,
+                        stage_x11->is_foreign_xwin ? "foreign" : "native",
+                        backend_glx->gl_context);
+
+          glXMakeCurrent (stage_x11->xdpy,
+                          stage_x11->xwin,
+                          backend_glx->gl_context);
+        }
+
+      if (clutter_x11_untrap_x_errors ())
+        g_critical ("Unable to make the stage window 0x%x the current "
+                    "GLX drawable",
+                    (int) stage_x11->xwin);
+    }
+}
+
+static void
+clutter_backend_glx_redraw (ClutterBackend *backend,
+                            ClutterStage   *stage)
+{
+  ClutterStageGLX *stage_glx;
+  ClutterStageX11 *stage_x11;
+  ClutterStageWindow *impl;
+
+  impl = _clutter_stage_get_window (stage);
+  if (!impl)
+    return;
+
+  g_assert (CLUTTER_IS_STAGE_GLX (impl));
+
+  stage_x11 = CLUTTER_STAGE_X11 (impl);
+  stage_glx = CLUTTER_STAGE_GLX (impl);
+
+  /* this will cause the stage implementation to be painted */
+  clutter_actor_paint (CLUTTER_ACTOR (stage));
 
   /* Why this paint is done in backend as likely GL windowing system
    * specific calls, like swapping buffers.
   */
   if (stage_x11->xwin)
     {
-      clutter_backend_glx_wait_for_vblank (CLUTTER_BACKEND_GLX(backend));
+      clutter_backend_glx_wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
       glXSwapBuffers (stage_x11->xdpy, stage_x11->xwin);
     }
   else
@@ -358,50 +457,36 @@ clutter_backend_glx_redraw (ClutterBackend *backend)
     }
 }
 
-gboolean
-clutter_backend_glx_init_stage (ClutterBackend  *backend,
-                                GError         **error)
+static ClutterActor*
+clutter_backend_glx_create_stage (ClutterBackend  *backend,
+                                  ClutterStage    *wrapper,
+                                  GError         **error)
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  ClutterStageX11   *stage_x11;
+  ClutterActor      *stage;
 
-  if (!backend_x11->stage)
-    {
-      ClutterStageX11 *stage_x11;
-      ClutterActor *stage;
+  CLUTTER_NOTE (BACKEND, "Creating stage of type `%s'",
+                g_type_name (CLUTTER_STAGE_TYPE));
 
-      stage = g_object_new (CLUTTER_TYPE_STAGE_GLX, NULL);
+  stage = g_object_new (CLUTTER_TYPE_STAGE_GLX, NULL);
 
-      /* copy backend data into the stage */
-      stage_x11 = CLUTTER_STAGE_X11 (stage);
-      stage_x11->xdpy = backend_x11->xdpy;
-      stage_x11->xwin_root = backend_x11->xwin_root;
-      stage_x11->xscreen = backend_x11->xscreen_num;
-      stage_x11->backend = backend_x11;
+  /* copy backend data into the stage */
+  stage_x11 = CLUTTER_STAGE_X11 (stage);
+  stage_x11->xdpy = backend_x11->xdpy;
+  stage_x11->xwin_root = backend_x11->xwin_root;
+  stage_x11->xscreen = backend_x11->xscreen_num;
+  stage_x11->backend = backend_x11;
+  stage_x11->wrapper = wrapper;
 
-      CLUTTER_NOTE (MISC, "X11 stage created (display:%p, screen:%d, root:%u)",
-                    stage_x11->xdpy,
-                    stage_x11->xscreen,
-                    (unsigned int) stage_x11->xwin_root);
+  CLUTTER_NOTE (BACKEND, "GLX stage created (display:%p, screen:%d, root:%u)",
+                stage_x11->xdpy,
+                stage_x11->xscreen,
+                (unsigned int) stage_x11->xwin_root);
 
-      g_object_set_data (G_OBJECT (stage), "clutter-backend", backend);
-
-      backend_x11->stage = g_object_ref_sink (stage);
-    }
-
-  clutter_actor_realize (backend_x11->stage);
-
-  if (!CLUTTER_ACTOR_IS_REALIZED (backend_x11->stage))
-    {
-      g_set_error (error, CLUTTER_INIT_ERROR,
-                   CLUTTER_INIT_ERROR_INTERNAL,
-                   "Unable to realize the main stage");
-      return FALSE;
-    }
-
-  return TRUE;
+  return stage;
 }
 
-
 static void
 clutter_backend_glx_class_init (ClutterBackendGLXClass *klass)
 {
@@ -409,21 +494,22 @@ clutter_backend_glx_class_init (ClutterBackendGLXClass *klass)
   ClutterBackendClass *backend_class = CLUTTER_BACKEND_CLASS (klass);
 
   gobject_class->constructor = clutter_backend_glx_constructor;
-  gobject_class->dispose = clutter_backend_glx_dispose;
-  gobject_class->finalize = clutter_backend_glx_finalize;
+  gobject_class->dispose     = clutter_backend_glx_dispose;
+  gobject_class->finalize    = clutter_backend_glx_finalize;
 
-  backend_class->pre_parse   = clutter_backend_glx_pre_parse;
-  backend_class->post_parse   = clutter_backend_glx_post_parse;
-  backend_class->init_stage   = clutter_backend_glx_init_stage;
-  backend_class->add_options  = clutter_backend_glx_add_options;
-  backend_class->get_features = clutter_backend_glx_get_features;
-  backend_class->redraw       = clutter_backend_glx_redraw;
+  backend_class->pre_parse      = clutter_backend_glx_pre_parse;
+  backend_class->post_parse     = clutter_backend_glx_post_parse;
+  backend_class->create_stage   = clutter_backend_glx_create_stage;
+  backend_class->add_options    = clutter_backend_glx_add_options;
+  backend_class->get_features   = clutter_backend_glx_get_features;
+  backend_class->redraw         = clutter_backend_glx_redraw;
+  backend_class->ensure_context = clutter_backend_glx_ensure_context;
 }
 
 static void
 clutter_backend_glx_init (ClutterBackendGLX *backend_glx)
 {
-  ;
+
 }
 
 /* every backend must implement this function */
