@@ -64,12 +64,19 @@ struct _ClutterBackendGLX
 {
   ClutterBackendX11 parent_instance;
 
+  /* Single context for all wins */
+  GLXContext             gl_context;
+
   /* Vblank stuff */
   GetVideoSyncProc       get_video_sync;
   WaitVideoSyncProc      wait_video_sync;
   SwapIntervalProc       swap_interval;
   gint                   dri_fd;
   ClutterGLXVBlankType   vblank_type;
+
+  /* props */
+  Atom atom_WM_STATE;
+  Atom atom_WM_STATE_FULLSCREEN;
 };
 
 struct _ClutterBackendGLXClass
