@@ -39,27 +39,33 @@
 #include "../clutter-shader.h"
 #include "../clutter-group.h"
 #include "../clutter-container.h"
+#include "../clutter-stage.h"
+#include "../clutter-stage-window.h"
 
-#include "cogl.h"
+#include "cogl/cogl.h"
 
 #include <GL/glx.h>
 #include <GL/gl.h>
 
-#include <gdk-pixbuf-xlib/gdk-pixbuf-xlib.h>
+static void clutter_stage_window_iface_init (ClutterStageWindowIface *iface);
 
-G_DEFINE_TYPE (ClutterStageGLX, clutter_stage_glx, CLUTTER_TYPE_STAGE_X11);
+G_DEFINE_TYPE_WITH_CODE (ClutterStageGLX,
+                         clutter_stage_glx,
+                         CLUTTER_TYPE_STAGE_X11,
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_STAGE_WINDOW,
+                                                clutter_stage_window_iface_init));
 
 static void
 clutter_stage_glx_unrealize (ClutterActor *actor)
 {
   ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (actor);
   ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (actor);
-
   gboolean was_offscreen;
 
-  CLUTTER_MARK();
+  /* Note unrealize should free up any backend stage related resources */
+  CLUTTER_NOTE (BACKEND, "Unrealizing stage");
 
-  g_object_get (actor, "offscreen", &was_offscreen, NULL);
+  g_object_get (stage_x11->wrapper, "offscreen", &was_offscreen, NULL);
 
   /* Chain up so all children get unrealized, needed to move texture data
    * across contexts
@@ -68,9 +74,6 @@ clutter_stage_glx_unrealize (ClutterActor *actor)
 
   clutter_x11_trap_x_errors ();
 
-  /* Unrealize all shaders, since the GL context is going away */
-  clutter_shader_release_all ();
-
   if (G_UNLIKELY (was_offscreen))
     {
       if (stage_glx->glxpixmap)
@@ -96,14 +99,6 @@ clutter_stage_glx_unrealize (ClutterActor *actor)
         stage_x11->xwin = None;
     }
 
-  glXMakeCurrent (stage_x11->xdpy, None, NULL);
-
-  if (stage_glx->gl_context != None)
-    {
-      glXDestroyContext (stage_x11->xdpy, stage_glx->gl_context);
-      stage_glx->gl_context = None;
-    }
-
   XSync (stage_x11->xdpy, False);
 
   clutter_x11_untrap_x_errors ();
@@ -114,13 +109,18 @@ clutter_stage_glx_unrealize (ClutterActor *actor)
 static void
 clutter_stage_glx_realize (ClutterActor *actor)
 {
-  ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (actor);
-  ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (actor);
-  gboolean is_offscreen;
+  ClutterStageX11   *stage_x11 = CLUTTER_STAGE_X11 (actor);
+  ClutterStageGLX   *stage_glx = CLUTTER_STAGE_GLX (actor);
+  ClutterBackendGLX *backend_glx;
+  ClutterBackendX11 *backend_x11;
+  gboolean           is_offscreen;
 
   CLUTTER_NOTE (MISC, "Realizing main stage");
 
-  g_object_get (actor, "offscreen", &is_offscreen, NULL);
+  g_object_get (stage_x11->wrapper, "offscreen", &is_offscreen, NULL);
+
+  backend_glx = CLUTTER_BACKEND_GLX (clutter_get_default_backend ());
+  backend_x11 = CLUTTER_BACKEND_X11 (clutter_get_default_backend ());
 
   if (G_LIKELY (!is_offscreen))
     {
@@ -136,19 +136,20 @@ clutter_stage_glx_realize (ClutterActor *actor)
         };
 
       if (stage_x11->xvisinfo)
-        XFree (stage_x11->xvisinfo);
-      stage_x11->xvisinfo = NULL;
+        {
+          XFree (stage_x11->xvisinfo);
+          stage_x11->xvisinfo = None;
+        }
 
       /* The following check seems strange */
       if (stage_x11->xvisinfo == None)
         stage_x11->xvisinfo = glXChooseVisual (stage_x11->xdpy,
-                                           stage_x11->xscreen,
+                                               stage_x11->xscreen,
                                                gl_attributes);
       if (!stage_x11->xvisinfo)
         {
           g_critical ("Unable to find suitable GL visual.");
-          CLUTTER_ACTOR_UNSET_FLAGS (actor, CLUTTER_ACTOR_REALIZED);
-          return;
+          goto fail;
         }
 
       if (stage_x11->xwin == None)
@@ -168,53 +169,63 @@ clutter_stage_glx_realize (ClutterActor *actor)
                                             AllocNone);
           mask = CWBackPixel | CWBorderPixel | CWColormap;
           stage_x11->xwin = XCreateWindow (stage_x11->xdpy,
-                                              stage_x11->xwin_root,
-                                              0, 0,
-                                              stage_x11->xwin_width,
-                                              stage_x11->xwin_height,
-                                              0,
-                                              stage_x11->xvisinfo->depth,
-                                              InputOutput,
-                                              stage_x11->xvisinfo->visual,
-                                              mask, &xattr);
+                                           stage_x11->xwin_root,
+                                           0, 0,
+                                           stage_x11->xwin_width,
+                                           stage_x11->xwin_height,
+                                           0,
+                                           stage_x11->xvisinfo->depth,
+                                           InputOutput,
+                                           stage_x11->xvisinfo->visual,
+                                           mask, &xattr);
         }
 
-      CLUTTER_NOTE (MISC, "XSelectInput");
-      XSelectInput (stage_x11->xdpy, stage_x11->xwin,
-                    StructureNotifyMask |
-                    FocusChangeMask |
-                    ExposureMask |
-                    /* FIXME: we may want to eplicity enable MotionMask */
-                    PointerMotionMask |
-                    KeyPressMask | KeyReleaseMask |
-                    ButtonPressMask | ButtonReleaseMask |
-                    PropertyChangeMask);
+      if (clutter_x11_has_event_retrieval())
+        {
+          if (clutter_x11_has_xinput())
+            {
+              XSelectInput (stage_x11->xdpy, stage_x11->xwin,
+                            StructureNotifyMask |
+                            FocusChangeMask |
+                            ExposureMask |
+                            KeyPressMask | KeyReleaseMask |
+                            PropertyChangeMask);
+#ifdef USE_XINPUT          
+              _clutter_x11_select_events (stage_x11->xwin);
+#endif
+            }
+          else
+            XSelectInput (stage_x11->xdpy, stage_x11->xwin,
+                          StructureNotifyMask |
+                          FocusChangeMask |
+                          ExposureMask |
+                          PointerMotionMask |
+                          KeyPressMask | KeyReleaseMask |
+                          ButtonPressMask | ButtonReleaseMask |
+                          PropertyChangeMask);
+        }
 
       /* no user resize.. */
       clutter_stage_x11_fix_window_size (stage_x11);
-
       clutter_stage_x11_set_wm_protocols (stage_x11);
 
-      if (stage_glx->gl_context)
-        glXDestroyContext (stage_x11->xdpy, stage_glx->gl_context);
-
-      CLUTTER_NOTE (GL, "Creating GL Context");
-      stage_glx->gl_context = glXCreateContext (stage_x11->xdpy, 
-                                                stage_x11->xvisinfo, 
-                                                0, 
-                                                True);
-
-      if (stage_glx->gl_context == None)
+      if (G_UNLIKELY (backend_glx->gl_context == None))
         {
-          g_critical ("Unable to create suitable GL context.");
+          CLUTTER_NOTE (GL, "Creating GL Context");
+          backend_glx->gl_context = glXCreateContext (stage_x11->xdpy, 
+                                                      stage_x11->xvisinfo, 
+                                                      0,
+                                                      True);
 
-          CLUTTER_ACTOR_UNSET_FLAGS (actor, CLUTTER_ACTOR_REALIZED);
-
-          return;
+          if (backend_glx->gl_context == None)
+            {
+              g_critical ("Unable to create suitable GL context.");
+              goto fail;
+            }
         }
 
-      CLUTTER_NOTE (GL, "glXMakeCurrent");
-      glXMakeCurrent (stage_x11->xdpy, stage_x11->xwin, stage_glx->gl_context);
+      CLUTTER_NOTE (BACKEND, "Marking stage as realized");
+      CLUTTER_ACTOR_SET_FLAGS (stage_x11, CLUTTER_ACTOR_REALIZED);
     }
   else
     {
@@ -244,9 +255,7 @@ clutter_stage_glx_realize (ClutterActor *actor)
           goto fail;
         }
 
-      if (stage_glx->gl_context)
-        glXDestroyContext (stage_x11->xdpy, stage_glx->gl_context);
-
+     
       stage_x11->xpixmap = XCreatePixmap (stage_x11->xdpy,
                                           stage_x11->xwin_root,
                                           stage_x11->xwin_width, 
@@ -258,113 +267,39 @@ clutter_stage_glx_realize (ClutterActor *actor)
                                                  stage_x11->xvisinfo,
                                                  stage_x11->xpixmap);
 
-      /* indirect */
-      stage_glx->gl_context = glXCreateContext (stage_x11->xdpy, 
-                                                stage_x11->xvisinfo, 
-                                                0, 
-                                                False);
-
-      clutter_x11_trap_x_errors ();
-
-      glXMakeCurrent (stage_x11->xdpy,
-                      stage_glx->glxpixmap,
-                      stage_glx->gl_context);
-
-      if (clutter_x11_untrap_x_errors ())
+      if (backend_glx->gl_context == None)
         {
-          g_critical ("Unable to set up offscreen context.");
-          goto fail;
+          CLUTTER_NOTE (GL, "Creating GL Context");
+
+          /* FIXME: we probably need a seperate offscreen context here
+           * - though it likely makes most sense to drop offscreen stages
+           * and rely on FBO's instead and GLXPixmaps seems mostly broken
+           * anyway..
+          */
+          backend_glx->gl_context =  glXCreateContext (stage_x11->xdpy, 
+                                                       stage_x11->xvisinfo, 
+                                                       0,
+                                                       False);
+
+          if (backend_glx->gl_context == None)
+            {
+              g_critical ("Unable to create suitable GL context.");
+              goto fail;
+            }
         }
+
+      CLUTTER_NOTE (BACKEND, "Marking stage as realized");
+      CLUTTER_ACTOR_SET_FLAGS (stage_x11, CLUTTER_ACTOR_REALIZED);
     }
 
-  /* Make sure the viewport gets set up correctly */
-  CLUTTER_SET_PRIVATE_FLAGS (actor, CLUTTER_ACTOR_SYNC_MATRICES);
+  /* we need to chain up to the X11 stage implementation in order to
+   * set the window state in case we set it before realizing the stage
+   */
+  CLUTTER_ACTOR_CLASS (clutter_stage_glx_parent_class)->realize (actor);
   return;
   
- fail:
-
-  /* For one reason or another we cant realize the stage.. */
+fail:
   CLUTTER_ACTOR_UNSET_FLAGS (actor, CLUTTER_ACTOR_REALIZED);
-  return;
-}
-
-static void
-snapshot_pixbuf_free (guchar   *pixels,
-                      gpointer  data)
-{
-  g_free (pixels);
-}
-
-static GdkPixbuf*
-clutter_stage_glx_draw_to_pixbuf (ClutterStage *stage,
-                                  gint          x,
-                                  gint          y,
-                                  gint          width,
-                                  gint          height)
-{
-  guchar *data;
-  GdkPixbuf *pixb;
-  ClutterActor *actor;
-  ClutterStageGLX *stage_glx;
-  ClutterStageX11 *stage_x11;
-  gboolean is_offscreen = FALSE;
-
-  stage_glx = CLUTTER_STAGE_GLX (stage);
-  stage_x11 = CLUTTER_STAGE_X11 (stage);
-  actor = CLUTTER_ACTOR (stage);
-
-  if (width < 0)
-    width = clutter_actor_get_width (actor);
-
-  if (height < 0)
-    height = clutter_actor_get_height (actor);
-
-  g_object_get (stage, "offscreen", &is_offscreen, NULL);
-
-  if (G_UNLIKELY (is_offscreen))
-    {
-      gdk_pixbuf_xlib_init (stage_x11->xdpy, stage_x11->xscreen);
-
-      pixb = gdk_pixbuf_xlib_get_from_drawable (NULL,
-                                                (Drawable) stage_x11->xpixmap,
-                                                DefaultColormap (stage_x11->xdpy,
-                                                                 stage_x11->xscreen),
-                                                stage_x11->xvisinfo->visual,
-                                                x, y,
-                                                0, 0,
-                                                width, height);
-    }
-  else
-    {
-      GdkPixbuf *tmp = NULL, *tmp2 = NULL;
-      gint stride;
-
-      stride = ((width * 4 + 3) &~ 3);
-
-      data = g_malloc0 (sizeof (guchar) * stride * height);
-
-      glReadPixels (x, 
-                    clutter_actor_get_height (actor) - y - height,
-                    width, 
-                    height, GL_RGBA, GL_UNSIGNED_BYTE, data);
-
-      tmp = gdk_pixbuf_new_from_data (data,
-                                      GDK_COLORSPACE_RGB, 
-                                      TRUE, 
-                                      8, 
-                                      width, height,
-                                      stride,
-                                      snapshot_pixbuf_free,
-                                      NULL);
-
-      tmp2 = gdk_pixbuf_flip (tmp, TRUE); 
-      g_object_unref (tmp);
-
-      pixb = gdk_pixbuf_rotate_simple (tmp2, GDK_PIXBUF_ROTATE_UPSIDEDOWN);
-      g_object_unref (tmp2);
-   }
-
-  return pixb;
 }
 
 static void
@@ -384,17 +319,20 @@ clutter_stage_glx_class_init (ClutterStageGLXClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
-  ClutterStageClass *stage_class = CLUTTER_STAGE_CLASS (klass);
 
   gobject_class->dispose = clutter_stage_glx_dispose;
   
   actor_class->realize = clutter_stage_glx_realize;
   actor_class->unrealize = clutter_stage_glx_unrealize;
-  stage_class->draw_to_pixbuf = clutter_stage_glx_draw_to_pixbuf;
 }
 
 static void
 clutter_stage_glx_init (ClutterStageGLX *stage)
 {
-  ;
+}
+
+static void
+clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
+{
+  /* the rest is inherited from ClutterStageX11 */
 }
