@@ -19,7 +19,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include "clutter-stage-x11.h"
 #include "clutter-backend-x11.h"
@@ -41,6 +43,10 @@
 
 #include <X11/Xatom.h>
 
+#ifdef USE_XINPUT
+#include <X11/extensions/XInput.h>
+#endif
+
 /* XEMBED protocol support for toolkit embedding */
 #define XEMBED_MAPPED                   (1 << 0)
 #define MAX_SUPPORTED_XEMBED_VERSION    1
@@ -158,7 +164,6 @@ void
 _clutter_backend_x11_events_init (ClutterBackend *backend)
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
-  ClutterStage *stage = CLUTTER_STAGE (backend_x11->stage);
   GSource *source;
   ClutterEventSource *event_source;
   int connection_number;
@@ -179,9 +184,6 @@ _clutter_backend_x11_events_init (ClutterBackend *backend)
   g_source_set_can_recurse (source, TRUE);
   g_source_attach (source, NULL);
 
-  xembed_set_info (backend_x11,
-                   clutter_x11_get_stage_window (stage),
-                   0);
 }
 
 void
@@ -216,38 +218,71 @@ set_user_time (ClutterBackendX11 *backend_x11,
     }
 }
 
+#if 0 /* See XInput keyboard comment below USE_XINPUT */
 static void
-translate_key_event (ClutterBackend *backend,
-                     ClutterEvent   *event,
-                     XEvent         *xevent)
+convert_xdevicekey_to_xkey (XDeviceKeyEvent *xkev, XEvent *xevent)
 {
+  xevent->xany.type = xevent->xkey.type = xkev->type;
+  xevent->xkey.serial = xkev->serial;
+  xevent->xkey.display = xkev->display;
+  xevent->xkey.window = xkev->window;
+  xevent->xkey.root = xkev->root;
+  xevent->xkey.subwindow = xkev->subwindow;
+  xevent->xkey.time = xkev->time;
+  xevent->xkey.x = xkev->x;
+  xevent->xkey.y = xkev->y;
+  xevent->xkey.x_root = xkev->x_root;
+  xevent->xkey.y_root = xkev->y_root;
+  xevent->xkey.state = xkev->state;
+  xevent->xkey.keycode = xkev->keycode;
+  xevent->xkey.same_screen = xkev->same_screen;
+}
+#endif /* USE_XINPUT */
+
+static void
+translate_key_event (ClutterBackend   *backend,
+                     ClutterEvent     *event,
+                     XEvent           *xevent)
+{
+  char buffer[256+1];
+  int n;
+  
   CLUTTER_NOTE (EVENT, "Translating key %s event",
                 xevent->xany.type == KeyPress ? "press" : "release");
 
-  event->key.type = (xevent->xany.type == KeyPress) ? CLUTTER_KEY_PRESS
-                                                    : CLUTTER_KEY_RELEASE;
   event->key.time = xevent->xkey.time;
   event->key.modifier_state = (ClutterModifierType) xevent->xkey.state;
   event->key.hardware_keycode = xevent->xkey.keycode;
 
-  /* FIXME: We need to handle other modifiers rather than just shift */
+  /* keyval is the key ignoring all modifiers ('1' vs. '!') */
   event->key.keyval =
     XKeycodeToKeysym (xevent->xkey.display,
                       xevent->xkey.keycode,
-                      (event->key.modifier_state & CLUTTER_SHIFT_MASK) ? 1
-                                                                       : 0);
+                      0);
+
+  /* unicode_value is the printable representation */
+  n = XLookupString (&xevent->xkey, buffer, sizeof (buffer) - 1, NULL, NULL);
+
+  if (n != NoSymbol)
+    {
+      event->key.unicode_value = g_utf8_get_char_validated (buffer, n);
+      if ((event->key.unicode_value != -1) &&
+          (event->key.unicode_value != -2))
+        return;
+    }
+  
+  event->key.unicode_value = (gunichar)'\0';
 }
 
 static gboolean
 handle_wm_protocols_event (ClutterBackendX11 *backend_x11,
+                           Window             window,
                            XEvent            *xevent)
 {
   Atom atom = (Atom) xevent->xclient.data.l[0];
-  ClutterStage *stage = CLUTTER_STAGE (backend_x11->stage);
-  Window stage_xwindow = clutter_x11_get_stage_window (stage);
 
   if (atom == backend_x11->atom_WM_DELETE_WINDOW &&
-      xevent->xany.window == stage_xwindow)
+      xevent->xany.window == window)
     {
       /* the WM_DELETE_WINDOW is a request: we do not destroy
        * the window right away, as it might contain vital data;
@@ -258,13 +293,13 @@ handle_wm_protocols_event (ClutterBackendX11 *backend_x11,
                     xevent->xclient.window);
 
       set_user_time (backend_x11,
-                     &stage_xwindow,
+                     &window,
                      xevent->xclient.data.l[1]);
 
       return TRUE;
     }
   else if (atom == backend_x11->atom_NET_WM_PING &&
-           xevent->xany.window == stage_xwindow)
+           xevent->xany.window == window)
     {
       XClientMessageEvent xclient = xevent->xclient;
 
@@ -286,7 +321,7 @@ handle_xembed_event (ClutterBackendX11 *backend_x11,
 {
   ClutterActor *stage;
 
-  stage = _clutter_backend_get_stage (CLUTTER_BACKEND (backend_x11));
+  stage = clutter_stage_get_default ();
 
   switch (xevent->xclient.data.l[1])
     {
@@ -330,16 +365,14 @@ event_translate (ClutterBackend *backend,
                  ClutterEvent   *event,
                  XEvent         *xevent)
 {
-  ClutterBackendX11 *backend_x11;
-  ClutterStageX11   *stage_x11;
-  ClutterStage      *stage;
-  gboolean           res;
-  Window             xwindow, stage_xwindow;
+  ClutterBackendX11  *backend_x11;
+  ClutterStageX11    *stage_x11;
+  ClutterStage       *stage;
+  ClutterStageWindow *impl;
+  gboolean            res, not_yet_handled = FALSE;
+  Window              xwindow, stage_xwindow;
 
   backend_x11    = CLUTTER_BACKEND_X11 (backend);
-  stage          = CLUTTER_STAGE (_clutter_backend_get_stage (backend));
-  stage_x11      = CLUTTER_STAGE_X11 (stage);
-  stage_xwindow  = clutter_x11_get_stage_window (stage);
 
   xwindow = xevent->xany.window;
 
@@ -375,34 +408,44 @@ event_translate (ClutterBackend *backend,
    * (the x11 filters might be getting events for other windows, so do not
    * mess them about.
    */
-  if (xwindow != stage_xwindow)
-    return FALSE;
+  stage = clutter_x11_get_stage_from_window (xwindow);
+
+  if (stage == NULL)
+      return FALSE;
+
+  impl           = _clutter_stage_get_window (stage);
+  stage_x11      = CLUTTER_STAGE_X11 (impl);
+  stage_xwindow  = xwindow; /* clutter_x11_get_stage_window (stage); */
+
+  event->any.stage = stage;
+
 
   res = TRUE;
 
   switch (xevent->type)
     {
     case ConfigureNotify:
-      {
-        guint stage_width, stage_height;
+      if (!stage_x11->is_foreign_xwin)
+        {
+	  /* Set a flag so that the stage will know the actor is being
+	     resized in response to the window size changing as
+	     opposed to a request from the application. This prevents
+	     it from trying to resize the window again */
+	  stage_x11->handling_configure = TRUE;
 
-        clutter_actor_get_size (CLUTTER_ACTOR (stage),
-                                &stage_width,
-                                &stage_height);
+	  clutter_actor_set_size (CLUTTER_ACTOR (stage),
+				  xevent->xconfigure.width,
+				  xevent->xconfigure.height);
 
-        if (xevent->xconfigure.width != stage_width ||
-            xevent->xconfigure.height != stage_height)
-          {
-            clutter_actor_set_size (CLUTTER_ACTOR (stage),
-                                    xevent->xconfigure.width,
-                                    xevent->xconfigure.height);
-          }
-      }
+	  stage_x11->handling_configure = FALSE;
+        }
       res = FALSE;
       break;
 
     case PropertyNotify:
-      if (xevent->xproperty.atom == backend_x11->atom_NET_WM_STATE)
+      if (xevent->xproperty.atom == backend_x11->atom_NET_WM_STATE &&
+          xevent->xproperty.window == stage_xwindow &&
+          !stage_x11->is_foreign_xwin)
         {
           Atom     type;
           gint     format;
@@ -454,6 +497,8 @@ event_translate (ClutterBackend *backend,
           else
             res = FALSE;
         }
+      else
+        res = FALSE;
       break;
 
     case MapNotify:
@@ -495,98 +540,11 @@ event_translate (ClutterBackend *backend,
 
     case Expose:
       {
-        XEvent foo_xev;
-
-        /* Cheap compress */
-        while (XCheckTypedWindowEvent (backend_x11->xdpy,
-                                       xevent->xexpose.window,
-                                       Expose,
-                                       &foo_xev));
-
-        /* FIXME: need to make stage an 'actor' so can que
-         * a paint direct from there rather than hack here...
-         */
+        CLUTTER_NOTE (MULTISTAGE, "expose for stage: %p, redrawing", stage);
         clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
         res = FALSE;
       }
       break;
-
-    case KeyPress:
-      event->type = CLUTTER_KEY_PRESS;
-      translate_key_event (backend, event, xevent);
-      set_user_time (backend_x11, &xwindow, xevent->xkey.time);
-      break;
-
-    case KeyRelease:
-      event->type = CLUTTER_KEY_RELEASE;
-      translate_key_event (backend, event, xevent);
-      break;
-
-    case ButtonPress:
-      switch (xevent->xbutton.button)
-        {
-        case 4: /* up */
-        case 5: /* down */
-        case 6: /* left */
-        case 7: /* right */
-          event->scroll.type = event->type = CLUTTER_SCROLL;
-
-          if (xevent->xbutton.button == 4)
-            event->scroll.direction = CLUTTER_SCROLL_UP;
-          else if (xevent->xbutton.button == 5)
-            event->scroll.direction = CLUTTER_SCROLL_DOWN;
-          else if (xevent->xbutton.button == 6)
-            event->scroll.direction = CLUTTER_SCROLL_LEFT;
-          else
-            event->scroll.direction = CLUTTER_SCROLL_RIGHT;
-
-          event->scroll.time = xevent->xbutton.time;
-          event->scroll.x = xevent->xbutton.x;
-          event->scroll.y = xevent->xbutton.y;
-          event->scroll.modifier_state = xevent->xbutton.state;
-
-          break;
-        default:
-          event->button.type = event->type = CLUTTER_BUTTON_PRESS;
-          event->button.time = xevent->xbutton.time;
-          event->button.x = xevent->xbutton.x;
-          event->button.y = xevent->xbutton.y;
-          event->button.modifier_state = xevent->xbutton.state;
-          event->button.button = xevent->xbutton.button;
-
-          break;
-        }
-
-      set_user_time (backend_x11, &xwindow, event->button.time);
-      break;
-
-    case ButtonRelease:
-      /* scroll events don't have a corresponding release */
-      if (xevent->xbutton.button == 4 ||
-          xevent->xbutton.button == 5 ||
-          xevent->xbutton.button == 6 ||
-          xevent->xbutton.button == 7)
-        {
-          res = FALSE;
-          break;
-        }
-
-      event->button.type = event->type = CLUTTER_BUTTON_RELEASE;
-      event->button.time = xevent->xbutton.time;
-      event->button.x = xevent->xbutton.x;
-      event->button.y = xevent->xbutton.y;
-      event->button.modifier_state = xevent->xbutton.state;
-      event->button.button = xevent->xbutton.button;
-      break;
-
-    case MotionNotify:
-      event->motion.type = event->type = CLUTTER_MOTION;
-      event->motion.time = xevent->xmotion.time;
-      event->motion.x = xevent->xmotion.x;
-      event->motion.y = xevent->xmotion.y;
-      event->motion.modifier_state = xevent->xmotion.state;
-      break;
-
     case DestroyNotify:
       CLUTTER_NOTE (EVENT, "destroy notify:\txid: %ld",
                     xevent->xdestroywindow.window);
@@ -606,17 +564,246 @@ event_translate (ClutterBackend *backend,
         res = handle_xembed_event (backend_x11, xevent);
       else if (xevent->xclient.message_type == backend_x11->atom_WM_PROTOCOLS)
         {
-          res = handle_wm_protocols_event (backend_x11, xevent);
+          res = handle_wm_protocols_event (backend_x11, stage_xwindow, xevent);
           event->type = event->any.type = CLUTTER_DELETE;
         }
       break;
 
+    case KeyPress:
+      event->key.type = event->type = CLUTTER_KEY_PRESS;
+      translate_key_event (backend, event, xevent);
+
+      set_user_time (backend_x11, &xwindow, xevent->xkey.time);
+      break;
+              
+    case KeyRelease:
+      event->key.type = event->type = CLUTTER_KEY_RELEASE;
+      translate_key_event (backend, event, xevent);
+      break;
+
     default:
       /* ignore every other event */
-      res = FALSE;
+      not_yet_handled = TRUE;
       break;
     }
 
+  /* Input device event handling.. */
+  if (not_yet_handled)
+    {
+      if (!clutter_x11_has_xinput ())
+        {
+          /* Regular X event */
+          switch (xevent->type)
+            {
+			/* KeyPress / KeyRelease should reside here if XInput
+             * worked properly
+             */
+            case ButtonPress:
+              switch (xevent->xbutton.button)
+                {
+                case 4: /* up */
+                case 5: /* down */
+                case 6: /* left */
+                case 7: /* right */
+                  event->scroll.type = event->type = CLUTTER_SCROLL;
+                  
+                  if (xevent->xbutton.button == 4)
+                    event->scroll.direction = CLUTTER_SCROLL_UP;
+                  else if (xevent->xbutton.button == 5)
+                    event->scroll.direction = CLUTTER_SCROLL_DOWN;
+                  else if (xevent->xbutton.button == 6)
+                    event->scroll.direction = CLUTTER_SCROLL_LEFT;
+                  else
+                    event->scroll.direction = CLUTTER_SCROLL_RIGHT;
+                  
+                  event->scroll.time = xevent->xbutton.time;
+                  event->scroll.x = xevent->xbutton.x;
+                  event->scroll.y = xevent->xbutton.y;
+                  event->scroll.modifier_state = xevent->xbutton.state;
+                  
+                  break;
+                default:
+                  event->button.type = event->type = CLUTTER_BUTTON_PRESS;
+                  event->button.time = xevent->xbutton.time;
+                  event->button.x = xevent->xbutton.x;
+                  event->button.y = xevent->xbutton.y;
+                  event->button.modifier_state = xevent->xbutton.state;
+                  event->button.button = xevent->xbutton.button;
+                  
+                  break;
+                }
+              
+              set_user_time (backend_x11, &xwindow, event->button.time);
+              break;
+              
+            case ButtonRelease:
+              /* scroll events don't have a corresponding release */
+              if (xevent->xbutton.button == 4 ||
+                  xevent->xbutton.button == 5 ||
+                  xevent->xbutton.button == 6 ||
+                  xevent->xbutton.button == 7)
+                {
+                  res = FALSE;
+                  break;
+                }
+              
+              event->button.type = event->type = CLUTTER_BUTTON_RELEASE;
+              event->button.time = xevent->xbutton.time;
+              event->button.x = xevent->xbutton.x;
+              event->button.y = xevent->xbutton.y;
+              event->button.modifier_state = xevent->xbutton.state;
+              event->button.button = xevent->xbutton.button;
+              break;
+              
+            case MotionNotify:
+              event->motion.type = event->type = CLUTTER_MOTION;
+              event->motion.time = xevent->xmotion.time;
+              event->motion.x = xevent->xmotion.x;
+              event->motion.y = xevent->xmotion.y;
+              event->motion.modifier_state = xevent->xmotion.state;
+              break;
+            default:
+              /* ignore every other event */
+              res = FALSE;
+              break;
+            }
+        }
+      else
+        {  /* XInput fun.. Needs clean up. */
+#ifdef USE_XINPUT
+          int *ev_types = backend_x11->event_types;
+
+          CLUTTER_NOTE (EVENT, "XInput event type: %d", xevent->type);
+
+          if (xevent->type == ev_types [CLUTTER_X11_XINPUT_BUTTON_PRESS_EVENT])
+            {
+              XDeviceButtonEvent *xbev = (XDeviceButtonEvent *)xevent;
+
+              switch (xbev->button)
+                {
+                case 4:
+                case 5:
+                case 6:
+                case 7:
+                  event->scroll.type = event->type = CLUTTER_SCROLL;
+                  
+                  if (xbev->button == 4)
+                    event->scroll.direction = CLUTTER_SCROLL_UP;
+                  else if (xbev->button == 5)
+                    event->scroll.direction = CLUTTER_SCROLL_DOWN;
+                  else if (xbev->button == 6)
+                    event->scroll.direction = CLUTTER_SCROLL_LEFT;
+                  else
+                    event->scroll.direction = CLUTTER_SCROLL_RIGHT;
+                  
+                  event->scroll.time = xbev->time;
+                  event->scroll.x = xbev->x;
+                  event->scroll.y = xbev->y;
+                  event->scroll.modifier_state = xbev->state;
+                  event->scroll.device 
+                    = (ClutterInputDevice *)_clutter_x11_get_device_for_xid (xbev->deviceid);
+                  break;
+                default:
+                  event->button.type = event->type = CLUTTER_BUTTON_PRESS;
+                  event->button.time = xbev->time;
+                  event->button.x = xbev->x;
+                  event->button.y = xbev->y;
+                  event->button.modifier_state = xbev->state;
+                  event->button.button = xbev->button;
+                  event->button.device 
+                    = (ClutterInputDevice *)_clutter_x11_get_device_for_xid (xbev->deviceid);
+                  break;
+                }
+
+              set_user_time (backend_x11, &xwindow, xbev->time);
+
+              CLUTTER_NOTE(EVENT, "XINPUT Button press event for %li %d %d", 
+                           xbev->deviceid, xbev->x, xbev->y);
+            } 
+          else if (xevent->type 
+                        == ev_types[CLUTTER_X11_XINPUT_BUTTON_RELEASE_EVENT])
+            {
+              XDeviceButtonEvent *xbev = (XDeviceButtonEvent *)xevent;
+
+              /* scroll events don't have a corresponding release */
+              if (xbev->button == 4 ||
+                  xbev->button == 5 ||
+                  xbev->button == 6 ||
+                  xbev->button == 7)
+                {
+                  res = FALSE;
+                  goto out;
+                }
+
+              event->button.type = event->type = CLUTTER_BUTTON_RELEASE;
+              event->button.time = xbev->time;
+              event->button.x = xbev->x;
+              event->button.y = xbev->y;
+              event->button.modifier_state = xbev->state;
+              event->button.button = xbev->button;
+              event->button.device 
+                    = (ClutterInputDevice *)_clutter_x11_get_device_for_xid (xbev->deviceid);
+              CLUTTER_NOTE(EVENT, "XINPUT Button release event for %li %d %d", 
+                           xbev->deviceid, xbev->x, xbev->y);
+            } 
+          else if (xevent->type 
+                       == ev_types [CLUTTER_X11_XINPUT_MOTION_NOTIFY_EVENT]) 
+            {
+              XDeviceMotionEvent *xmev = (XDeviceMotionEvent *)xevent;
+
+              event->motion.type = event->type = CLUTTER_MOTION;
+              event->motion.time = xmev->time;
+              event->motion.x = xmev->x;
+              event->motion.y = xmev->y;
+              event->motion.modifier_state = xmev->state;
+              event->motion.device 
+                    = (ClutterInputDevice *) _clutter_x11_get_device_for_xid (xmev->deviceid);
+              CLUTTER_NOTE(EVENT, "XINPUT Motion event for %li %d %d", 
+                           xmev->deviceid, 
+                           xmev->x, 
+                           xmev->y);
+            } 
+#if 0
+        /* the Xinput handling of key presses/releases disabled for now since
+         * it makes keyrepeat, and key presses and releases outside the window
+         * not generate events even when the window has focus
+         */
+
+          else if (xevent->type 
+                        == ev_types [CLUTTER_X11_XINPUT_KEY_PRESS_EVENT]) 
+            {
+              XEvent xevent_converted;
+              XDeviceKeyEvent *xkev = (XDeviceKeyEvent *)xevent;
+              
+              convert_xdevicekey_to_xkey (xkev, &xevent_converted);
+
+              event->key.type = event->type = CLUTTER_KEY_PRESS;
+              translate_key_event (backend, event, &xevent_converted);
+
+              set_user_time (backend_x11, &xwindow, xkev->time);
+            } 
+          else if (xevent->type 
+                   == ev_types [CLUTTER_X11_XINPUT_KEY_RELEASE_EVENT]) 
+            {
+              XEvent xevent_converted;
+              XDeviceKeyEvent *xkev = (XDeviceKeyEvent *)xevent;
+              
+              convert_xdevicekey_to_xkey (xkev, &xevent_converted);
+
+              event->key.type = event->type = CLUTTER_KEY_RELEASE;
+              translate_key_event (backend, event, &xevent_converted);
+            }
+#endif
+          else 
+#endif
+            {
+              CLUTTER_NOTE (EVENT, "Uknown Event");
+              res = FALSE;
+            }
+        }
+    }
+
+ out:  
   return res;
 }
 
@@ -649,6 +836,56 @@ events_queue (ClutterBackend *backend)
     }
 }
 
+/**
+ * clutter_x11_handle_event:
+ * @xevent: pointer to XEvent structure
+ *
+ * This function processes a single X event; it can be used to hook
+ * into external X event retrieval (for example that done by GDK).
+ *
+ * Return value: #ClutterX11FilterReturn indicating what the caller
+ *   should do with the original event.
+ *
+ * Since:  0.8
+ */
+ClutterX11FilterReturn
+clutter_x11_handle_event (XEvent *xevent)
+{
+  ClutterBackend      *backend;
+  ClutterEvent        *event;
+  ClutterMainContext  *clutter_context;
+
+  clutter_threads_enter ();
+
+  clutter_context = clutter_context_get_default ();
+  backend = clutter_context->backend;
+
+  event = clutter_event_new (CLUTTER_NOTHING);
+
+  if (event_translate (backend, event, xevent))
+    {
+      /* push directly here to avoid copy of queue_put */
+      g_queue_push_head (clutter_context->events_queue, event);
+    }
+  else
+    {
+      clutter_event_free (event);
+    }
+
+  event = clutter_event_get ();
+
+  if (event)
+    {
+      /* forward the event into clutter for emission etc. */
+      clutter_do_event (event);
+      clutter_event_free (event);
+    }
+
+  clutter_threads_leave ();
+
+  return CLUTTER_X11_FILTER_CONTINUE;
+}
+
 static gboolean
 clutter_event_prepare (GSource *source,
                        gint    *timeout)
