@@ -22,7 +22,7 @@
 #ifndef __CLUTTER_STAGE_X11_H__
 #define __CLUTTER_STAGE_X11_H__
 
-#include <glib-object.h>
+#include <clutter/clutter-group.h>
 #include <clutter/clutter-stage.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
@@ -43,10 +43,12 @@ typedef struct _ClutterStageX11Class    ClutterStageX11Class;
 
 struct _ClutterStageX11
 {
-  ClutterStage parent_instance;
+  ClutterGroup parent_instance;
 
-  guint        is_foreign_xwin   : 1;
-  guint        fullscreen_on_map : 1;
+  guint        is_foreign_xwin    : 1;
+  guint        fullscreen_on_map  : 1;
+  guint        handling_configure : 1;
+  guint        is_cursor_visible  : 1;
 
   Display     *xdpy;
   Window       xwin_root;
@@ -56,14 +58,22 @@ struct _ClutterStageX11
   gint         xwin_width;
   gint         xwin_height; /* FIXME target_width / height */
   Pixmap       xpixmap;
+  gchar       *title;
 
   ClutterBackendX11 *backend;
   ClutterStageState  state;
+
+#ifdef USE_XINPUT
+  int          event_types[CLUTTER_X11_XINPUT_LAST_EVENT];
+  GList       *devices;
+#endif
+
+  ClutterStage *wrapper;
 };
 
 struct _ClutterStageX11Class
 {
-  ClutterStageClass parent_class;
+  ClutterGroupClass parent_class;
 };
 
 GType clutter_stage_x11_get_type (void) G_GNUC_CONST;
@@ -71,9 +81,10 @@ GType clutter_stage_x11_get_type (void) G_GNUC_CONST;
 /* Private to subclasses */
 void clutter_stage_x11_fix_window_size  (ClutterStageX11 *stage_x11);
 void clutter_stage_x11_set_wm_protocols (ClutterStageX11 *stage_x11);
+void clutter_stage_x11_map              (ClutterStageX11 *stage_x11);
+void clutter_stage_x11_unmap            (ClutterStageX11 *stage_x11);
 
-void clutter_stage_x11_map   (ClutterStageX11 *stage_x11);
-void clutter_stage_x11_unmap (ClutterStageX11 *stage_x11);
+GList *clutter_stage_x11_get_input_devices (ClutterStageX11 *stage_x11);
 
 G_END_DECLS
 
