@@ -6,6 +6,7 @@ typedef struct
 {
   ClutterActor *stage;
   ClutterActor *label;
+  ClutterActor *progress;
 
   ClutterTimeline *timeline;
 
@@ -25,6 +26,7 @@ test_thread_data_new (void)
 static void
 test_thread_data_free (TestThreadData *data)
 {
+  g_object_unref (data->progress);
   g_object_unref (data->label);
   g_object_unref (data->stage);
   g_object_unref (data->timeline);
@@ -59,14 +61,26 @@ static gboolean
 update_label_idle (gpointer data)
 {
   TestUpdate *update = data;
+  guint width;
   gchar *text;
 
   text = g_strdup_printf ("Count to %d", update->count);
 
   clutter_label_set_text (CLUTTER_LABEL (update->thread_data->label), text);
+  clutter_actor_set_width (update->thread_data->label, -1);
+
+  if (update->count == 0)
+    width = 0;
+  else if (update->count == 100)
+    width = 350;
+  else
+    width = (guint) (update->count / 100.0 * 350.0);
+
+  clutter_actor_set_width (update->thread_data->progress, width);
 
   g_free (text);
   g_free (update);
+
   return FALSE;
 }
 
@@ -82,17 +96,24 @@ do_something_very_slow (void)
 
   for (i = 0; i < 100; i++)
     {
-      /* sleep for a while */
-      g_usleep((rand() / (RAND_MAX / 3) + 1) * 1000000);
+      gint msecs;
 
-	  if ((i % 10) == 0)
+      msecs = 1 + (int) (100.0 * rand () / ((RAND_MAX + 1.0) / 3));
+
+      /* sleep for a while */
+      g_usleep (msecs * 1000);
+
+      if ((i % 10) == 0)
         {
           TestUpdate *update;
 
           update = g_new (TestUpdate, 1);
           update->count = i;
           update->thread_data = data;
-          clutter_threads_add_idle (update_label_idle, update);
+
+          clutter_threads_add_idle_full (G_PRIORITY_DEFAULT + 30,
+                                         update_label_idle,
+                                         update, NULL);
         }
     }
 }
@@ -107,13 +128,17 @@ test_thread_func (gpointer user_data)
 
   do_something_very_slow ();
 
-  clutter_threads_add_idle (test_thread_done_idle, data);
+  clutter_threads_add_idle_full (G_PRIORITY_DEFAULT + 30,
+                                 test_thread_done_idle,
+                                 data, NULL);
 
   return NULL;
 }
 
-static ClutterTimeline *timeline = NULL;
-static ClutterActor *count_label = NULL;
+static ClutterTimeline *timeline   = NULL;
+static ClutterActor *count_label   = NULL;
+static ClutterActor *help_label    = NULL;
+static ClutterActor *progress_rect = NULL;
 
 static void
 on_key_press_event (ClutterStage    *stage,
@@ -125,11 +150,14 @@ on_key_press_event (ClutterStage    *stage,
   switch (clutter_key_event_symbol (event))
     {
     case CLUTTER_s:
+      clutter_label_set_text (CLUTTER_LABEL (help_label), "Press 'q' to quit");
+
       clutter_timeline_start (timeline);
       
       data = test_thread_data_new ();
       data->stage = g_object_ref (stage);
       data->label = g_object_ref (count_label);
+      data->progress = g_object_ref (progress_rect);
       data->timeline = g_object_ref (timeline);
       g_thread_create (test_thread_func, data, FALSE, NULL);
       break;
@@ -143,49 +171,60 @@ int
 main (int argc, char *argv[])
 {
   ClutterActor *stage;
-  ClutterActor *label;
   ClutterActor *rect;
   ClutterColor stage_color = { 0xcc, 0xcc, 0xcc, 0xff };
   ClutterColor rect_color = { 0xee, 0x55, 0x55, 0x99 };
-  ClutterBehaviour *behaviour;
+  ClutterColor progress_color = { 0x55, 0xee, 0x55, 0xbb };
+  ClutterBehaviour *r_behaviour, *p_behaviour;
+  const ClutterKnot knots[] = {
+    {  75, 150 },
+    { 400, 150 }
+  };
 
   g_thread_init (NULL);
   clutter_threads_init ();
-  clutter_threads_enter ();
-
   clutter_init (&argc, &argv);
 
   stage = clutter_stage_get_default ();
   clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);
-  clutter_actor_set_size (stage, 800, 600);
+  clutter_actor_set_size (stage, 600, 300);
   
-  count_label = clutter_label_new_with_text ("Mono 16", "Counter");
+  count_label = clutter_label_new_with_text ("Mono 12", "Counter");
   clutter_actor_set_position (count_label, 350, 50);
-  clutter_actor_show (count_label);
 
-  label = clutter_label_new_with_text ("Mono 16", "Press 's' to start");
-  clutter_actor_set_position (label, 50, 50);
-  clutter_actor_show (label);
+  help_label = clutter_label_new_with_text ("Mono 12", "Press 's' to start");
+  clutter_actor_set_position (help_label, 50, 50);
 
   rect = clutter_rectangle_new_with_color (&rect_color);
-  clutter_actor_set_position (rect, 150, 150);
-  clutter_actor_set_size (rect, 25, 25);
-  clutter_actor_show (rect);
+  clutter_actor_set_position (rect, 75, 150);
+  clutter_actor_set_size (rect, 50, 50);
+  clutter_actor_set_anchor_point (rect, 25, 25);
+
+  progress_rect = clutter_rectangle_new_with_color (&progress_color);
+  clutter_actor_set_position (progress_rect, 50, 225);
+  clutter_actor_set_size (progress_rect, 350, 50);
 
   clutter_container_add (CLUTTER_CONTAINER (stage),
-                         count_label, label,
-                         rect,
+                         count_label, help_label,
+                         rect, progress_rect,
                          NULL);
 
   timeline = clutter_timeline_new (150, 50);
   clutter_timeline_set_loop (timeline, TRUE);
-  behaviour = clutter_behaviour_rotate_new (clutter_alpha_new_full (timeline,
+  r_behaviour = clutter_behaviour_rotate_new (clutter_alpha_new_full (timeline,
+                                                                      CLUTTER_ALPHA_RAMP_INC,
+                                                                      NULL, NULL),
+                                              CLUTTER_Z_AXIS,
+                                              CLUTTER_ROTATE_CW,
+                                              0.0, 360.0);
+  clutter_behaviour_apply (r_behaviour, rect);
+
+  p_behaviour = clutter_behaviour_path_new (clutter_alpha_new_full (timeline,
                                                                     CLUTTER_ALPHA_SINE,
                                                                     NULL, NULL),
-                                            CLUTTER_Z_AXIS,
-                                            CLUTTER_ROTATE_CW,
-                                            0.0, 360.0);
-  clutter_behaviour_apply (behaviour, rect);
+                                            knots,
+                                            G_N_ELEMENTS (knots));
+  clutter_behaviour_apply (p_behaviour, rect);
 
   g_signal_connect (stage,
                     "button-press-event", G_CALLBACK (clutter_main_quit),
@@ -196,10 +235,12 @@ main (int argc, char *argv[])
 
   clutter_actor_show (stage);
 
+  clutter_threads_enter ();
   clutter_main ();
   clutter_threads_leave ();
 
-  g_object_unref (behaviour);
+  g_object_unref (p_behaviour);
+  g_object_unref (r_behaviour);
   g_object_unref (timeline);
 
   return EXIT_SUCCESS;
